
/*-------------------------------------------------------------
  module: toesacQX.h
  author: Angel L. Diaz

  date: 10/26/96

  description: 
     Interpolates a factor of an nXn symetric toeplitz determinant 
     utilizing SACLIB's rational field and polynomial arithmetic 

  arguments:
     N       - size of toeplitz matrix
     DegDet  - bound on the degree of the determiant
     DegVar  - bound on the degree of the variables

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbtoeplitz.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbfactors.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxSymToeDet< SaclibQ, SaclibQX >            BBSymToeDet;
typedef BlackBoxFactors< SaclibQ, SaclibQX, BBSymToeDet > BBFactors;
typedef BlackBoxSelector< SaclibQ, BBFactors >            BBFactor;

int main( int argc, char *argv[] ) 
{
   Word stack;
   SaclibInitEnv(3000000,stack);
   int    i,N, DegDet, DegVar,Fac,Seed,IsRestart;
   double Prob;
   char   FileName[BBDEFS_FILEZ];
   char   ProbName[BBDEFS_FILEZ];
   MP_INT MPCard;

   if ( argc < 6 ) {
      cerr << "usage: n deg_det deg_var factor restart" << endl;
      SaclibCleanUpEnv();
      exit( -1 );
   }
   else {
      mpz_init_set_ui( &MPCard, 100000007 ); 
      Prob   = 1.0;
      Seed   = 1;
      N      = atoi( argv[1] );
      DegDet = atoi( argv[2] );
      DegVar = atoi( argv[3] );
      Fac    = atoi( argv[4] );
      IsRestart = atoi( argv[5] ); // BBDEFS_FALSE indicates no restart

      sprintf( ProbName, "%dx%d%s", N, N, "toesacQX" );
      GetCPUTime( ProbName );

      BBSymToeDet T( N, DegDet , Prob );
      BBFactors   FactorsT1( T, Prob, Seed, &MPCard );
      BBFactors   FactorsT2( T );

      sprintf( FileName, "%s.ins", ProbName );
      ofstream OutFile( FileName, ios::out );
      if ( !OutFile ) {
         cerr << "toefacshmap : cannot open " << FileName
              << " for output" << endl;
         exit(-1);
      }

      OutFile << FactorsT1;
      OutFile.close();
      
      ifstream InFile( FileName , ios::in );
      if ( !InFile ) {
         cerr << "toefacshmap : cannot open " << FileName
              << " for input" << endl;
      }

      InFile >> FactorsT2;
      InFile.close();

      BBFactor    FirstFactT( FactorsT2, Fac ); 

      BBSPARSEAUX_VarVector Vars;
      BBSPARSEAUX_DegVector  Degs;
      for( i=1; i<=N; i++ ) Vars.push_back(i);
      for( i=1; i<=N; i++ ) Degs.push_back(DegVar); 
      SaclibQX  SaclibQXElm[BBSPARSEAUX_MAXTERMS];
      BBSPARSEAUX_VecDegVector PruneDegs;
      bbvector< SaclibQ >      PruneMons;

      SparseInterp( FirstFactT, Vars, Degs, DegDet, &MPCard, 
                    PruneDegs, PruneMons, SaclibQXElm, ProbName, IsRestart );

      mpz_clear( &MPCard );
   }

   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

