
/*-------------------------------------------------------------
  module: toesacQXsacZPX.C
  author: Angel L. Diaz

  date: 1/2/98

  description: 
     Interpolates a factor of an nXn symetric toeplitz determinant 
     utilizing SACLIB's rational field and polynomial arithmetic for
     black box construction and SACLIB's modular arithmetic for
     black box evaluation

  arguments:
     N       - size of toeplitz matrix
     DegDet  - bound on the degree of the determiant
     DegVar  - bound on the degree of the variables

  sample make:

     make toesacQXNTLzz_pX B=Linux2.0 L=sac P=10 M=OFF

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "PlugIns/ntl2.0.h"
#include "PlugIns/maps_ntl2.0.h"
#include "BlackBox/CommonObjects/bbtoeplitz.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbfactors.h"
#include "BlackBox/HomomorphicMaps/bbfachmap.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxSymToeDet< SaclibQ, SaclibQX >            BBSymToeDetSacQ;
typedef BlackBoxSymToeDet< NTLzz_p, NTLzz_pX >            BBSymToeDetNTLzz_p;
typedef BlackBoxFactors< SaclibQ, SaclibQX, 
        BBSymToeDetSacQ >                                 BBFactors;
typedef BlackBoxFactorsHMap< SaclibQ, SaclibQX, NTLzz_p, 
           NTLzz_pX, BBSymToeDetSacQ, BBSymToeDetNTLzz_p, 
           SaclibQNTLzz_p >                              BBFactorsHMap;
typedef BlackBoxSelector< NTLzz_p, BBFactorsHMap >       BBFactorNTLzz_p;

int main( int argc, char *argv[] ) 
{
   Word stack;
   //SaclibInitEnv(5000000,stack);
   SaclibInitEnv(1000000,stack);
   int    i,N, DegDet, DegVar,Fac,Seed;
   double Prob;
   char   FileName[100];
   MP_INT MPCard;

   GetCPUTime( "toesacQXsacZPX.C" );

   if ( argc < 4 ) {
      cerr << "usage: n deg_det deg_var factor" << endl;
      SaclibCleanUpEnv();
      exit( -1 );
   }
   else {
      mpz_init_set_ui( &MPCard,  65521 ); 
//      mpz_init_set_ui( &MPCard,  101 );
//      mpz_init_set_ui( &MPCard, 1009 );
//      mpz_init_set_ui( &MPCard, 1073741789 );
      SaclibSetPrime( &MPCard );
      Prob   = 1.0;
      Seed   = 102043;
      N      = atoi( argv[1] );
      DegDet = atoi( argv[2] );
      DegVar = atoi( argv[3] );
      Fac    = atoi( argv[4] );

      BBSymToeDetSacQ  TSacQ( N, DegDet , Prob );
      BBFactors        FactorsTSacQ( TSacQ, Prob, Seed, &MPCard );

      NTLInitEnv( &MPCard );
      BBSymToeDetNTLzz_p TNTLzz_p( N, DegDet , Prob );

      SaclibQNTLzz_p  h;
      BBFactorsHMap    FactorsTNTLzz_p( TNTLzz_p, FactorsTSacQ, h );

      BBFactorNTLzz_p         FirstFactTNTLzz_p( FactorsTNTLzz_p, Fac ); 
      BBSPARSEAUX_VarVector Vars;
      BBSPARSEAUX_DegVector  Degs;
      for( i=1; i<=N; i++ ) Vars.push_back(i);
      for( i=1; i<=N; i++ ) Degs.push_back(DegVar); 
      NTLzz_pX NTLzz_pXElm[1000];

      BBSPARSEAUX_VecDegVector PruneDegs;
      bbvector< NTLzz_p >      PruneMons;
      char   ProbName[BBDEFS_FILEZ];
      sprintf( ProbName, "%dx%d%s", N, N, "toesacQXNTL_zzp" );
      int isRestart = 0;

      SparseInterp( FirstFactTNTLzz_p, Vars, Degs, DegDet, &MPCard,
                    PruneDegs, PruneMons, NTLzz_pXElm, ProbName, isRestart );

      cout << "Answer in Maple input format:" << endl;
      PrintMaple( Vars, PruneDegs, PruneMons );

      mpz_clear( &MPCard );
   }

   GetCPUTime( "toesacQXNTLzz_pX" );

   SaclibCleanUpEnv();
   NTLCleanUpEnv();
}

