
/*-------------------------------------------------------------
  module: cauchysacZPX.C
  author: Angel L. Diaz

  date: 12/5/96

  description: 
     Interpolates the numerator and denominator of a 2x2 Cauchy determinant
     (y1 - y2) (x1 - x2) / (x1 + y1) (x2 + y2) (x1 + y2) (x2 + y1)
     utilizingSACLIB's modular polynomial arithmetic for
     black box construction and evaluation 

  arguments:
     none

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbcauchy.h"
#include "BlackBox/Algorithms/bbnumden.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxCauchyDet< SaclibZP >                        BBCauchySacZP;
typedef BlackBoxNumDen< SaclibZP, SaclibZPX, BBCauchySacZP > BBNumDen;
typedef BlackBoxSelector< SaclibZP, BBNumDen >               BBSelNumDenSacZP;

int main( ) 
{
   Word stack;
   SaclibInitEnv(4000000,stack);
   int    N,Seed,DegDet,DegNum,DegDen,IsRestart;
   double Prob;
   MP_INT MPCard;
   char   ProbName[BBDEFS_FILEZ];

   mpz_init_set_ui( &MPCard, 100000007 ); 
   SaclibSetPrime( &MPCard );
   DegDet    = -1;
   DegNum    = 10;
   DegDen    = 10;
   Prob      = 1.0;
   Seed      = 1;
   N         = 4;
   IsRestart = BBDEFS_FALSE;
   sprintf( ProbName, "%dx%d%s", N/2, N/2, "cauchysacZPX" );
   GetCPUTime( ProbName );

   BBCauchySacZP    CSacZP( N, DegDet, Prob );
   BBNumDen         NumDenCSacZP( CSacZP, &MPCard, DegNum, Prob, Seed );

   // interpolate numerator 
   BBSelNumDenSacZP NumSacZP( NumDenCSacZP, 0 );
   BBSPARSEAUX_VarVector Vars;
   BBSPARSEAUX_DegVector Degs;
   Vars.push_back(1); Vars.push_back(2); Vars.push_back(3); Vars.push_back(4);
   Degs.push_back(1); Degs.push_back(1); Degs.push_back(1); Degs.push_back(1);
   SaclibZPX SaclibZPXElm[BBSPARSEAUX_MAXTERMS];
   BBSPARSEAUX_VecDegVector PruneDegs;
   bbvector< SaclibZP >     PruneMons;

   SparseInterp( NumSacZP, Vars, Degs, DegNum, &MPCard, 
                 PruneDegs, PruneMons, SaclibZPXElm, ProbName, IsRestart );
/*
   // interpolate denominator 
   BBSelNumDenSacZP DenSacZP( NumDenCSacZP, 1 );
   BBSPARSEAUX_VarVector Vars;
   BBSPARSEAUX_DegVector Degs;
   Vars.push_back(1); Vars.push_back(2); Vars.push_back(3); Vars.push_back(4);
   Degs.push_back(2); Degs.push_back(2); Degs.push_back(2); Degs.push_back(2);
   SaclibZPX  SaclibZPXElm[BBSPARSEAUX_MAXTERMS];
   BBSPARSEAUX_VecDegVector PruneDegs;
   bbvector< SaclibZP >     PruneMons;

   SparseInterp( DenSacZP, Vars, Degs, DegDen, &MPCard, 
                 PruneDegs, PruneMons, SaclibZPXElm, ProbName, IsRestart );
*/

   mpz_clear( &MPCard );

   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

