/*-------------------------------------------------------------
  module: faccauchyshoupZPX.C
  author: Angel L. Diaz

  date: 12/5/96

  description: 

  arguments:
     none

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/shoup.h"
#include "BlackBox/CommonObjects/bbcauchy.h"
#include "BlackBox/Algorithms/bbnumden.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Algorithms/bbfactors.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxCauchyDet< ShoupZP >                         BBCauchyShoupZP;
typedef BlackBoxNumDen< ShoupZP, ShoupZPX, BBCauchyShoupZP > BBNumDen;
typedef BlackBoxSelector< ShoupZP, BBNumDen >                BBSelNumDenShoupZP;
typedef BlackBoxFactors< ShoupZP, ShoupZPX, 
        BBSelNumDenShoupZP >                                 BBFactors;
typedef BlackBoxSelector< ShoupZP, BBFactors >               BBSelFactor;

int main( int argc, char *argv[] ) 
{
   int    N,F,Seed,DegDet,DegNum,DegDen,DegFac,i,IsRestart;
   double Prob;
   MP_INT MPCard;
   mpz_init_set_ui( &MPCard, 100000007 ); 
   // mpz_init_set_str( &MPCard, "10000000000000061", 10 );
   char   ProbName[BBDEFS_FILEZ];

   ShoupInitEnv( &MPCard );

   if ( argc < 4 ) {
      cerr << "usage: faccauchysacZPX num_vars factor restart" << endl;
      exit( -1 );
   }

   DegDet    = -1;
   DegNum    = 200000;
   DegDen    = 200000;
   Prob      = 1.0;
   Seed      = 1;
   N         = atoi( argv[1] );
   F         = atoi( argv[2] );
   IsRestart = atoi( argv[3] );
   sprintf( ProbName, "%dx%d%s", N/2, N/2, "faccauchyshoupZPX" );
   GetCPUTime( ProbName );

   BBCauchyShoupZP    CShoupZP( N, DegDet, Prob );
   BBNumDen           NumDenCShoupZP( CShoupZP, &MPCard, DegNum, Prob, Seed );
   BBSelNumDenShoupZP NumShoupZP( NumDenCShoupZP, 0 );

   BBFactors        FactorsNumShoupZP( NumShoupZP, Prob, Seed, &MPCard );
   BBSelFactor      FirstFacZP( FactorsNumShoupZP, F );

   BBSPARSEAUX_VarVector Vars;
   BBSPARSEAUX_DegVector Degs;
   for( i=0; i<N; i++ ) Vars.push_back(i); 
   for( i=0; i<N; i++ ) Degs.push_back(1); 
   ShoupZPX ShoupZPXElm[BBSPARSEAUX_MAXTERMS];
   BBSPARSEAUX_VecDegVector PruneDegs;
   bbvector< ShoupZP >     PruneMons;

   DegFac = 1;
   SparseInterp( FirstFacZP, Vars, Degs, DegFac, &MPCard, 
                 PruneDegs, PruneMons, ShoupZPXElm, ProbName, IsRestart );

   PrintMaple( Vars, PruneDegs, PruneMons);

   mpz_clear( &MPCard );

   ShoupCleanUpEnv();
   GetCPUTime( ProbName );
}

