
/*-------------------------------------------------------------
  module: ratfuncsacQXsacZPX.C
  author: Angel L. Diaz

  date: 12/5/96

  description: 
     Interpolates the numerator of a rational function (x^2+y) 
     utilizing 
        - SACLIB's rational field and polynomial arithmetic for 
          black box construction 
        - SACLIB's modular arithmetic for black box evaluation

  arguments:
     none

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "PlugIns/maps.h"
#include "BlackBox/CommonObjects/bbratfunc.h"
#include "BlackBox/Algorithms/bbnumden.h"
#include "BlackBox/HomomorphicMaps/bbnumdenhmap.h"
#include "BlackBox/Aux/bbtime.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbsparse.h"

typedef BlackBoxURatFunc< SaclibQ >                         BBRatFuncSacQ;
typedef BlackBoxURatFunc< SaclibZP >                        BBRatFuncSacZP;
typedef BlackBoxNumDen< SaclibQ, SaclibQX, BBRatFuncSacQ >  BBNumDen;
typedef BlackBoxNumDenHMap< SaclibQ, SaclibQX, SaclibZP,
        SaclibZPX, BBRatFuncSacQ, BBRatFuncSacZP, 
        SaclibQSaclibZP >                                   BBNumDenHMap;
typedef BlackBoxSelector< SaclibZP, BBNumDenHMap >          BBSelNumDenSacZP;

SaclibQ  RFSacQ( SaclibQ* );
SaclibZP RFSacZP( SaclibZP* );

int main(  ) 
{
   Word stack;
   SaclibInitEnv(1000000,stack);
   int    i,N,Seed,Deg,DegNum,IsRestart;
   double Prob;
   MP_INT MPCard;
   char   ProbName[BBDEFS_FILEZ];

   mpz_init_set_ui( &MPCard, 100000007 ); 
   SaclibSetPrime( &MPCard );
   Deg      = -1;
   DegNum   = 2;
   Prob     = 1.0;
   Seed     = 1;
   N        = 3;
   IsRestart = BBDEFS_FALSE;
   sprintf( ProbName, "%d%s", N, "ratfuncsacQXsacZPX" );
   GetCPUTime( ProbName );

   BBRatFuncSacQ    BBRFSacQ( RFSacQ, N, Deg, Prob );
   BBRatFuncSacZP   BBRFSacZP( RFSacZP, N, Deg, Prob );
   BBNumDen         NumDenRFSacQ( BBRFSacQ, &MPCard, DegNum, Prob, Seed );
   SaclibQSaclibZP  h;
   BBNumDenHMap     NumDenRFSacZP( BBRFSacZP, NumDenRFSacQ, h );
   BBSelNumDenSacZP NumSacZP( NumDenRFSacZP, 0 );
 
   BBSPARSEAUX_VarVector Vars;
   BBSPARSEAUX_DegVector Degs;
   Vars.push_back(1); Vars.push_back(2); Vars.push_back(3);
   Degs.push_back(2); Degs.push_back(2); Degs.push_back(1);
   SaclibZPX SaclibZPXElm[BBSPARSEAUX_MAXTERMS];
   BBSPARSEAUX_VecDegVector PruneDegs;
   bbvector< SaclibZP >     PruneMons;

   SparseInterp( NumSacZP, Vars, Degs, DegNum, &MPCard, 
                 PruneDegs, PruneMons, SaclibZPXElm, ProbName, IsRestart );

   mpz_clear( &MPCard );
   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

SaclibQ RFSacQ(
   SaclibQ* Pnts_i
   ) 
{
   SaclibQ X,Y,Z;
   X = Pnts_i[0];
   Y = Pnts_i[1];
   Z = Pnts_i[2];
   return ( ((X*X) + (Y*Y) + Y +  X + Y*X + Z) / (Y*Y) );
}

SaclibZP RFSacZP(
   SaclibZP* Pnts_i
   ) 
{
   SaclibZP X,Y,Z;
   X = Pnts_i[0];
   Y = Pnts_i[1];
   Z = Pnts_i[2];
   return ( ((X*X) + (Y*Y) + Y + X + Y*X + Z) / (Y*Y) );
}

