/*-------------------------------------------------------------
  module: det.C
  author: Angel L. Diaz

  date: 10/21/96

  Test the black box determinant common object.

-------------------------------------------------------------*/

// arithmetic includes
#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbdet.h"

void Toe4x4Q( SaclibQ*, SaclibQ [BBDET_MAXN][BBDET_MAXM] );
void Toe4x4QX( SaclibQX*, SaclibQX [BBDET_MAXN][BBDET_MAXM] );

int main() {
   Word stack;
   SaclibInitEnv(1000000,stack);

   unsigned int NumVars = 4;
   int          DegPoly = -1;
   double       Prob    = 1;

   BlackBoxDet< SaclibQ > BBToe4x4Q( Toe4x4Q,  NumVars, DegPoly, Prob, 
                                     NumVars, NumVars );

   SaclibQ Pnts[4] = { (long)1, (long)2, (long)3, (long)4 };
   cout << "BBToe4x4Q( Pnts ) = " << *BBToe4x4Q( Pnts ) << endl;

   BlackBoxDetEx< SaclibQ, SaclibQX > BBToe4x4QX( Toe4x4Q,  Toe4x4QX, NumVars, 
      DegPoly, Prob, NumVars, NumVars );

   cout << "BBToe4x4QX( Pnts ) = " << *BBToe4x4QX( Pnts ) << endl;
   SaclibQX Pnts2[4] = { SaclibQX( (long)1, 1), SaclibQX( (long)2, 2),
                         SaclibQX( (long)3, 3), SaclibQX( (long)4, 4) };
   cout << "BBToe4x4QX( Pnts2 ) = " << *BBToe4x4QX( Pnts2 ) << endl;

   SaclibCleanUpEnv();
}

void Toe4x4Q(
   SaclibQ* Pnts_i,
   SaclibQ A_i[BBDET_MAXN][BBDET_MAXM]
   )
{
  int r,c;

  for( r=0; r<4; r++ ) {
     for( c=0; c<4; c++ ) {
        if ( c >= r )
           A_i[r][c] = Pnts_i[3-c+r];
        else
           A_i[r][c] = Pnts_i[3+c-r];
     }
  }
}

void Toe4x4QX(
   SaclibQX* Pnts_i,
   SaclibQX A_i[BBDET_MAXN][BBDET_MAXM]
   )
{
  int r,c;

  for( r=0; r<4; r++ ) {
     for( c=0; c<4; c++ ) {
        if ( c >= r )
           A_i[r][c] = Pnts_i[3-c+r];
        else
           A_i[r][c] = Pnts_i[3+c-r];
     }
  }
}

