/*-------------------------------------------------------------
  module: poly.C
  author: Angel L. Diaz
  date: 8/14/96

  Test the black box polynomial classes and its sub components.
-------------------------------------------------------------*/

// arithmetic includes
#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbpoly.h"

SaclibQ  p1( SaclibQ *); 
SaclibQX p2( SaclibQX *);

int main() {
   Word stack;
   SaclibInitEnv(1000000,stack);

   SaclibQX Poly1( (long)21, 2 ), Poly2( (long)40, 1 ), Poly3;
   Poly3 = Poly1 + Poly2 + (long) 11;
   SaclibQ Elm( (long) 100 );
   cout << "poly  : Poly3 = " << Poly3 << endl;

   unsigned int NumVars = 1;
   unsigned int Deg = 2;
   unsigned int Prob = 1;
   BlackBoxPoly< SaclibQ, SaclibQX> BBPoly1( Poly3, NumVars, Deg, Prob );
   cout << "poly  : BBPoly1.Deg() = " << BBPoly1.Deg() << endl;
   cout << "poly  : BBPoly1.NumVars() = " << BBPoly1.NumVars() << endl;
   cout << "poly  : BBPoly1.Prob() = " << BBPoly1.Prob() << endl;
   cout << "poly  : BBPoly1( &Elm ) = " << *BBPoly1( &Elm ) << endl;

   BlackBoxUPoly< SaclibQ > BBUPoly1( p1, NumVars, Deg, Prob );
   cout << "poly  : BBUPoly1( &Elm ) = " << *BBUPoly1( &Elm ) << endl;

   BlackBoxUPolyEx< SaclibQ, SaclibQX > BBUPoly2( p1, p2, NumVars, Deg, Prob );
   cout << "poly  : BBUPoly2( &Elm ) = " << *BBUPoly2( &Elm ) << endl;
   SaclibQX Elm2( (long)1, 1 );
   cout << "poly  : BBUPoly2( &Elm2 ) = " << *BBUPoly2( &Elm2 ) << endl;
 
   SaclibCleanUpEnv();
}

// p1 := 21*x^2 + 40*x + 1 
SaclibQ p1(
   SaclibQ *pnts
   ) 
{
   SaclibQ X = pnts[0];
   return( X * X * (long) 21 + X * (long) 40 + (long) 1 );
}

SaclibQX p2(
   SaclibQX *pnts
   )
{
   SaclibQX X = pnts[0];
   return( X * X * (long) 21 + X * (long) 40 + (long) 1 );
}

