
/*-------------------------------------------------------------
  module: bbnetwork.h
  author: Angel L. Diaz

  date: 11/11/96

  description: 

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#define BBNET_MAXMESSZ  256
#define BBNET_TCP_PORT  6001
#define BBNET_HOST_ADDR "127.0.0.1"

#define BBNET_ERROR   -1
#define BBNET_SUCCESS 0

#define BBNET_ZP             0
#define BBNET_Q              1
#define BBNET_TOE            10
#define BBNET_VAND           20
#define BBNET_VANDSHIFT      30
#define BBNET_FAC            100
#define BBNET_FAC_EVAL       110
#define BBNET_FACS           120
#define BBNET_FACS_EVAL      130
#define BBNET_FACS_HMAP      150
#define BBNET_FACS_HMAP_EVAL 160
#define BBNET_FAC_HMAP_EVAL  170
#define BBNET_GCD            200
#define BBNET_GCD_EVAL       210
#define BBNET_SPARSE         300
#define BBNET_SEL            400
#define BBNET_SETPRIME       1000
#define BBNET_DELETE         1100
#define BBNET_KILL           1200

int ServerSocket( );
int Accept( int );
int Socket( );
int ReadLine( int, char* ptr, int );
int WriteLine( int, char *, int );
void ErrorExit( const char*, int ); 
void OutputError( const char*, int ); 
void CheckReadStatus( int );
void Reverse( char [] );
void itoa( int, char [] );

int ServerSocket()
{

   int                sock, s;
   struct sockaddr_in serv_addr;

   if ( ( sock = socket( AF_INET, SOCK_STREAM, 0 ) ) < 0 )
      ErrorExit( "bbserver : ServerSocket : can not open a stream", errno );

   bzero( (char *) &serv_addr, sizeof( serv_addr ) );
   serv_addr.sin_family      = AF_INET;
   serv_addr.sin_addr.s_addr = htonl( INADDR_ANY );
   serv_addr.sin_port        = htons( BBNET_TCP_PORT ); 
 
   if ( bind( sock, (struct sockaddr * ) &serv_addr, 
        sizeof( serv_addr ) ) < 0 ) 
      ErrorExit( "bbserver : ServerSocket : can not bind to local address", 
                 errno ); 
 
   listen( sock, 5 ); 

   return ( sock );
}

int Accept( int sock_i )
{
   int                newsock, cl_len;
   struct sockaddr_in cl_addr;

   cl_len = sizeof( cl_addr );
   newsock   = accept( sock_i, ( struct sockaddr * ) &cl_addr, &cl_len );
      
   return ( newsock );
}

int Socket()
{
   int                sock;
   struct sockaddr_in serv_addr;

   bzero( (char *) &serv_addr, sizeof(serv_addr) );
   serv_addr.sin_family      = AF_INET;

   serv_addr.sin_port        = htons( BBNET_TCP_PORT ); 

   if( ( sock = socket( AF_INET, SOCK_STREAM, 0 ) ) < 0 )
      ErrorExit( "client : Socket : can not open stream", errno );

   if( connect( sock, (struct sockaddr *) &serv_addr, 
        sizeof( serv_addr ) ) < 0 )
      ErrorExit( "client : Socket : can not connect to server", errno ); 

   return( sock );
}

/* read a line, including newline */
int ReadLine( int sock, char* ptr, int maxlen )
{
   int  n;  /*  > 0 number of character read ; < 0 error ; 0 empty file */
   int  rc;
   char c;

   for( n= 1; n< maxlen; n++ ) {
      /* read a character */
      if ( (rc = read( sock, &c, 1) ) == 1) {
         *ptr++ = c;
         /* reached end of line? */
         if ( c == '\n' )
            break;
      } 
      else
         /* end of file */
         if ( rc == 0 ) {
            if ( n == 1 )
               return( 0 ); 
            else
               break;
         }
         /* error condition */
         else {
            OutputError( "ReadLine : problem with read", errno );
            return( -1 );
         }
   } 
   *ptr = 0;
   return( n ); 
}

/* write a line, including newline */
int WriteLine( int sock, char *ptr, int nbytes )
{
   int nleft, nwritten;
   nleft = nbytes;

   while ( nleft > 0 ) {
       nwritten = write( sock, ptr, nleft );
       /* < 0 error, 0 nothing written */
       if ( nwritten <= 0 )
           return( nwritten );
       nleft -= nwritten;
       ptr   += nwritten;
   }
   return ( nbytes - nleft );  /* number of bytes written */
}

void OutputError( const char* message, int errno ) 
{
   perror( message );
}

void ErrorExit( const char* message, int errno ) 
{
   OutputError( message, errno );
   exit( errno );
}

void CheckReadStatus(
   int n 
   )
{
   if ( n == 0 ) 
      ErrorExit( "bbserver : connection terminated", errno );
   else
      if ( n < 0 )
         ErrorExit( "bbserver : ReadLine error", errno );
}

void Reverse(
   char s[]
   )
{
   int c, i, j;

   for( i=0, j=strlen(s)-1; i<j; i++, j-- ) {
      c    = s[i];
      s[i] = s[j];
      s[j] = c;
   }
}

void itoa(
   int n,
   char s[]
   )
{
   int i, sign;

   if ( (sign = n) < 0 )
      n = -n;
   i = 0;
   do {
      s[i++] = n % 10 + '0';
   } while ((n /= 10) > 0);
   if ( sign < 0 )
      s[i++] = '-';
   s[i] = '\0';
   Reverse(s);
}

