
/*-------------------------------------------------------------
  program: bbserver.C
  author: Angel L. Diaz

  date: 11/11/96

  description: 

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "bbserver.h"

void HandleSetPrime( int );
void HandleVand( int, BlackBoxHouse& );
void HandleVandShift( int, BlackBoxHouse& );
void HandleGCD( int, BlackBoxHouse& );
void HandleGCDEval( int, BlackBoxHouse& );
void HandleToe( int, BlackBoxHouse& );
void HandleFac( int, BlackBoxHouse& );
void HandleFacEval( int, BlackBoxHouse& );
void HandleFacs( int, BlackBoxHouse& );
void HandleFacsEval( int, BlackBoxHouse& );
void HandleSel( int, BlackBoxHouse& );
void HandleFacsHMap( int, BlackBoxHouse& );
void HandleFacsHMapEval( int, BlackBoxHouse& );
void HandleFacHMapEval( int, BlackBoxHouse& );
void HandleSparseConversion( int, BlackBoxHouse&, BBSPARSEAUX_VecDegVector&, 
   SaclibZPX*, bbvector< SaclibZP >&, SaclibQX*, bbvector< SaclibQ >& );

int main( int argc, char *argv[] ) 
{
   Word stack;
   SaclibInitEnv(2000000,stack);

   int                      sock, newsock, n;
   int                      Request;
   char                     mess[BBNET_MAXMESSZ];
   BlackBoxHouse            BBHouse;
   BBSPARSEAUX_VecDegVector PruneDegs;
   SaclibZPX                SaclibZPXElm[BBSPARSEAUX_MAXTERMS];
   bbvector< SaclibZP >     PruneMonsZP;
   SaclibQX                 SaclibQXElm[BBSPARSEAUX_MAXTERMS];
   bbvector< SaclibQ >      PruneMonsQ;

#ifdef BBDEFS_SERVER_TRC
BBSERVERTrace1( );
#endif

   // wait for and establish a new connection
   sock    = ServerSocket( ); 
   newsock = Accept( sock );
   if ( newsock < 0 )
      ErrorExit( "bbserver : accept error", errno );
 
   // in deamon mode we will run a long time 
   for( ; ; ) {

#ifdef BBDEFS_SERVER_TRC
BBSERVERTrace3( );
#endif

      // read request
      n = ReadLine( newsock, mess, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      Request = atoi( mess );

#ifdef BBDEFS_SERVER_TRC
BBSERVERTrace2( Request );
#endif

      switch( Request ) {

         case BBNET_TOE:
            HandleToe( newsock, BBHouse );
         break;

         case BBNET_VAND:
            HandleVand( newsock, BBHouse );
         break;

         case BBNET_VANDSHIFT:
            HandleVandShift( newsock, BBHouse );
         break;

         case BBNET_FAC:
            HandleFac( newsock, BBHouse );
         break;

         case BBNET_FAC_EVAL:
            HandleFacEval( newsock, BBHouse );
         break;

         case BBNET_FACS:
            HandleFacs( newsock, BBHouse );
         break;

         case BBNET_FACS_EVAL:
            HandleFacsEval( newsock, BBHouse );
         break;

         case BBNET_SEL:
            HandleSel( newsock, BBHouse );
         break;

         case BBNET_FACS_HMAP:
            HandleFacsHMap( newsock, BBHouse );
         break;

         case BBNET_FACS_HMAP_EVAL:
            HandleFacsHMapEval( newsock, BBHouse );
         break;

         case BBNET_FAC_HMAP_EVAL:
            HandleFacHMapEval( newsock, BBHouse );
         break;

         case BBNET_GCD:
            HandleGCD( newsock, BBHouse );
         break;

         case BBNET_GCD_EVAL:
            HandleGCDEval( newsock, BBHouse );
         break;

         case BBNET_SPARSE:
            HandleSparseConversion( newsock, BBHouse, PruneDegs, SaclibZPXElm, 
               PruneMonsZP, SaclibQXElm, PruneMonsQ );
         break;

         case BBNET_SETPRIME:
            HandleSetPrime( newsock );
         break;

         case BBNET_DELETE:
         break;

         case BBNET_KILL:
            goto XDONE; 
         break;

      }
   }

   XDONE:
   close(sock);
   close(newsock);
   SaclibCleanUpEnv();
}

/*-------------------------------------------------------------
  function: HandleSetPrime
  author: Angel L. Diaz

  date: 11/15/96

  description: 

  Format: 
     prime               (ASCII string) 
-------------------------------------------------------------*/

void HandleSetPrime( 
   int            newsock_i
   )
{
   int n;
   MP_INT MPPrime;
   char   mess[BBNET_MAXMESSZ];

#ifdef BBDEFS_SERVER_TRC
BBSERVERSetPrimeTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   n = strlen( mess ); mess[n] = '\0';

#ifdef BBDEFS_SERVER_TRC
BBSERVERSetPrimeTrace2( mess );
#endif

   mpz_init_set_str( &MPPrime, mess, 10 );
   SaclibSetPrime( &MPPrime );

   mpz_clear( &MPPrime );
}

/*-------------------------------------------------------------
  function: HandleVand
  author: Angel L. Diaz

  date: 11/14/96

  description: 

  Format: 
     field               (int)
     number of variables (int)
     degree              (int)
     probability         (double)     
-------------------------------------------------------------*/

void HandleVand( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int n, N, DegDet, Field, Index;
   double Prob;
   char mess[BBNET_MAXMESSZ];

#ifdef BBDEFS_SERVER_TRC
BBSERVERVandTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // number of variables
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   N = atoi( mess );

   // degreee
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   DegDet = atoi( mess );

   // probability 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Prob = atof( mess );

#ifdef BBDEFS_SERVER_TRC
BBSERVERVandTrace2( Field, N, DegDet, Prob );
#endif
  
   Index = BBHouse_i.ConsVandDet( Field, N, DegDet, Prob );
   itoa( Index, mess ); 

   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );
}

/*-------------------------------------------------------------
  function: HandleVandShift
  author: Angel L. Diaz

  date: 11/14/96

  description: 

  Format: 
     field               (int)
     number of variables (int)
     degree              (int)
     probability         (double)     
-------------------------------------------------------------*/

void HandleVandShift( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int n, N, DegDet, Field, Index;
   double Prob;
   char mess[BBNET_MAXMESSZ];

#ifdef BBDEFS_SERVER_TRC
BBSERVERVandSTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // number of variables
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   N = atoi( mess );

   // degreee
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   DegDet = atoi( mess );

   // probability 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Prob = atof( mess );

#ifdef BBDEFS_SERVER_TRC
BBSERVERVandSTrace2( Field, N, DegDet, Prob );
#endif
  
   Index = BBHouse_i.ConsVandDetShift( Field, N, DegDet, Prob );
   itoa( Index, mess ); 

   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );
}

/*-------------------------------------------------------------
  function: HandleGCD
  author: Angel L. Diaz

  date: 11/15/96

  description: 

  Format: 
     number of BB's 1..r (int)
     index of BB's 1..r  (int) 
     field               (int)
     card                (ASCII string)
     probability         (double)     
     seed                (int)
-------------------------------------------------------------*/

void HandleGCD( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int i, n, r, Seed, Field, Index;
   double Prob;
   char mess[BBNET_MAXMESSZ];
   int  IndexArray[BBSERVER_MAXOBJ*BBSERVER_NUMOBJ];
   MP_INT MPCard; 

#ifdef BBDEFS_SERVER_TRC
BBSERVERGCDTrace1( );
#endif

   // number of BB's
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   r = atoi( mess );

   // index of each BB
   for( i=0; i<r; i++ ) {
      n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      IndexArray[i] = atoi( mess );
   }

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // card
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   mpz_init_set_str( &MPCard, mess, 10 );

   // probability 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Prob = atof( mess );

   // seed
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Seed = atoi( mess );

#ifdef BBDEFS_SERVER_TRC
BBSERVERGCDTrace2( r, IndexArray, Field, &MPCard, Prob, Seed );
#endif
  
   Index = BBHouse_i.ConsGCD( r, IndexArray, Field, &MPCard, Prob, Seed );
   itoa( Index, mess ); 

   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   mpz_clear( &MPCard );
}

/*-------------------------------------------------------------
  function: HandleGCDEval
  author: Angel L. Diaz

  date: 11/15/96

  description: 

  Format: 
     field               (int)
     number of Pnts      (int)
     Points              (ASCII string)
-------------------------------------------------------------*/

void HandleGCDEval( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int i, n, NumPnts, Field, Index, Status;
   char mess[BBNET_MAXMESSZ];
   SaclibZP SacZPPnts[BBSERVER_MAXVARS];
   SaclibZP SacZPValue;
   SaclibQ  SacQPnts[BBSERVER_MAXVARS];
   SaclibQ  SacQValue;

#ifdef BBDEFS_SERVER_TRC
BBSERVERGCDEvalTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Index = atoi( mess );

   // number of BB's
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumPnts = atoi( mess );

   // each point
   if ( Field == BBNET_ZP ) {
      // index of each BB
      for( i=0; i<NumPnts; i++ ) {
         n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
         CheckReadStatus( n ); 
         SacZPPnts[i] = SaclibZP( mess );
      }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERGCDEvalTrace2( Index, Field, NumPnts, SacZPPnts );
#endif

      SacZPValue = BBHouse_i.EvalZP( Index, NumPnts, SacZPPnts, Status );
      SacZPValue.toString( mess );
   }
   else {
      // index of each BB
      for( i=0; i<NumPnts; i++ ) {
         n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
         CheckReadStatus( n ); 
         SacQPnts[i] = SaclibQ( mess );
      }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERGCDEvalTrace2( Index, Field, NumPnts, SacQPnts );
#endif
      
      SacQValue = BBHouse_i.EvalQ( Index, NumPnts, SacQPnts, Status );
      SacQValue.toString( mess );
   }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERGCDEvalTrace3( mess );
#endif

   // value
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

}

/*-------------------------------------------------------------
  function: HandleToe
  author: Angel L. Diaz

  date: 11/15/96

  description: 

  Format: 
     field               (int)
     number of variables (int)
     degree              (int)
     probability         (double)     
-------------------------------------------------------------*/

void HandleToe( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int n, N, DegDet, Field, Index;
   double Prob;
   char mess[BBNET_MAXMESSZ];

#ifdef BBDEFS_SERVER_TRC
BBSERVERToeTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // number of variables
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   N = atoi( mess );

   // degreee
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   DegDet = atoi( mess );

   // probability 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Prob = atof( mess );

#ifdef BBDEFS_SERVER_TRC
BBSERVERToeTrace2( Field, N, DegDet, Prob );
#endif
  
   Index = BBHouse_i.ConsSymToeDet( Field, N, DegDet, Prob );
   itoa( Index, mess ); 

   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );
}

/*-------------------------------------------------------------
  function: HandleFac
  author: Angel L. Diaz

  date: 11/16/96

  description: 

  Format: 
     index               (int) 
     selected factor     (int)
     field               (int)
     card                (ASCII string)
     probability         (double)     
     seed                (int)
-------------------------------------------------------------*/

void HandleFac( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int i, n, BoxIndex, FacIndex, Seed, Field, Index;
   double Prob;
   char mess[BBNET_MAXMESSZ];
   MP_INT MPCard; 

#ifdef BBDEFS_SERVER_TRC
BBSERVERFacTrace1( );
#endif

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   BoxIndex = atoi( mess );

   // factor
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   FacIndex = atoi( mess );

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // card
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   mpz_init_set_str( &MPCard, mess, 10 );

   // probability 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Prob = atof( mess );

   // seed
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Seed = atoi( mess );

#ifdef BBDEFS_SERVER_TRC
BBSERVERFacTrace2( BoxIndex, FacIndex, Field, &MPCard, Prob, Seed );
#endif
  
   Index = BBHouse_i.ConsFac( BoxIndex, FacIndex, Field, &MPCard, Prob, Seed );
   itoa( Index, mess ); 

   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   mpz_clear( &MPCard );
}

/*-------------------------------------------------------------
  function: HandleFacEval
  author: Angel L. Diaz

  date: 11/16/96

  description: 

  Format: 
     field               (int)
     index               (int)
     number of Pnts      (int)
     Points              (ASCII string)
-------------------------------------------------------------*/

void HandleFacEval( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   MP_INT   MPZero;
   int      i, n, NumPnts, Field, Index, Status;
   char     mess[BBNET_MAXMESSZ];
   SaclibZP SacZPPnts[BBSERVER_MAXVARS];
   SaclibZP SacZPValue;
   SaclibQ  SacQPnts[BBSERVER_MAXVARS];
   SaclibQ  SacQValue;
   mpz_init_set_si( &MPZero, 0 );
 
   SacZPValue = SaclibZP( &MPZero );
   SacQValue  = SaclibQ( &MPZero );

#ifdef BBDEFS_SERVER_TRC
BBSERVERFacEvalTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Index = atoi( mess );

   // number of points
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumPnts = atoi( mess );

   if ( Field == BBNET_ZP ) {
      // each point
      for( i=0; i<NumPnts; i++ ) {
         n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
         CheckReadStatus( n ); 
         SacZPPnts[i] = SaclibZP( mess );
      }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacEvalTrace2( Index, Field, NumPnts, SacZPPnts );
#endif

      SacZPValue = BBHouse_i.EvalZP( Index, NumPnts, SacZPPnts, Status );
      SacZPValue.toString( mess );
   }
   else {
      // each point
      for( i=0; i<NumPnts; i++ ) {
         n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
         CheckReadStatus( n ); 
         SacQPnts[i] = SaclibQ( mess );
      }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacEvalTrace2( Index, Field, NumPnts, SacQPnts );
#endif
      
      SacQValue = BBHouse_i.EvalQ( Index, NumPnts, SacQPnts, Status );
      SacQValue.toString( mess );
   }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacEvalTrace3( mess );
#endif

   // value
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   mpz_clear( &MPZero );
}

/*-------------------------------------------------------------
  function: HandleFacs
  author: Angel L. Diaz

  date: 11/20/96

  description: 

  Format: 
     index               (int) 
     field               (int)
     card                (ASCII string)
     probability         (double)     
     seed                (int)
-------------------------------------------------------------*/

void HandleFacs( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int i, n, BoxIndex, FacIndex, Seed, Field, Index;
   double Prob;
   char mess[BBNET_MAXMESSZ];
   MP_INT MPCard; 

#ifdef BBDEFS_SERVER_TRC
BBSERVERFacsTrace1( );
#endif

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   BoxIndex = atoi( mess );

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // card
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   mpz_init_set_str( &MPCard, mess, 10 );

   // probability 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Prob = atof( mess );

   // seed
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Seed = atoi( mess );

#ifdef BBDEFS_SERVER_TRC
BBSERVERFacsTrace2( BoxIndex, Field, &MPCard, Prob, Seed );
#endif
  
   Index = BBHouse_i.ConsFacs( BoxIndex, Field, &MPCard, Prob, Seed );
   itoa( Index, mess ); 

   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   mpz_clear( &MPCard );
}

/*-------------------------------------------------------------
  function: HandleFacsEval
  author: Angel L. Diaz

  date: 11/20/96

  description: 

  Format: 
     field               (int)
     index               (int)
     number of Pnts      (int)
     points              (ASCII string)
-------------------------------------------------------------*/

void HandleFacsEval( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int       i, n, NumPnts, NumFacs, Field, Index, Status;
   char      mess[BBNET_MAXMESSZ];
   SaclibZP  SacZPPnts[BBSERVER_MAXVARS];
   SaclibZP* SacZPValue;
   SaclibQ   SacQPnts[BBSERVER_MAXVARS];
   SaclibQ*  SacQValue;
 
#ifdef BBDEFS_SERVER_TRC
BBSERVERFacsEvalTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Index = atoi( mess );

   // number of points
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumPnts = atoi( mess );

   if ( Field == BBNET_ZP ) {
      // MUST ADD SUPPORT FOR ZP
   }
   else {
      // each point
      for( i=0; i<NumPnts; i++ ) {
         n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
         CheckReadStatus( n ); 
         SacQPnts[i] = SaclibQ( mess );
      }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacsEvalTrace2( Index, Field, NumPnts, SacQPnts );
#endif
      
      SacQValue = BBHouse_i.FacsEvalQ( Index, NumPnts, SacQPnts, NumFacs,
                                       Status );

   // number of factors
   itoa( NumFacs, mess );
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacsEvalTrace4( NumFacs );
#endif

   // values
   for( i=0; i<NumFacs; i++ ) {
      SacQValue[i].toString( mess );

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacsEvalTrace3( mess );
#endif

      n = strlen(mess);
      mess[n] = '\n'; n++;
      if ( WriteLine( newsock_i, mess, n )  != n )
         ErrorExit( "Client : WriteLine error", errno );
   }

   }

}

/*-------------------------------------------------------------
  function: HandleSel
  author: Angel L. Diaz

  date: 11/20/96

  description: 

  Format: 
-------------------------------------------------------------*/

void HandleSel( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int       i, n, Field, BoxIndex, SelIndex, Index, Status;
   char      mess[BBNET_MAXMESSZ];
 
   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   BoxIndex = atoi( mess );

   // selected value
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   SelIndex = atoi( mess );

   Index = BBHouse_i.ConsSel( Field, BoxIndex, SelIndex );

#ifdef BBDEFS_SERVER_TRC
   BBSERVERSelTrace1( Field, BoxIndex, SelIndex, Index );
#endif

   itoa( Index, mess ); 
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

}

/*-------------------------------------------------------------
  function: HandleFacsHMap
  author: Angel L. Diaz

  date: 11/26/96

  description: 

  Format: 
     index of factor box over Q     (int)
     index of origional box over ZP (int)
-------------------------------------------------------------*/

void HandleFacsHMap( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int IndexQ,IndexZP, MapedIndex, n;
   char      mess[BBNET_MAXMESSZ];

   // index of factor box over Q 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   IndexQ = atoi( mess );

   // index of origional box over ZP
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   IndexZP = atoi( mess );

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacsHMapTrace1( IndexQ, IndexZP );
#endif

   MapedIndex = BBHouse_i.ConsFacsHMap( IndexQ, IndexZP );
   itoa( MapedIndex, mess ); 
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

}

/*-------------------------------------------------------------
  function: HandleFacsHMapEval
  author: Angel L. Diaz

  date: 11/20/96

  description: 

  Format: 
     index               (int)
     number of Pnts      (int)
     points              (ASCII string)
-------------------------------------------------------------*/

void HandleFacsHMapEval( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   int       i, n, NumPnts, NumFacs, Index, Status;
   char      mess[BBNET_MAXMESSZ];
   SaclibZP  SacZPPnts[BBSERVER_MAXVARS];
   SaclibZP* SacZPValue;
 
#ifdef BBDEFS_SERVER_TRC
BBSERVERFacsHMapEvalTrace1( );
#endif

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Index = atoi( mess );

   // number of points
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumPnts = atoi( mess );

   // each point
   for( i=0; i<NumPnts; i++ ) {
      n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      SacZPPnts[i] = SaclibZP( mess );
   }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacsHMapEvalTrace2( Index, NumPnts, SacZPPnts );
#endif
      
   SacZPValue = BBHouse_i.FacsHMapEval( Index, NumPnts, SacZPPnts, NumFacs,
                                        Status );

   // number of factors
   itoa( NumFacs, mess );
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacsHMapEvalTrace4( NumFacs );
#endif

   // values
   for( i=0; i<NumFacs; i++ ) {
      SacZPValue[i].toString( mess );

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacsHMapEvalTrace3( mess );
#endif

      n = strlen(mess);
      mess[n] = '\n'; n++;
      if ( WriteLine( newsock_i, mess, n )  != n )
         ErrorExit( "Client : WriteLine error", errno );
   }

}

/*-------------------------------------------------------------
  function: HandleFacHMapEval
  author: Angel L. Diaz

  date: 11/26/96

  description: 

  Format: 
     index               (int)
     number of Pnts      (int)
     points              (ASCII string)
-------------------------------------------------------------*/

void HandleFacHMapEval( 
   int            newsock_i,
   BlackBoxHouse& BBHouse_i 
   )
{
   MP_INT   MPZero;
   int      i, n, NumPnts, Index, Status;
   char     mess[BBNET_MAXMESSZ];
   SaclibZP SacZPPnts[BBSERVER_MAXVARS];
   SaclibZP SacZPValue;
   mpz_init_set_si( &MPZero, 0 );
 
   SacZPValue = SaclibZP( &MPZero );

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Index = atoi( mess );

   // number of points
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumPnts = atoi( mess );

   // each point
   for( i=0; i<NumPnts; i++ ) {
      n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      SacZPPnts[i] = SaclibZP( mess );
   }

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacHMapEvalTrace1( Index, NumPnts, SacZPPnts );
#endif

   SacZPValue = BBHouse_i.EvalZP( Index, NumPnts, SacZPPnts, Status );
   SacZPValue.toString( mess );

#ifdef BBDEFS_SERVER_TRC
   BBSERVERFacHMapEvalTrace2( mess );
#endif

   // value
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   mpz_clear( &MPZero );
}

/*-------------------------------------------------------------
  function: HandleSparseConversion
  author: Angel L. Diaz

  date: 11/25/96

  description: 

  Format: 
     field                          (int)
     index                          (int)
     number of variables            (int)
     degree bound for each variable (int)
     bound on total degree          (int)
     card of set to choose rnd nums (int)
-------------------------------------------------------------*/

void HandleSparseConversion( 
   int                       newsock_i,
   BlackBoxHouse&            BBHouse_i,
   BBSPARSEAUX_VecDegVector& PruneDegs_i,
   SaclibZPX*                SaclibZPXElm_i,
   bbvector< SaclibZP >&     PruneMonsZP_i,
   SaclibQX*                 SaclibQXElm_i,
   bbvector< SaclibQ >&      PruneMonsQ_i
   )
{
   int i, j, n, Field, Index, NumVars,VarDeg,TotalDeg,Status; 
   BBSPARSEAUX_VarVector Vars;
   BBSPARSEAUX_DegVector Degs;
   char      mess[BBNET_MAXMESSZ];
   MP_INT MPCard;

#ifdef BBDEFS_SERVER_TRC
BBSERVERSCTrace1( );
#endif

   // field 
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Field = atoi( mess );

   // index
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   Index = atoi( mess );

   // number of varibales
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumVars = atoi( mess );
   for( i=1; i<=NumVars; i++ ) Vars.push_back(i);

   // degree bound for each variable
   for( i=1; i<=NumVars; i++ ) {
         n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
         CheckReadStatus( n ); 
         VarDeg = atoi( mess );
         Degs.push_back( VarDeg ); 
   }

   // bound on total degree
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   TotalDeg = atoi( mess );

   // card
   n = ReadLine( newsock_i, mess, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   mpz_init_set_str( &MPCard, mess, 10 );

#ifdef BBDEFS_SERVER_TRC
BBSERVERSCTrace2( Field, Index, NumVars, Vars, Degs, TotalDeg, &MPCard );
#endif
  
   if ( Field == BBNET_ZP ) {
      BBHouse_i.SparseConversionZP( Field, Index, NumVars, Vars, Degs, 
       TotalDeg, &MPCard, SaclibZPXElm_i, PruneDegs_i, PruneMonsZP_i, Status );

   // number of monomials
   itoa( PruneMonsZP_i.size(), mess );
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // each monomial/degree pair
   for ( i=0; i<PruneMonsZP_i.size(); i++ ) {

      // ith monomial
      PruneMonsZP_i[i].toString( mess );
      n = strlen(mess);
      mess[n] = '\n'; n++;
      if ( WriteLine( newsock_i, mess, n )  != n )
         ErrorExit( "Client : WriteLine error", errno );

      // number of degs (exclude the total degree of each monomial)
      itoa( (PruneDegs_i[i]).size()-1, mess );
      n = strlen(mess);
      mess[n] = '\n'; n++;
      if ( WriteLine( newsock_i, mess, n )  != n )
         ErrorExit( "Client : WriteLine error", errno );

      // each degree
      for( j=1; j<(PruneDegs_i[i]).size(); j++ ) {
         itoa( (PruneDegs_i[i])[j] , mess );
         n = strlen(mess);
         mess[n] = '\n'; n++;
         if ( WriteLine( newsock_i, mess, n )  != n )
            ErrorExit( "Client : WriteLine error", errno );
      } 
   }
   }
   else {
      BBHouse_i.SparseConversionQ( Field, Index, NumVars, Vars, Degs, 
       TotalDeg, &MPCard, SaclibQXElm_i, PruneDegs_i, PruneMonsQ_i, Status );

   // number of monomials
   itoa( PruneMonsQ_i.size(), mess );
   n = strlen(mess);
   mess[n] = '\n'; n++;
   if ( WriteLine( newsock_i, mess, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // each monomial/degree pair
   for ( i=0; i<PruneMonsQ_i.size(); i++ ) {

      // ith monomial
      PruneMonsQ_i[i].toString( mess );
      n = strlen(mess);
      mess[n] = '\n'; n++;
      if ( WriteLine( newsock_i, mess, n )  != n )
         ErrorExit( "Client : WriteLine error", errno );

      // number of degs (exclude the total degree of each monomial)
      itoa( (PruneDegs_i[i]).size()-1, mess );
      n = strlen(mess);
      mess[n] = '\n'; n++;
      if ( WriteLine( newsock_i, mess, n )  != n )
         ErrorExit( "Client : WriteLine error", errno );

      // each degree
      for( j=1; j<(PruneDegs_i[i]).size(); j++ ) {
         itoa( (PruneDegs_i[i])[j] , mess );
         n = strlen(mess);
         mess[n] = '\n'; n++;
         if ( WriteLine( newsock_i, mess, n )  != n )
            ErrorExit( "Client : WriteLine error", errno );
      } 
   }
   }

   // clean up
   while ( PruneDegs_i.size() > 0 ) PruneDegs_i.pop_back();
   while ( PruneMonsQ_i.size() > 0 ) PruneMonsQ_i.pop_back();
   while ( PruneMonsZP_i.size() > 0 ) PruneMonsZP_i.pop_back();

   mpz_clear( &MPCard );
}

