#-----------------------------------------------------------------------
# module: bridge.mpl
#
# author: Angel Diaz
# date: 11/20/96
#
# (c) Copyright 1996 Angel Diaz and Eirch Kaltofen, All Rights Reserved 
# This code is provided WITHOUT WARRANTY either expressed or implied.
#-----------------------------------------------------------------------

readlib(write):    # open(f), writeln(r), close()

BBNET_COMFILE   := 'BBNET_COMFILE':
BBNET_ANSFILE   := 'BBNET_ANSFILE':
BBNET_BPIDFILE  := 'BBNET_BPIDFILE':
BBNET_CPIDFILE  := 'BBNET_CPIDFILE':

BBNET_ZP             := 0:
BBNET_Q              := 1:
BBNET_TOE            := 10:
BBNET_VAND           := 20:
BBNET_VANDSHIFT      := 30:
BBNET_FAC            := 100:
BBNET_FAC_EVAL       := 110:
BBNET_FACS           := 120:
BBNET_FACS_EVAL      := 130:
BBNET_FACS_HMAP      := 150:
BBNET_FACS_HMAP_EVAL := 160:
BBNET_FAC_HMAP_EVAL  := 170:
BBNET_GCD            := 200:
BBNET_GCD_EVAL       := 210:
BBNET_SPARSE         := 300:
BBNET_SEL            := 400:
BBNET_SETPRIME       := 1000:
BBNET_DELETE         := 1100:
BBNET_KILL           := 1200:

FoxBoxInitEnv := proc(
   BPIDFile_i,
   CPIDFile_i,
   ComFile_i,
   AnsFile_i
   )
   local Status;
   global BBNET_COMFILE, BBNET_ANSFILE, BBNET_BPIDFILE, BBNET_CPIDFILE;

   BBNET_COMFILE   := ComFile_i;
   BBNET_ANSFILE   := AnsFile_i;
   BBNET_BPIDFILE  := BPIDFile_i;
   BBNET_CPIDFILE  := CPIDFile_i;

   Status := system(`client `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE.` `.
                    BBNET_COMFILE.` `.BBNET_ANSFILE.` &`);
   RETURN( Status );
end:

FoxBoxCleanUp := proc()
   local Status;
   open(BBNET_COMFILE);
   writeln(BBNET_KILL);
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   RETURN( Status );
end:

FoxBoxSetPrime := proc(
   Modulus_i
   )
   local Status;
   open(BBNET_COMFILE);
   writeln(BBNET_SETPRIME);
   writeln(Modulus_i);
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   RETURN( Status );
end:

BlackBoxVand :=proc(
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   VandType_i,    # either BBNET_VAND or BBNET_VANDSHIFT
   NumVars_i,
   Degree_i,
   Probability_i
   )
   local Index, Line, Status;
   open(BBNET_COMFILE);
   writeln(VandType_i);
   writeln(FieldType_i);
   writeln(NumVars_i);
   writeln(Degree_i);
   writeln(Probability_i);
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   Line   := readline(BBNET_ANSFILE);
   Index  := parse(Line);
   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;
   RETURN( Index );
end:

BlackBoxGCD :=proc(
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   IndexList_i,
   Card_i,
   Probability_i,
   Seed_i
   )
   local Index, Line, Status, n, i;
   n := nops(IndexList_i);

   open(BBNET_COMFILE);
   writeln(BBNET_GCD);
   writeln(n);
   for i from 1 to n do
      writeln(IndexList_i[i]);
   od;
   writeln(FieldType_i);
   writeln(Card_i);
   writeln(Probability_i);
   writeln(Seed_i);
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   Line   := readline(BBNET_ANSFILE);
   Index  := parse(Line);
   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;
   RETURN( Index );
end:

BlackBoxSymToe := proc( 
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   NumVars_i,     
   Degree_i,
   Probability_i
   )
   local Index, Line, Status; 
   open(BBNET_COMFILE);
   writeln(BBNET_TOE);
   writeln(FieldType_i);
   writeln(NumVars_i);
   writeln(Degree_i);
   writeln(Probability_i); 
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE); 
   Line   := readline(BBNET_ANSFILE);
   Index  := parse(Line);
   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;
   RETURN( Index );
end:

BlackBoxFactor := proc(
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   Index_i,
   FactorID_i,
   Card_i,
   Probability_i,
   Seed_i
   )
   local Index, Line, Status;
   open(BBNET_COMFILE);
   writeln(BBNET_FAC);
   writeln(Index_i);
   writeln(FactorID_i);
   writeln(FieldType_i);
   writeln(Card_i);
   writeln(Probability_i);
   writeln(Seed_i);
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   Line   := readline(BBNET_ANSFILE);
   Index  := parse(Line);
   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;
   RETURN( Index );
end:

BlackBoxFactors := proc(
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   Index_i,
   Card_i,
   Probability_i,
   Seed_i
   )
   local Index, Line, Status;
   open(BBNET_COMFILE);
   writeln(BBNET_FACS);
   writeln(Index_i);
   writeln(FieldType_i);
   writeln(Card_i);
   writeln(Probability_i);
   writeln(Seed_i);
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   Line   := readline(BBNET_ANSFILE);
   Index  := parse(Line);
   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;
   RETURN( Index );
end:

BlackBoxHomomorphicMap := proc(
   Type_i,
   TargetIndex_i,
   ModBoxIndex_i
   )
   local Type,Line,Status,Index;
   Index := -1;

   if ( Type_i = BBNET_FACS ) then
      Type := BBNET_FACS_HMAP;
   else
      RETURN( Index );
   fi;

   open(BBNET_COMFILE);
   writeln(Type);
   writeln(TargetIndex_i);
   writeln(ModBoxIndex_i);
   close(BBNET_COMFILE);

   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   Line   := readline(BBNET_ANSFILE);
   Index  := parse(Line);
   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;
   RETURN( Index );
 
end:
 
BlackBoxSelectValue := proc(
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   Index_i,
   SelVal_i
   )
   open(BBNET_COMFILE);
   writeln(BBNET_SEL);
   writeln(FieldType_i);
   writeln(Index_i);
   writeln(SelVal_i);
   close(BBNET_COMFILE);

   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   Line   := readline(BBNET_ANSFILE);
   Index  := parse(Line);
   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;
   RETURN( Index );

end:

BlackBoxEval := proc(
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   Type_i,
   Index_i,
   Pnts_i
   )
   local NumValues, Value, Line, Status, n,i;
   n := nops(Pnts_i);
   open(BBNET_COMFILE);
   writeln(Type_i);
   if ( (Type_i <> BBNET_FACS_HMAP_EVAL) and 
        (Type_i <> BBNET_FAC_HMAP_EVAL )     ) then
      writeln(FieldType_i);
   fi;
   writeln(Index_i);
   writeln(n);
   for i from 1 to n do
      writeln(Pnts_i[i]);
   od;
   close(BBNET_COMFILE);
   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);
   if ( Type_i = BBNET_FACS_EVAL or Type_i = BBNET_FACS_HMAP_EVAL ) then
      Line      := readline(BBNET_ANSFILE);
      NumValues := parse(Line);
      Value     := [];
      for i from 1 to NumValues do 
         Line  := readline(BBNET_ANSFILE);
         Value := [ op(Value), parse(Line) ];
      od;
      while Line <> 0 do
         Line := readline(BBNET_ANSFILE); # get EOF
      od;
   else
      Line  := readline(BBNET_ANSFILE);
      Value := parse(Line);
      while Line <> 0 do
         Line := readline(BBNET_ANSFILE); # get EOF
      od;
   fi;
   RETURN( Value );
end:

SparseConversion := proc(
   FieldType_i,   # either BBNET_Q or BBNET_ZP
   Index_i,
   Vars_i,
   DegBnds_i,
   TotalDeg,
   Card_i
   )
   local i,j,Line,NumVars,NumMon,NumDegs,Coeff,VarVec,Deg,Poly,Status;

   NumVars := nops(Vars_i);
   Poly    := 0;

   open(BBNET_COMFILE);
   writeln(BBNET_SPARSE);
   writeln(FieldType_i);
   writeln(Index_i);
   writeln(NumVars);
   for i from 1 to NumVars do
      writeln(DegBnds_i[i]);
   od;
   writeln(TotalDeg);
   writeln(Card_i);
   close(BBNET_COMFILE);

   Status := system(`bridge `.BBNET_BPIDFILE.` `.BBNET_CPIDFILE);

   Line   := readline(BBNET_ANSFILE);
   NumMon := parse(Line);

   for i from 1 to NumMon do
      # coeff
      Line   := readline(BBNET_ANSFILE);
      Coeff  := parse(Line);

      # degree vector
      Line    := readline(BBNET_ANSFILE);
      NumDegs := parse(Line);
      VarVec  := 1;
      for j from 1 to NumDegs do
         Line    := readline(BBNET_ANSFILE);
         Deg     := parse(Line);
         VarVec  := VarVec * Vars_i[j]^Deg; 
      od;

      Poly := Poly + Coeff * VarVec;

   od;

   while Line <> 0 do
      Line := readline(BBNET_ANSFILE); # get EOF
   od;

   RETURN( Poly );
end: 

