
/*-------------------------------------------------------------
  module: bbgcdaux.h
  author: Angel L. Diaz

  date: 10/24/96

  description: 
     The class in this module provides auxiliary functionality 
     utilized by the black box GCD algorithm. 

  class:
     MapFtoGCDFBar

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#ifndef _BBGCDAUX
#define _BBGCDAUX

#define BBGCD_MAXVAR   100
#define BBGCD_MAXBB    100
#define BBGCD_MAXPOLYS 10
#define BBGCD_MAXDELTA 475

#include "BlackBox/CommonObjects/blackbox.h"
#include "BlackBox/Aux/bbdense.h"
#include "BlackBox/Algorithms/bbgcdtrace.h"

/*-------------------------------------------------------------
  class: MapFtoGCDFBar
  date: 8/25/96

  reference:

  description: 

  assumptions: 
     The input black box polynomial is in at least two variables.
-------------------------------------------------------------*/

template < class K, class P, class B > 
class MapFtoGCDFBar : public BlackBox< K > {
   private:
      B* BB;                // black box polynomial
      K* RandA;             // elements for multivariate polynomial projection
      K* RandB;             // elements for multivariate polynomial projection
      K ConstPnts[BBGCD_MAXVAR]; // pn - an*p1 - bn
      K Pnts[BBGCD_MAXVAR]; 
   public:
      MapFtoGCDFBar( ) { }
      ~MapFtoGCDFBar( ) { }
      inline void SetF( B*, K*, K*, K*, unsigned int );
      inline K* operator () ( K* );
};

template < class K, class P, class B > 
inline void MapFtoGCDFBar< K, P, B >::SetF( 
   B* BB_i, 
   K* RandA_i, 
   K* RandB_i, 
   K* Pnts_i, 
   unsigned int NumPnts_i
   )
{
   unsigned int i;

   BB    = BB_i;
   RandA = RandA_i;
   RandB = RandB_i;
   NumVariables  = NumPnts_i;

   // pn - an*p1 - bn
   for( i=0; i<NumVariables-1; i++ ) 
      ConstPnts[i] = Pnts_i[i+1] - RandA[i]*Pnts_i[0] - RandB[i];
}

template < class K, class P, class B > 
inline K* MapFtoGCDFBar< K, P, B >::operator () ( 
   K* Pnts_i
   )
{ 
   unsigned int i;
   K X;
   K Y;
   X = Pnts_i[0];
   Y = Pnts_i[1];

   Pnts[0] = X;
   // Y*(pn - an*p1 - bn) + an*X + bn
   for( i=1; i<NumVariables; i++ ) 
      Pnts[i] = Y*ConstPnts[i-1] + RandA[i-1]*X + RandB[i-1];  

   return ( (*BB)( Pnts ) );
}

#endif

