/*-------------------------------------------------------------
  module: bbgcdtrace.h

  author: Angel L. Diaz
  date: 9/1/96

  desciption: Tracing routines for: bbgcd.h, 
     PRINTLEVEL = 0 ; tracing off
     PRINTLEVEL > 0 ; function tags
     PRINTLEVEL > 1 ; milestone messages
     PRINTLEVEL > 2 ; state variables
     PRINTLEVEL > 3 ; state variables and arrays/vectors
-------------------------------------------------------------*/

#ifndef _BBGCDTRACE
#define _BBGCDTRACE

#ifdef BBDEFS_BBGCD_TRC

#include "BlackBox/Aux/bbtrace.h"

template< class K > void BBGCDTrace1(
   double Prob_i,
   unsigned int Seed_i,
   unsigned int NumVariables_i,
   K& ProbeElm_i,
   K& GenElm_i,
   int StaticDeg_i,
   K* RandA_i,
   K* RandB_i,
   int NumAB_i,
   K* RandC_i,
   int NumC_i 
   )
{
   if ( PRINTLEVEL > 0 ) {
   cout << "bbgcd : Constructing black box GCD" << endl;
   }
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbgcd : ","Prob = ",Prob_i); 
   OutputSingleItem("bbgcd : ","Seed = ",Seed_i);
   OutputSingleItem("bbgcd : ","NumVariables = ",NumVariables_i);
   OutputSingleItem("bbgcd : ","ProbeElm = ",ProbeElm_i);
   OutputSingleItem("bbgcd : ","GenElm = ",GenElm_i);
   OutputSingleItem("bbgcd : ","StaticDeg = ",StaticDeg_i);
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("bbgcd : ","Random elements a2, ... , an",
               "RandA[",RandA_i,0,NumAB_i);
   OutputArray("bbgcd : ","Random elements b2, ... , bn",
               "RandB[",RandB_i,0,NumAB_i);
   OutputArray("bbgcd : ","Random elements c3, ... , cr",
               "RandC[",RandC_i,0,NumC_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Computing F1Bar( X, 0 )" << endl;
   }
}
 
template< class K, class P > void BBGCDTrace2(
   P& F1Bar_i,
   K& GenElm_i,
   int StaticDeg_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputSingleItem("bbgcd : ","F1Bar = ",F1Bar_i);
   OutputSingleItem("bbgcd : ","GenElm = ",GenElm_i);
   OutputSingleItem("bbgcd : ","StaticDeg = ",StaticDeg_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Computing F0Bar( X, 0 )" << endl;
   }
}

template< class K, class P > void BBGCDTrace3(
   P& F0Bar_i,
   K& GenElm_i,
   int StaticDeg_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputSingleItem("bbgcd : ","F0Bar = ",F0Bar_i);
   OutputSingleItem("bbgcd : ","GenElm = ",GenElm_i);
   OutputSingleItem("bbgcd : ","StaticDeg = ",StaticDeg_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Computing gcd( F0Bar(X,0), F1Bar(X,0) )" << endl;
   }
}

template< class K, class P > void BBGCDTrace4(
   P& G0_i,
   K& LG0_i,
   unsigned int Delta_i 
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbgcd : ","G0 = ",G0_i);
   OutputSingleItem("bbgcd : ","LG0 = ",LG0_i);
   OutputSingleItem("bbgcd : ","Delta = ",Delta_i);
   }
   if ( PRINTLEVEL > 0 ) {
   cout << "bbgcd : Done!" << endl;
   }
}

template< class K > void BBGCDTrace5(
   K* Pnts_i,
   unsigned int NumPnts_i,
   unsigned int Delta_i
   )
{
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Evaluating black box GCD" << endl;
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("bbgcd : ","Evaluation points p1, ... , pn",
               "Pnts_i[",Pnts_i,0,NumPnts_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Computing " << Delta_i << " valid GCD images" << endl;
   cout << "bbgcd : Computing  F1Bar" << endl;
   }
}

template< class K, class P > void BBGCDTrace6(
   P& F1Bar_i,
   unsigned int F1BarDeg_i,
   K& GenElm_i,
   int StaticDeg_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputSingleItem("bbgcd : ","F1Bar = ",F1Bar_i);
   OutputSingleItem("bbgcd : ","F1Bar.dg() = ",F1BarDeg_i);
   OutputSingleItem("bbgcd : ","GenElm = ",GenElm_i);
   OutputSingleItem("bbgcd : ","StaticDeg = ",StaticDeg_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Computing  F0Bar" << endl;
   }
}

template< class K, class P > void BBGCDTrace7(
   P& F0Bar_i,
   unsigned int F0BarDeg_i,
   K& GenElm_i,
   int StaticDeg_i,
   int NumValidGCDs_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputSingleItem("bbgcd : ","F0Bar = ",F0Bar_i);
   OutputSingleItem("bbgcd : ","F0Bar.dg() = ",F0BarDeg_i);
   OutputSingleItem("bbgcd : ","GenElm = ",GenElm_i);
   OutputSingleItem("bbgcd : ","StaticDeg = ",StaticDeg_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Computing gcd( F0Bar(X," << NumValidGCDs_i
        << "), F1Bar(X," << NumValidGCDs_i << ") )" << endl;
   }
}

template< class P > void BBGCDTrace8(
   P& GCD_i,
   unsigned int GCDDeg_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbgcd : ","GCD = ",GCD_i);
   OutputSingleItem("bbgcd : ","GCD.dg() = ",GCDDeg_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Checking for early termination" << endl;
   }
}
 
void BBGCDTrace9(
   )
{
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Early termination!" << endl;
   }
}

void BBGCDTrace10(
   )
{
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Valid GCD" << endl;
   }
}

void BBGCDTrace11(
   )
{
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Discarding point" << endl;
   }
}

template< class T > void BBGCDTrace12(
   T& ValidGCDImages_i,
   unsigned int NumValidGCDs_i
   )
{
   unsigned int i;
   if ( PRINTLEVEL > 3 ) {
   for( i=0; i<NumValidGCDs_i; i++ ) {
      cout << "bbgcd : GCD[" << i <<"] = " << ValidGCDImages_i[i].GCD 
           << endl;
      cout << "bbgcd : Y = "  << ValidGCDImages_i[i].Y << endl;
   } 
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbgcd : Interpolating coefficients" << endl;
   }
}

template< class K > void BBGCDTrace13(
   K& GCDValue_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbgcd : ","GCDValue = ",GCDValue_i);
   }
   if ( PRINTLEVEL > 0 ) {
   cout << "bbgcd : Done!" << endl;
   }
}

#endif

#endif
