/*-------------------------------------------------------------
  module: bbsparsetrace.h

  author: Angel L. Diaz
  date: 9/1/96

  desciption: Tracing routines for: bbgcd.h, 
     PRINTLEVEL = 0 ; tracing off
     PRINTLEVEL > 0 ; function tags
     PRINTLEVEL > 1 ; milestone messages
     PRINTLEVEL > 2 ; state variables
     PRINTLEVEL > 3 ; state variables and arrays/vectors
-------------------------------------------------------------*/

#ifndef _BBSPARSETRACE
#define _BBSPARSETRACE

#ifdef BBDEFS_BBSPARSE_TRC

#include "BlackBox/Aux/bbtrace.h"

template< class K > void BBSPARSETrace1( 
   BBSPARSEAUX_DegVector& VarVector_i,
   unsigned int NumVars_i,
   BBSPARSEAUX_DegVector& DegVector_i,
   unsigned int d0_i,
   K* A_i,
   K* InputPnts_i
   ) 
{
   if ( PRINTLEVEL > 1 ) {
   OutputVec("SparseInterp : ", "Variable numbers", "VarVector[", 
               VarVector_i, 0, NumVars_i );
   OutputVec("SparseInterp : ", "Variable degrees", "DegVector_i[", 
               DegVector_i, 0, NumVars_i - 1 );
   OutputSingleItem("SparseInterp : ", 
                    "Bound on all of the degs of each var (d0) = ", d0_i );
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("SparseInterp : ", "Initial random points", "A[", 
               A_i, 1, NumVars_i );
   OutputArray("SparseInterp : ", "Initial input points", "InputPnts[", 
               InputPnts_i, 1, NumVars_i );
   }
}

template< class T > void BBSPARSETrace2(
   T& VarNum_i,
   T& T_i,
   bbvector< unsigned int >& Ell_i,
   T& ji_i,
   BBSPARSEAUX_VecDegVector& Ji_i
   )
{
   if ( PRINTLEVEL > 1 ) {
   OutputSingleItem("SparseInterp : ", "Working on variable number = ", 
                    VarNum_i );
   OutputSingleItem("SparseInterp : ", "Number of terms (T) = ", T_i );
   }
   if ( PRINTLEVEL > 2 ) {
   OutputVec("SparseInterp : ", "Remaining degree of each monomial", 
               "Ell[", Ell_i, 0, Ell_i.size() );
   OutputSingleItem("SparseInterp : ", "Size of linear system (ji x ji) = ", 
                    ji_i);
   if ( Ji_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   else
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "SparseInterp : Pruning ... " << endl;
   }
}

template< class K > void BBSPARSETrace3(
   bbvector< unsigned int >& Ell_i,
   bbvector< K >& PruneMons_i,
   BBSPARSEAUX_VecDegVector& PruneDegs_i, 
   BBSPARSEAUX_VecDegVector& Ji_i,
   bbvector< K >& X_i,
   K* P_i,
   unsigned int i_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputVec("SparseInterp : ", "Remaining degree of each monomial", 
               "Ell[", Ell_i, 0, Ell_i.size() );
   OutputVec("SparseInterp : ", "Pruned Monomials","PruneMons[", 
               PruneMons_i, 0, PruneMons_i.size() );
   if ( PruneDegs_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Corressponding exponent vectors",
                   "PruneDegs[",PruneDegs_i, PruneDegs_i.size() );
   else
      OutputVecVec("SparseInterp : ", "Corressponding exponent vectors",
                   "PruneDegs[",PruneDegs_i, PruneDegs_i.size() );
   if ( Ji_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   else
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   OutputVec("SparseInterp : ", "Solution vector", "X[", X_i, 0, 
               X_i.size() );
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("SparseInterp : ", "Random evaluation points", "P[", P_i, 
               0, i_i+1 );
   } 
}

template< class K > void BBSPARSETrace4(
   bbvector< K >& V_i,
   bbvector< K >& gki_i, 
   bbvector< K >& X_i,
   BBSPARSEAUX_VecDegVector& Ji_i,
   BBSPARSEAUX_VecDegVector& nJi_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputVec("SparseInterp : ", "Vandermonde system", "V[", V_i, 0, 
               V_i.size() );
   OutputVec("SparseInterp : ", "Adjusted black box evaluation (gki) ", 
               "gki[", gki_i, 0, gki_i.size() );
   }
   if ( PRINTLEVEL > 2 ) {
   OutputVec("SparseInterp : ", "Solution vector", "X[", X_i, 0, 
               X_i.size() );
   if ( Ji_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   else
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   if ( nJi_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (nJi) = ", 
                   "nJi[", nJi_i, nJi_i.size() );
   else
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (nJi) = ", 
                   "nJi[", nJi_i, nJi_i.size() );
   }
}
template< class T > void BBSPARSETrace5(
   T& Ji_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   if ( Ji_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   else
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (Ji) = ", 
                   "Ji[", Ji_i, Ji_i.size() );
   }
}

template< class K > void BBSPARSETrace6(
   bbvector< K >& X_i,
   BBSPARSEAUX_VecDegVector& nJi_i,
   bbvector< K >& PruneMons_i,
   BBSPARSEAUX_VecDegVector& PruneDegs_i, 
   unsigned int BBCnt_i
   )
{
   if ( PRINTLEVEL > 0 ) {
   cout << "SparseInterp : Number of Black Box Calls (BBCnt) = " 
        << BBCnt_i << endl; 
   OutputVec("SparseInterp : ", "Pruned Monomials","PruneMons[", PruneMons_i, 
               0, PruneMons_i.size() );
   if ( PruneDegs_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Corressponding exponent vectors",
                   "PruneDegs[",PruneDegs_i, PruneDegs_i.size());
   else
      OutputVecVec("SparseInterp : ","Corressponding exponent vectors",
                   "PruneDegs[",PruneDegs_i, PruneDegs_i.size() );
   OutputVec("SparseInterp : ", "Solution vector", "X[", X_i, 0, 
               X_i.size() );
   if ( nJi_i.size() == 0 )
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (nJi) = ", 
                   "nJi[", nJi_i, nJi_i.size());
   else
      OutputVecVec("SparseInterp : ", "Vector of exponent vectors (nJi) = ", 
                   "nJi[", nJi_i, nJi_i.size());
   }
}

void BBSPARSETrace7(
   int BBCnt_i
   )
{
   if ( PRINTLEVEL > 1 ) {
   OutputSingleItem("SparseInterp : ", "Black Box evaluation number = ", 
       BBCnt_i );
   }
}
#endif

#endif
