
/*-------------------------------------------------------------
  module: bbfactortrace.h
  author: Angel L. Diaz

  date: 10/23/96

  desciption: 
     Tracing routines for: bbfactors.h, bbfactoraux.h
 
     PRINTLEVEL = 0 ; tracing off
     PRINTLEVEL > 0 ; function tags
     PRINTLEVEL > 1 ; milestone messages
     PRINTLEVEL > 2 ; state variables
     PRINTLEVEL > 3 ; state variables and arrays/vectors

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.

-------------------------------------------------------------*/

#ifndef _BBFACTORTRACE
#define _BBFACTORTRACE

#ifdef BBDEFS_BBFACTOR_TRC

#include "BlackBox/Aux/bbtrace.h"

/*-------------------------------------------------------------

 bbfactors.h

-------------------------------------------------------------*/

template < class K >
void BBFACSTrace1(
   double Prob_i,
   int    Seed_i,
   int    NumVaribales_i,
   int    StaticDeg_i,
   K&     ProbeElm_i,
   K&     GenElm_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("BlackBoxFactors : Construction : ","Prob = ",Prob_i);
   OutputSingleItem("BlackBoxFactors : Construction : ","Seed = ",Seed_i);
   OutputSingleItem("BlackBoxFactors : Construction : ","NumVariables = ",
                    NumVaribales_i);
   OutputSingleItem("BlackBoxFactors : Construction : ","StaticDeg = ",
                    StaticDeg_i);
   OutputSingleItem("BlackBoxFactors : Construction : ","ProbeElm = ",
                    ProbeElm_i);
   OutputSingleItem("BlackBoxFactors : Construction : ","GenElm = ",GenElm_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Construction : Generating random elements" 
        << endl;
   }
}

template < class K >
void BBFACSTrace2(
   K*  RandA_i,
   K*  RandB_i,
   K*  RandC_i,
   int NumVariables_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactors : Construction : ", 
               "Random elemnts a2, ... , an", "RandA[", RandA_i, 0, 
               NumVariables_i-1 );
   OutputArray("BlackBoxFactors : Construction : ",
               "Random elemnts b1, ... , bn", "RandB[", RandB_i, 0,
               NumVariables_i );
   OutputArray("BlackBoxFactors : Construction : ",
               "Random elemnts c1,c3 ... , cn", "RandC[", RandC_i, 0,
               NumVariables_i-1 );
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Construction : Interpolating F1(0,Y)"
        << endl;
   }
}

template < class K, class P >
void BBFACSTrace3(
   P&            F1_i,
   K&            Cont_i,
   P*            FactorImages_i,
   unsigned int* Exps_i, 
   int           Sign_i,
   int           NumFactors_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("BlackBoxFactors : Construction : ","F1 = ",F1_i);
   OutputSingleItem("BlackBoxFactors : Construction : ","Cont = ",Cont_i);
   OutputSingleItem("BlackBoxFactors : Construction : ","Sign = ",Sign_i);
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactors : Construction : ", "Factor Images", 
               "FactorImages[", FactorImages_i, 0, NumFactors_i );
   OutputArray("BlackBoxFactors : Construction : ", "Exponents", 
               "Exps[", Exps_i, 0, NumFactors_i );
   }
}
   
template < class P >
void BBFACSTrace4(
   int SumExps_i,
   P*  FactorProducts_i,
   P&  pgsem1_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("BlackBoxFactors : Construction : ","SumExps = ",
                    SumExps_i);
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactors : Construction : ", "Factor Products", 
               "FactorProducts[", FactorProducts_i, 0, SumExps_i );
   }
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("BlackBoxFactors : Construction : ","pgsem1 = ",
                    pgsem1_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Construction : Computing partial fractions"
        << endl;
   }
}

template < class P >
void BBFACSTrace5(
   P ParFrac_i[BBFACTOR_MAXDEG][BBFACTOR_MAXFACTORS],
   int ParFracR_i,
   int ParFracC_i
   )
{
   int i,j;
   if ( PRINTLEVEL > 3 ) {
   for( i=0; i<=ParFracC_i; i++ )
      for( j=0; j<=ParFracR_i; j++ )
         cout << "BlackBoxFactors : Construction : ParFrac[" << i << "][" 
              << j << "] = " <<  ParFrac_i[i][j] << endl;
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Construction : Done!" << endl;
   }
}

template < class P >
void BBFACSTrace6(
   P*  Pnts_i,
   int NumberVariables_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactors : Evaluation : ", "Input Points", 
               "Pnts_i[", Pnts_i, 0, NumberVariables_i );
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Evaluation : Interpolating FBar(X,Y)" << endl;
   }
}

template < class P >
void BBFACSTrace7(
   BiPoly< P >& FBar_i,
   int          Deg_i
   ) 
{
   int i;
   if ( PRINTLEVEL > 3 ) {
   for(i=0; i<=Deg_i; i++)
      cout << "BlackBoxFactors : Evaluation : FBar[" << i << "] = " 
           << FBar_i[i] << endl;
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Evaluation : Lifting Factors" << endl;
   }
}

template < class P >
void BBFACSTrace8(
   BiPoly< P >* BiFactors_i,
   int          NumFactors_i
   )
{
   int i;
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactors : Evaluation : ", "Bivariate Factors", 
               "BiFactors[", BiFactors_i, 0, NumFactors_i );
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Evaluation : Implicit interpolation" << endl;
   }
}

template < class K >
void BBFACSTrace9(
   K*  ValueOfFactors_i,
   int NumFactors_i
   )
{
   int i;
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactors : Evaluation : ", "Value of the Factors",
               "ValueOfFactors[", ValueOfFactors_i, 0, NumFactors_i );
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactors : Evaluation : Done!" << endl;
   }
}

/*-------------------------------------------------------------

 bbfactoraux.h

-------------------------------------------------------------*/

void BBFACLIFTTrace1(
   unsigned int* FactorDegs_i,
   unsigned int NumFactors_i,
   unsigned int MaxDeg_i,
   unsigned int SumFactorDegs_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputArray("Hensel Lift : ", "Factor Degrees", "FactorDegs[", FactorDegs_i, 
               0, NumFactors_i);
   OutputSingleItem("Hensel Lift : ","MaxDeg = ", MaxDeg_i);
   OutputSingleItem("Hensel Lift : ","SumFactorDegs = ", SumFactorDegs_i);
   }
}

void BBFACLIFTTrace2(
   unsigned int k_i
   )
{
   if ( PRINTLEVEL > 1 ) {
   OutputSingleItem("Hensel Lift : ","Lifting to Y^", k_i);
   }
}

template < class P > void BBFACLIFTTrace3(
   BiPoly< P >* gp_i, 
   unsigned int SumExps_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputArray("Hensel Lift : ", "Incremental product of factors", 
               "gp[", gp_i, 0, SumExps_i );
   }
}

template < class P > void BBFACLIFTTrace4(
   BiPoly< P >* ngp_i,
   unsigned int SumExps_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputArray("Hensel Lift : ", "Incremental product of factors",
               "ngp[", ngp_i, 0, SumExps_i );
   }
}

template < class P > void BBFACLIFTTrace5(
   P& TmpCoeff_i,
   P& ttk_i,
   P& uk_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("Hensel Lift : ","TmpCoeff = ", TmpCoeff_i );
   OutputSingleItem("Hensel Lift : ","ttk = ", ttk_i);
   OutputSingleItem("Hensel Lift : ","uk = ", uk_i);
   }
}

template < class P > void BBFACLIFTTrace6(
   unsigned int m_i,
   unsigned int i_i,
   P& ParFrac_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   Output2DArrayElm("Hensel Lift : ","ParFrac[",m_i,i_i,ParFrac_i);
   }
}

template < class P > void BBFACLIFTTrace7(
   P& gik_i,
   BiPoly< P >& gikm1_i,
   BiPoly< P >& g_i, 
   unsigned int i_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputArrayElm("Hensel Lift : ","gik[",i_i, gik_i);
   OutputArrayElm("Hensel Lift : ","gikm1[",i_i,gikm1_i);
   OutputArrayElm("Hensel Lift : ","g[",i_i,g_i);
   }
}

#endif

#endif
