/*-------------------------------------------------------------
  module: bbdense.h
  author: Angel L. Diaz
  date: 8/22/96
-------------------------------------------------------------*/

#ifndef _BBDENSE
#define _BBDENSE

#define BBDENSE_MAXCOEFFS 400
#define BBDENSE_MAXDEG    400

/*
// for Cachy and Vandermonde
#define BBDENSE_MAXCOEFFS 600
#define BBDENSE_MAXDEG    600
*/

#define BBDENSE_XVALUE 0
#define BBDENSE_YVALUE 1

#include "BlackBox/Aux/bbdefs.h"
#include "BlackBox/Aux/bbbipoly.h"
#include "BlackBox/Aux/bbtrace.h"

/*-------------------------------------------------------------
  function: UniDenseInterp
  date: 8/22/96

  reference:

  description:

-------------------------------------------------------------*/

template< class K, class P > void UniDenseInterp(
   K* X_i,            // array of independent values X_i[1], ... ,X_i[Deg_i+1]
   K* Y_i,            // array of dependent values   Y_i[1], ... ,Y_i[Deg_i+1]
   int Deg_i,
   int& StaticDeg_m,  // -1 indicates a refresh
   K* Reciprocal_m,
   P* Basis_m,
   P& Poly_o
   )
{
   int k,i;
   MP_INT MPOne;
   mpz_init_set_ui( &MPOne, 1 ); 
   P  X( (MP_INT*)&MPOne, 1 );
   K V[BBDENSE_MAXCOEFFS]; 
   K Prod,TempCoeff;

#ifdef BBDEFS_DENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : computing inverses" << endl;
   }
#endif

  // compute the required inverses
  for( k=1; k<=Deg_i; k++) {
     // only if necessary
     if ( StaticDeg_m < k ) {
        Prod = X_i[k] - X_i[0];
        for( i=1; i<=k-1; i++) 
           Prod = Prod*( X_i[k] - X_i[i]);
        Reciprocal_m[k] = K( (MP_INT*)&MPOne ) / Prod;
     }
  }

#ifdef BBDEFS_DENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : computing basis" << endl;
   }
#endif

  // compute the basis
  Basis_m[0] = P( (MP_INT*)&MPOne, 0);
  for( k=1; k<=Deg_i; k++)  {
     // only if necessary
     if ( StaticDeg_m < k ) {
        Basis_m[k] = ( X - X_i[k-1] );
        Basis_m[k] = Basis_m[k] * Basis_m[k-1];
     }
  }

#ifdef BBDEFS_DENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : computing newton coefficients" << endl;
   }
#endif

  // compute the newton coefficients
  V[0] = Y_i[0];
  for( k=1; k<=Deg_i; k++) {
     TempCoeff = V[k-1];
     for( i=k-2; i>=0; i=i-1 )  
        TempCoeff = TempCoeff *( X_i[k] - X_i[i] ) + V[i];
     V[k] = ( Y_i[k] - TempCoeff ) * Reciprocal_m[k];
  } 

#ifdef BBDEFS_DENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : converting from newton to standard form" << endl;
   }
#endif

  // convert from newton form to standard form
  Poly_o = Basis_m[0] * V[0];
  for( k=1; k<=Deg_i; k++ ) 
     Poly_o = Poly_o + Basis_m[k] * V[k];

  if ( StaticDeg_m < Deg_i )
     StaticDeg_m = Deg_i;

  mpz_clear( &MPOne );
}

/*-------------------------------------------------------------
  function: UniDenseInterp
  date: 8/22/96

  reference:

  description:

  assumptions: 
-------------------------------------------------------------*/

template< class K, class P, class B > void UniDenseInterp(
   B* B_i,             // black box polynomial
   int Deg_i,          // > 0 --> exact degree, == 0 --> degree guessing
   K& Pnt_i,           //
   unsigned int PntType_i, // either an X or Y value
   K& ProbeElm_i,      // element to utilize for terminatin of deg guess loop
   K& GenElm_m,        // distinct field element generator
   int& StaticDeg_m,   // -1 indicates refresh 
   K* Ind_m,           // independent values X_i[1], ... ,X_i[Deg_i+1]
   K* Reciprocal_m,
   P* Basis_m,
   P& Poly_o
   )
{
   int i,k;
   MP_INT MPOne;
   mpz_init_set_ui( &MPOne, 1 ); 
   P  X( (MP_INT*)&MPOne, 1 );
   P TmpMonomial;
   K V[BBDENSE_MAXCOEFFS]; 
   K Dep[BBDENSE_MAXCOEFFS];  // dependent values   Y_i[1], ... ,Y_i[Deg_i+1]
   K Prod,TempCoeff;
   K ProbeVal;                // condition for deg guess loop termination
   K ProbePnts[2];
   K CurrVal;                 // currently interpolated poly at ProbeVal
   int Deg;                   // degree of currently interpolated poly
   int Loc;                   // location of varying point (0,1)
   
   if ( PntType_i == BBDENSE_XVALUE ) {
      ProbePnts[0] = Pnt_i;
      Loc = 1;
   }
   else {
      Loc = 0;
      ProbePnts[1] = Pnt_i;
   }

   // exact bound on the degree
   if ( Deg_i > 0 ) {
      for( i=0; i<=Deg_i; i++ ) {
         // generate more independent/dependent values if necessary
         if ( StaticDeg_m < i ) { 
            Ind_m[i] = GenElm_m.DistinctSelect();
         }
         ProbePnts[Loc] = Ind_m[i];
         Dep[i]   = *((*B_i)( ProbePnts ));
#ifdef BBDEFS_DENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbdense : ","Point generation Deg =", i );
   }
#endif
      }
      UniDenseInterp( Ind_m, Dep, Deg_i, StaticDeg_m, Reciprocal_m, 
                      Basis_m, Poly_o );
   }
   // degree guessing
   else {
      ProbePnts[Loc] = ProbeElm_i;
      ProbeVal = *((*B_i)( ProbePnts )); 
      Deg      = 0;

      if ( StaticDeg_m < Deg ) {
         Ind_m[0] = GenElm_m.DistinctSelect();
         Basis_m[0] = P( (MP_INT*)&MPOne, 0);
      }
      ProbePnts[Loc] = Ind_m[0];
      Dep[0]  = *((*B_i)( ProbePnts ));
      V[0]    = Dep[0];
      Poly_o  = Basis_m[0] * V[0]; 
      CurrVal = Poly_o.eval( &ProbeElm_i );
 
      // incrementally build polynomial
      while ( !(CurrVal == ProbeVal) ) {
         Deg = Deg + 1;

#ifdef BBDEFS_DENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbdense : ","Interpolation Deg =", Deg );
   }
#endif
         if ( StaticDeg_m < Deg ) {
            // compute the required inverses
            Ind_m[Deg] = GenElm_m.DistinctSelect();
            Prod = Ind_m[Deg] - Ind_m[0];
            for( i=1; i<=Deg-1; i++) 
               Prod = Prod *( Ind_m[Deg] - Ind_m[i] );
            Reciprocal_m[Deg] = K( (MP_INT*)&MPOne ) / Prod;
         }

         if ( StaticDeg_m < Deg ) {
            // compute the basis
            Basis_m[Deg] = ( X - Ind_m[Deg-1] );
            Basis_m[Deg] = Basis_m[Deg] * Basis_m[Deg-1];
         }

         // retrieve corresponding dependent value
         ProbePnts[Loc] = Ind_m[Deg];
         Dep[Deg] = *((*B_i)( ProbePnts ));

         // compute the newton coefficients
         TempCoeff = V[Deg-1];
         for( i=Deg-2; i>=0; i=i-1 ) { 
            TempCoeff = TempCoeff *( Ind_m[Deg] - Ind_m[i] ) + V[i];
         }
         V[Deg] = ( Dep[Deg] - TempCoeff ) * Reciprocal_m[Deg];
         TmpMonomial = Basis_m[Deg] * V[Deg];
         Poly_o = Poly_o + TmpMonomial;
         CurrVal = CurrVal + TmpMonomial.eval( &ProbeElm_i );
      }
      if ( StaticDeg_m < Deg )
         StaticDeg_m = Deg;
   }

   mpz_clear( &MPOne );

}

/*-------------------------------------------------------------
  function: BiDenseInterp
  date: 8/22/96

  reference:

  description:

  assumptions: 
-------------------------------------------------------------*/

template< class K, class P, class B > void BiDenseInterp(
   B* BB_i,             // black box polynomial
   K& ProbeElm_i,       // element to utilize for terminatin of deg guess loop
   K& GenElm_m,         // distinct field element generator
   int& StaticDeg_m,    // -i indicates refresh 
   K* Ind_m,            // independent values X_i[1], ... ,X_i[DegY+1]
   K* Reciprocal_m,
   P* Basis_m,
   BiPoly< P >& Poly_o
   )
{
   int i;   // iterate through X's
   int j;   // iterate through Y's
   int Deg,DegY;
   K Dep[BBDENSE_MAXCOEFFS];  // dependent values   Y_i[1], ... ,Y_i[DegY+1]
   K XVal;
   K Pnts[2];
   P YPoly[BBDENSE_MAXDEG];

   if ( StaticDeg_m > 0 )
      XVal = Ind_m[0];
   else
      XVal = GenElm_m;

   // interpolate first polynomial with degree guessing
   // this will compute the static information necessary further interpolations
   UniDenseInterp( BB_i, 0, XVal, BBDENSE_XVALUE, ProbeElm_i, GenElm_m, 
                   StaticDeg_m, Ind_m, Reciprocal_m, Basis_m, YPoly[0] );
   DegY = YPoly[0].dg();
   Deg  = DegY + 1;

#ifdef BBDEFS_BIDENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : BiDenseInterp : Degree in Y " << DegY << endl;
   }
#endif

   // interpolate polynomials in Y
   for( i=1; i<Deg; i++ ) {
      for( j=0; j<Deg; j++ ) {
         Pnts[0]  = Ind_m[i];
         Pnts[1]  = Ind_m[j];
         Dep[j]   = *((*BB_i)( Pnts ));       
#ifdef BBDEFS_BIDENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : BiDenseInterp : YPoly[" << i << "] at deg = " << j 
        << endl;
   }
#endif
      }
      UniDenseInterp( Ind_m, Dep, DegY, StaticDeg_m, Reciprocal_m, 
                      Basis_m, YPoly[i]);
   }

   // interpolate coefficients in X
   for( i=0; i<Deg; i++ ) {
      for( j=0; j<Deg; j++ ) 
         Dep[j]   = (YPoly[j])[i];
      UniDenseInterp( Ind_m, Dep,DegY, StaticDeg_m, Reciprocal_m, 
                      Basis_m, Poly_o[i]);
   }

   Poly_o.SetDeg( DegY );
}

// WARNING THIS ROUTINE STILL HAS TO BE MADE FULLY GENERIC!!!!!!!

template< class K, class P, class B > void SpecialBiDenseInterp(
   B* BB_i,             // black box polynomial
   int DegF_i,          // degree of of bi-variate polynomial to interpolate
   K& GenElm_m,         // distinct field element generator
   int& StaticDeg_m,    // -i indicates refresh 
   K* Ind_m,            // independent values X_i[1], ... ,X_i[DegY+1]
   K* Reciprocal_m,
   P* Basis_m,
   BiPoly< P >& PrunePoly_o
   )
{
   int i;    // iterate through X's
   int j;    // iterate through Y's
   int Deg,k,exp;
   K Dep[BBDENSE_MAXCOEFFS];  // dependent values   Y_i[1], ... ,Y_i[Deg+1]
   K Pnts[2];
   K PruneValue;
   P YPoly[BBDENSE_MAXDEG];

   Deg  = DegF_i + 1;
   StaticDeg_m  = -1;

#ifdef BBDEFS_BIDENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : SpecialBiDenseInterp : Degree in Y " << DegF_i << endl;
   }
#endif

   // iterate through X's
   for( i=1; i<=DegF_i+1; i++ ) {

      // interpolate a polynomial in Y
      for( j=1; j<=Deg; j++ ) {
         Ind_m[j-1] = (long)j;
         Pnts[0]    = (long)i;
         Pnts[1]    = (long)j;
         PruneValue = (long)0;
         for(k=1; k<i; k++ ) {
             PruneValue = PruneValue + 
                          PrunePoly_o[ DegF_i + 1 -k ].eval( &Pnts[0] ) * 
                          Pnts[1].Exp( DegF_i - k + 1 );
         }
         Dep[j-1]   = *((*BB_i)( Pnts )) - PruneValue;       
      }
      UniDenseInterp( Ind_m, Dep, Deg-1, StaticDeg_m, Reciprocal_m, 
                      Basis_m, YPoly[i]);
      Deg = Deg - 1;

      // interpolate coefficient in X
      for( j=1; j<=DegF_i-Deg+1; j++ ) {
         Ind_m[j-1] = (long)j;
         Dep[j-1]   = (YPoly[j])[Deg];
      }
      UniDenseInterp( Ind_m, Dep, DegF_i-Deg, StaticDeg_m, Reciprocal_m, 
                      Basis_m, PrunePoly_o[DegF_i-i+1]);

#ifdef BBDEFS_BIDENSE_TRC
   if ( PRINTLEVEL > 2 ) {
   cout << "bbdense : SpecialBiDenseInterp : PrunePoly[" << DegF_i-i+1 <<
           "] = " << PrunePoly_o[DegF_i-i+1]  << endl;
   }
#endif

   }

   PrunePoly_o.SetDeg( DegF_i );
}

#endif


