
/*-------------------------------------------------------------
  module: bbselector.h
  author: Angel L. Diaz

  date: 10/23/96

  description: 
     The classes in this module provides for selecting a single 
     value from black box objects that return multiple values.

  classes:
   BlackBoxSelector
   BlackBoxSelectorEx

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#ifndef _BBSELECTOR
#define _BBSELECTOR

#include "BlackBox/CommonObjects/blackbox.h"

/*-------------------------------------------------------------
  class: BlackBoxSelector
  date: 10/23/96

  description: 
     Provides for constructing a wrapper for black box objects 
     which returns an array of values from a native imported 
     domain arithmetic (K).

-------------------------------------------------------------*/

template < class K, class B > 
class BlackBoxSelector : public BlackBox< K > {
   protected:
      B* BB;
      int Index;
   public:
      BlackBoxSelector( );
      BlackBoxSelector( B&, int );
      BlackBoxSelector( const BlackBoxSelector< K, B >& );
      ~BlackBoxSelector( ) { }

      inline void Construct(  B&, int );
      inline K* operator () ( K* );
      inline BlackBoxSelector& operator=( const BlackBoxSelector< K, B >& );
};

template < class K, class B > 
BlackBoxSelector< K, B >::BlackBoxSelector( ) {
}

template < class K, class B > 
BlackBoxSelector< K, B >::BlackBoxSelector(
   const BlackBoxSelector< K, B >&  BlackBoxSelector_i
   )
{
   *this = BlackBoxSelector_i;
}

template < class K, class B > 
BlackBoxSelector< K, B >::BlackBoxSelector(
   B&  BB_i,
   int Index_i
   )
{
   Construct( BB_i, Index_i );
}

template < class K, class B > 
inline void BlackBoxSelector< K, B >::Construct(
   B&  BB_i,
   int Index_i
   )
{
   BB           = &BB_i;
   Index        = Index_i;
   NumVariables = BB_i.NumVars();
   Probability  = BB_i.Prob();
   Degree       = BB_i.Deg();
}

template < class K, class B > inline K* 
BlackBoxSelector< K, B >::operator() (
   K* Pnts_i
   )
{
   return ( &(*BB)( Pnts_i )[ Index ] );
}

template < class K, class B >
inline BlackBoxSelector< K, B >& BlackBoxSelector< K, B >::operator= (
   const BlackBoxSelector< K, B >& BlackBoxSelector_i
)
{
   BB           = BlackBoxSelector_i.BB;
   Index        = BlackBoxSelector_i.Index;
   NumVariables = BlackBoxSelector_i.NumVariables;
   Probability  = BlackBoxSelector_i.Probability;
   Degree       = BlackBoxSelector_i.Degree;
}

/*-------------------------------------------------------------
  class: BlackBoxSelectorEx
  date: 10/23/96

  description: 
     Provides for constructing a wrapper for black box objects 
     which returns an array of values from a native imported 
     domain arithmetic (K) and extended domain arithmetic (L).

-------------------------------------------------------------*/

template < class K, class L, class B > 
class BlackBoxSelectorEx : public BlackBoxEx< K, L > {
   private:
      B* BB;
      unsigned int Index;
   public:
      BlackBoxSelectorEx( );
      BlackBoxSelectorEx( const BlackBoxSelectorEx< K, L, B >& );
      BlackBoxSelectorEx( B&, unsigned int );
      ~BlackBoxSelectorEx( ) { }

      inline K* operator () ( K* );
      inline L* operator () ( L* );
      inline BlackBoxSelectorEx& operator=( const BlackBoxSelectorEx<K,L,B>& );
};

template < class K, class L, class B > 
BlackBoxSelectorEx< K, L, B >::BlackBoxSelectorEx( ) {
}

template < class K, class L, class B > 
BlackBoxSelectorEx< K, L, B >::BlackBoxSelectorEx(
   const BlackBoxSelectorEx< K, L, B >&  BlackBoxSelectorEx_i
   )
{
   *this = BlackBoxSelectorEx_i;
}

template < class K, class L, class B > 
BlackBoxSelectorEx< K, L, B >::BlackBoxSelectorEx(
   B&           BB_i,
   unsigned int Index_i
   )
{
   BB           = &BB_i;
   Index        = Index_i;
   NumVariables = BB_i.NumVars();
   Probability  = BB_i.Prob();
   Degree       = BB_i.Deg();
}

template < class K, class L, class B > 
inline K* BlackBoxSelectorEx< K, L, B >::operator() (
   K* Pnts_i
   )
{
   return ( &(*BB)( Pnts_i )[ Index ] );
}

template < class K, class L, class B > 
inline L* BlackBoxSelectorEx< K, L, B >::operator() (
   L* Pnts_i
   )
{
   return ( &(*BB)( Pnts_i )[ Index ] );
}

template < class K, class L, class B >
inline BlackBoxSelectorEx<K, L, B>& BlackBoxSelectorEx<K, L, B>::operator= (
   const BlackBoxSelectorEx< K, L, B >& BlackBoxSelectorEx_i
)
{
   BB           = BlackBoxSelectorEx_i.BB;
   Index        = BlackBoxSelectorEx_i.Index;
   NumVariables = BlackBoxSelectorEx_i.NumVariables;
   Probability  = BlackBoxSelectorEx_i.Probability;
   Degree       = BlackBoxSelectorEx_i.Degree;
}

#endif
