/*-------------------------------------------------------------
  module: bbbbvector.h
  author: Angel L. Diaz
  date: 8/18/96
-------------------------------------------------------------*/

#ifndef BBVECTOR
#define BBVECTOR

#define BBVECTOR_MAXSIZE  1000
#define BBSVECTOR_MAXSIZE 100

/*
// for Toeplitz
#define BBVECTOR_MAXSIZE  15000
#define BBSVECTOR_MAXSIZE 20
*/

/*-------------------------------------------------------------
  class: bbvector 
  date: 8/18/96

  description: implements a subset of STL bbvectors w/o utilizing 
     C++ dynamic memory allocation (eg. new). This sequence 
     conatiner may be supplied in place of the STL definition in
     situations when an arithmetic plug-in requires applications
     to statically allocate memory (eg. SACLIB).

  assumptions: this class only implements the subset of the STL
     bbvector class. Those operations utilizied by the FoxBox 
     libraries.
-------------------------------------------------------------*/

template < class T > class bbvector {
   public:
      typedef T* iterator;
      bbvector( );
      bbvector( unsigned int );
      bbvector( unsigned int, T );
      ~bbvector( );

      inline iterator begin( );
      inline iterator end( );
      inline unsigned int size( );
      inline T pop_back( );
      inline void push_back( T );

      inline void insert( T*, T*, T* );
      inline T& operator[] ( unsigned int );
      inline bbvector< T >& operator=( const bbvector< T>& );
   private:
      T Elms[BBVECTOR_MAXSIZE];
      unsigned int SIZE;
};

template< class T > bbvector<T>::bbvector( ) 
{ 
   SIZE   = 0; 
}

template< class T > bbvector<T>::bbvector( 
   unsigned int size_i 
   ) 
{ 
   SIZE   = size_i; 
}

template< class T > bbvector<T>::bbvector( 
   unsigned int size_i, 
   T init_i 
   ) 
{ 
   unsigned int i;

   // initialize bbvector 
   for( i=0; i<size_i; i++ )
       Elms[i] = init_i;
   SIZE   = size_i;
}

template< class T > bbvector<T>::~bbvector( ) { 
}

template< class T > inline T* bbvector<T>::begin( ) { 
   return Elms; 
}

template< class T > inline T* bbvector<T>::end( ) 
{ 
   return Elms+SIZE; 
}

template< class T > inline unsigned int bbvector<T>::size( ) 
{ 
   return SIZE; 
}

template< class T > inline T bbvector<T>::pop_back( ) 
{ 
   SIZE--; 
   return( Elms[SIZE] ); 
}

template< class T > inline void bbvector<T>::push_back( 
   T Elm_i 
   ) 
{ 
   Elms[SIZE] = Elm_i; 
   SIZE++; 
}

template< class T > inline void bbvector<T>::insert( 
   T* Dest_i, 
   T* Start_i, 
   T* End_i 
   ) 
{
   T* ElmPntr;
   T* Dest;
   unsigned int size = 0;

   ElmPntr = Elms;
   Dest    = Dest_i;
  
   // find location in destination array, adjust size accordingly
   while ( ElmPntr != Dest ) { 
      size++; ElmPntr++; 
   }

   // copy source to destination 
   ElmPntr = Start_i;
   while ( ElmPntr != End_i ) {
      *Dest = *ElmPntr;
      size++;
      ElmPntr++; Dest++;
   }

   // adjust for a possible size increase
   if ( size > SIZE )  
      SIZE = size; 
}    

template< class T > inline T& bbvector<T>::operator[] ( 
   unsigned int Index_i 
   ) 
{ 
   return Elms[ Index_i ]; 
}

template< class T > inline bbvector< T >& bbvector<T>::operator=( 
   const bbvector< T>& input_i 
   )
{
   unsigned int i;

   // copy element array and adjust size
   for( i=0; i<BBVECTOR_MAXSIZE; i++ )
      Elms[i] = input_i.Elms[i];  

   SIZE = input_i.SIZE;

   return *this;
}

/*-------------------------------------------------------------
  class: bbsvector 
  date: 8/18/96

  description: implements a subset of STL bbsvectors w/o utilizing 
     C++ dynamic memory allocation (eg. new). This sequence 
     conatiner may be supplied in place of the STL definition in
     situations when an arithmetic plug-in requires applications
     to statically allocate memory (eg. SACLIB).

  assumptions: this class only implements the subset of the STL
     bbsvector class. Those operations utilizied by the FoxBox 
     libraries.
-------------------------------------------------------------*/

template < class T > class bbsvector {
   public:
      typedef T* iterator;
      bbsvector( );
      bbsvector( unsigned int );
      bbsvector( unsigned int, T );
      ~bbsvector( );

      inline iterator begin( );
      inline iterator end( );
      inline unsigned int size( );
      inline T pop_back( );
      inline void push_back( T );

      inline void insert( T*, T*, T* );
      inline T& operator[] ( unsigned int );
      inline bbsvector< T >& operator=( const bbsvector< T>& );
   private:
      T Elms[BBSVECTOR_MAXSIZE];
      unsigned int SIZE;
};

template< class T > bbsvector<T>::bbsvector( ) 
{ 
   SIZE   = 0; 
}

template< class T > bbsvector<T>::bbsvector( 
   unsigned int size_i 
   ) 
{ 
   SIZE   = size_i; 
}

template< class T > bbsvector<T>::bbsvector( 
   unsigned int size_i, 
   T init_i 
   ) 
{ 
   unsigned int i;

   // initialize bbsvector 
   for( i=0; i<size_i; i++ )
       Elms[i] = init_i;
   SIZE   = size_i;
}

template< class T > bbsvector<T>::~bbsvector( ) { 
}

template< class T > inline T* bbsvector<T>::begin( ) { 
   return Elms; 
}

template< class T > inline T* bbsvector<T>::end( ) 
{ 
   return Elms+SIZE; 
}

template< class T > inline unsigned int bbsvector<T>::size( ) 
{ 
   return SIZE; 
}

template< class T > inline T bbsvector<T>::pop_back( ) 
{ 
   SIZE--; 
   return( Elms[SIZE] ); 
}

template< class T > inline void bbsvector<T>::push_back( 
   T Elm_i 
   ) 
{ 
   Elms[SIZE] = Elm_i; 
   SIZE++; 
}

template< class T > inline void bbsvector<T>::insert( 
   T* Dest_i, 
   T* Start_i, 
   T* End_i 
   ) 
{
   T* ElmPntr;
   T* Dest;
   unsigned int size = 0;

   ElmPntr = Elms;
   Dest    = Dest_i;
  
   // find location in destination array, adjust size accordingly
   while ( ElmPntr != Dest ) { 
      size++; ElmPntr++; 
   }

   // copy source to destination 
   ElmPntr = Start_i;
   while ( ElmPntr != End_i ) {
      *Dest = *ElmPntr;
      size++;
      ElmPntr++; Dest++;
   }

   // adjust for a possible size increase
   if ( size > SIZE )  
      SIZE = size; 
}    

template< class T > inline T& bbsvector<T>::operator[] ( 
   unsigned int Index_i 
   ) 
{ 
   return Elms[ Index_i ]; 
}

template< class T > inline bbsvector< T >& bbsvector<T>::operator=( 
   const bbsvector< T>& input_i 
   )
{
   unsigned int i;

   // copy element array and adjust size
   for( i=0; i<BBVECTOR_MAXSIZE; i++ )
      Elms[i] = input_i.Elms[i];  

   SIZE = input_i.SIZE;

   return *this;
}

#endif
