
/*-------------------------------------------------------------
  module: bbfachmaptrace.h
  author: Angel L. Diaz

  date: 10/26/96

  desciption: 
     Tracing routines for: bbfachmap.h
 
     PRINTLEVEL = 0 ; tracing off
     PRINTLEVEL > 0 ; function tags
     PRINTLEVEL > 1 ; milestone messages
     PRINTLEVEL > 2 ; state variables
     PRINTLEVEL > 3 ; state variables and arrays/vectors

  functions:
     BBFACSHMTrace1
     BBFACSHMTrace2
     BBFACSHMTrace3
     BBFACSHMTrace4
     BBFACSHMTrace5

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.

-------------------------------------------------------------*/

#ifndef _BBFACHMAPTRACE
#define _BBFACHMAPTRACE

#ifdef BBDEFS_BBFACTOR_HMAP_TRC

#include "BlackBox/Aux/bbtrace.h"

template < class K >
void BBFACSHMTrace1(
   int           Degree_i,
   int           NumVaribales_i,
   double        Probability_i,
   unsigned int  NumFactors_i,
   unsigned int  SumExps_i,
   int           Sign_i,
   unsigned int  DegF_i,
   int           ParFracR_i,
   int           ParFracC_i,
   unsigned int* Exps_i,
   int           StaticDeg_i,
   K&            ProbeElm_i,
   K&            GenElm_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "Degree = ",Degree_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "NumVaribales = ",NumVaribales_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "Probability = ",Probability_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "NumFactors_i = ",NumFactors_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "SumExps = ",SumExps_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "Sign = ",Sign_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "DegF = ",DegF_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "ParFracR = ",ParFracR_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "ParFracC = ",ParFracC_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "StaticDeg = ", StaticDeg_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "ProbeElm = ", ProbeElm_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "GenElm = ",GenElm_i);
   }
   if ( PRINTLEVEL > 1 ) {
   OutputArray("BlackBoxFactorsHMap : Construction : ", 
               "Exponets", "Exps[", Exps_i, 0,NumFactors_i );
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactorsHMap : Construction : Converting random elements" 
        << endl;
   }
}

template < class K >
void BBFACSHMTrace2(
   K*  RandA_i,
   K*  RandB_i,
   K*  RandC_i,
   int NumVariables_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactorsHMap : Construction : ", 
               "Random elemnts a2, ... , an", "RandA[", RandA_i, 0, 
               NumVariables_i-1 );
   OutputArray("BlackBoxFactorsHMap : Construction : ",
               "Random elemnts b1, ... , bn", "RandB[", RandB_i, 0,
               NumVariables_i );
   OutputArray("BlackBoxFactorsHMap : Construction : ",
               "Random elemnts c1,c3 ... , cn", "RandC[", RandC_i, 0,
               NumVariables_i-1 );
   }
}

template < class K, class P >
void BBFACSHMTrace3(
   K& Cont_i,
   P& pgsem1_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "Cont = ",Cont_i);
   OutputSingleItem("BlackBoxFactorsHMap : Construction : ",
      "pgsem1 = ",pgsem1_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactorsHMap : Construction : Converting factor images"
        << endl;
   }
}

template < class P >
void BBFACSHMTrace4(
   P*           FactorImages_i,
   unsigned int NumFactors_i,
   P*           FactorProducts_i,
   unsigned int SumExps_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputArray("BlackBoxFactorsHMap : Construction : ", 
               "Factor Images", "FactorImages[", FactorImages_i, 0, 
               NumFactors_i );
   OutputArray("BlackBoxFactorsHMap : Construction : ",
               "Factor Products", "FactorProducts[",FactorProducts_i, 0,
               SumExps_i );
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactorsHMap : Construction : Converting partial fractions"
        << endl;
   }
}

template < class P >
void BBFACSHMTrace5(
   P ParFrac_i[BBFACTOR_MAXDEG][BBFACTOR_MAXFACTORS],
   int ParFracR_i,
   int ParFracC_i
   )
{
  int i,j;
  for( i=0; i<=ParFracC_i; i++ ) {
      for( j=0; j<=ParFracR_i; j++ ) {
         cout << "BlackBoxFactorsHMap : Construction : ParFrac[" 
              << i << "][" << j << "] = " << ParFrac_i[i][j] << endl;
      } 
  }
   if ( PRINTLEVEL > 1 ) {
   cout << "BlackBoxFactorsHMap : Construction : Done!" << endl;
   }
}

#endif

#endif
