
/*-------------------------------------------------------------
  module: bbhmap.h
  author: Angel L. Diaz

  date: 10/24/96

  class:
     HPoly

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#ifndef __BBHMAP
#define __BBHMAP

/*-------------------------------------------------------------
  class: HPoly
  date: 10/23/96

  description:
     Provides for performing a homorphic map of polynomials (KP,FP)
     given a base domain element mapping funcition H.

-------------------------------------------------------------*/

template < class K, class KP, class F, class FP, class H > 
class HPoly {
   private:
      H h;
      int ConvertStatus;
   public:
      HPoly( H& h_i ) { h = h_i; }
      ~HPoly()        { }

      inline KP operator () ( FP& );
      inline FP operator () ( KP& );
      inline int IsConverted() { return ConvertStatus; }
};

template < class K, class KP, class F, class FP, class H >
inline KP HPoly< K, KP, F, FP, H >::operator() (
   FP& FP_i
   )
{
   MP_INT  MPZero;
   KP      KPoly;
   K       KCoeff;
   F       FCoeff;
   int     deg,i;

   mpz_init_set_ui( &MPZero, 0 );
   deg   = FP_i.dg();
   KPoly = KP( (MP_INT*) &MPZero );
   ConvertStatus = BBDEFS_TRUE;

   for( i=0; i<=deg; i++ ) {
      FCoeff = FP_i[i];
      KCoeff = h( FCoeff );
      if ( h.IsConverted() == BBDEFS_FALSE ) {
         ConvertStatus = BBDEFS_FALSE;
         KPoly = KP( (MP_INT*) &MPZero );
         break;
      }
      KPoly = KPoly + KP( KCoeff, i );
   }

   // check for degree change
   if ( KPoly.dg() != deg ) {
      ConvertStatus = BBDEFS_FALSE;
      KPoly = KP( (MP_INT*) &MPZero );
   }

   mpz_clear( &MPZero );
   return KPoly;
}

template < class K, class KP, class F, class FP, class H >
inline FP HPoly< K, KP, F, FP, H >::operator() (
   KP& KP_i
   )
{
   MP_INT  MPZero;
   FP      FPoly;
   K       KCoeff;
   F       FCoeff;
   int     deg,i;

   mpz_init_set_ui( &MPZero, 0 );
   deg   = KP_i.dg();
   FPoly = FP( (MP_INT*) &MPZero );
   ConvertStatus = BBDEFS_TRUE;

   for( i=0; i<=deg; i++ ) {
      KCoeff = KP_i[i];
      FCoeff = h( KCoeff );
      if ( h.IsConverted() == BBDEFS_FALSE ) {
         ConvertStatus = BBDEFS_FALSE;
         FPoly = FP( (MP_INT*) &MPZero );
         break;
      }
      FPoly = FPoly + FP( FCoeff, i );
   }

   // check for degree change
   if ( FPoly.dg() != deg ) {
      ConvertStatus = BBDEFS_FALSE;
      FPoly = FP( (MP_INT*) &MPZero );
   }

   mpz_clear( &MPZero );
   return FPoly;
}

#endif

