
/*-------------------------------------------------------------
  module: bbservermpi.h
  author: Angel L. Diaz

  date: 10/29/96

  description: 

  class:
     BlackBoxServer

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#ifndef _BBSERVERMPI
#define _BBSERVERMPI

#include "BlackBox/Aux/bbdefs.h"
#include "BlackBox/Aux/bbtime.h"
#include "BlackBox/Algorithms/bbgcdaux.h"
#include "BlackBox/Algorithms/bbfactoraux.h"
#include "BlackBox/Aux/bbdefsmpi.h"

template < class K, class P, class B > 
class BlackBoxServerMPI {
   private:
      // Common staic information
      unsigned int NumVariables;
      K RandA[BBMPI_MAXVAR];  
      K RandB[BBMPI_MAXVAR];  
      K RandC[BBMPI_MAXVAR];  
      K ProbeElm;       
      K GenElm;  
      int StaticDeg;
      K Ind[BBDENSE_MAXCOEFFS];
      K Reciprocal[BBDENSE_MAXCOEFFS];
      P Basis[BBDENSE_MAXCOEFFS];
      // GCD specific Information
      B** BBGCD;
      unsigned int NumBlackBoxes;
      unsigned int Delta;
      int PolyDegs[BBGCD_MAXPOLYS];
      P G0;    
      K LG0;    
      // Factors specific Information
      B* BBFAC; 
      unsigned int NumFactors; 
      unsigned int SumExps;
      unsigned int Exps[BBFACTOR_MAXFACTORS];
      int Sign;
      unsigned int DegF;
      int ParFracR;
      int ParFracC;
      K Cont;
      P FactorImages[BBFACTOR_MAXFACTORS]; 
      P FactorProducts[BBFACTOR_MAXFACTORS*BBFACTOR_MAXDEG]; 
      P pgsem1; 
      P ParFrac[BBFACTOR_MAXDEG][BBFACTOR_MAXFACTORS];
      K ValueOfFactors[BBFACTOR_MAXFACTORS];
      void HandleGCD( int, int, ofstream& );
      void HandleGCDEval( int, int, ofstream& );
      void HandleFactors( int, int, ofstream& );
      void HandleFactorsEval( int, int, ofstream& );
   public:
      BlackBoxServerMPI( int, int, B&, char*, int );
      ~BlackBoxServerMPI( ) { }
};

template < class K, class P, class B > 
BlackBoxServerMPI< K, P, B >::BlackBoxServerMPI( 
   int DriverRank_i,
   int MyRank_i,
   B&    BB_i, 
   char* PreStr_i,
   int   IsClock = BBMPI_CLOCK_OFF
   )
{
   int        Request;                  // Storage for the request 
   int        IsWaiting;                // deamon mode flag
   MPI_Status status;                   // Return status for receive 
   char       FileName[BBMPI_FILESTRZ];
   
   sprintf(FileName,"%s_%d.log",PreStr_i,MyRank_i);
   ofstream LogFile( FileName, ios::out ); 

   if ( IsClock ) {
      GetCPUTime( LogFile );
   }
 
   LogFile << "BlackBoxServerMPI : Starting up" << endl;

   IsWaiting  = BBDEFS_TRUE;
   BBFAC      = &BB_i;

   // in deamon mode we will run a long time
   while ( IsWaiting ) {

      //LogFile << "BlackBoxServerMPI : Waiting for requests ... " << endl; 

      MPI_Recv( &Request, 1, MPI_INT, DriverRank_i, BBMPI_REQUEST, 
                MPI_COMM_WORLD, &status);

      switch( Request ) {

         case BBMPI_GCD:
            HandleGCD( DriverRank_i, MyRank_i, LogFile );
         break;

         case BBMPI_GCD_EVAL:
            HandleGCDEval( DriverRank_i, MyRank_i, LogFile );
         break;

         case BBMPI_FACTORS:
            HandleFactors( DriverRank_i, MyRank_i, LogFile );
         break;

         case BBMPI_FACTORS_EVAL:
            HandleFactorsEval( DriverRank_i, MyRank_i, LogFile );
            if ( IsClock ) {
               GetCPUTime( LogFile );
            }
         break;

         case BBMPI_KILL:
            LogFile << "BlackBoxServerMPI : Killing local process" << endl;
            if ( IsClock ) {
               GetCPUTime( LogFile );
            }
            LogFile.close();
            IsWaiting = BBDEFS_FALSE;
         break;

      }
   }
}

template < class K, class P,class B > 
void BlackBoxServerMPI< K, P, B >::HandleGCD( 
   int       DriverRank_i,
   int       MyRank_i,
   ofstream& LogFile_m 
   )
{
   int        i;
   int        Request; // Storage for the request 
   MPI_Status status;  // Return status for receive 
}

template < class K, class P, class B > 
void BlackBoxServerMPI< K, P, B >::HandleGCDEval( 
   int       DriverRank_i,
   int       MyRank_i,
   ofstream& LogFile_m 
   )
{
   int        i;
   int        Request; // Storage for the request 
   MPI_Status status;  // Return status for receive 
}

template < class K, class P, class B > 
void BlackBoxServerMPI< K, P, B >::HandleFactors( 
   int       DriverRank_i,
   int       MyRank_i,
   ofstream& LogFile_m
   )
{
   int        i,j,k,deg;
   int        Request; // Storage for the request 
   MPI_Status status;  // Return status for receive 
   MP_INT     MPZero;
   char       ZPElmStr[BBMPI_KSTRZ];
   K          Coeff;
   mpz_init_set_si( &MPZero, 0 );

   LogFile_m << "BlackBoxServerMPI : HandleFactors : Constructing" << endl;

   MPI_Recv( &StaticDeg, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC, 
             MPI_COMM_WORLD, &status);
   MPI_Recv( &NumVariables, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC, 
             MPI_COMM_WORLD, &status);
   MPI_Recv( &NumFactors, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC, 
             MPI_COMM_WORLD, &status);
   MPI_Recv( &SumExps, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
   MPI_Recv( &Sign, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
   MPI_Recv( &DegF, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
   MPI_Recv( &ParFracR, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
   MPI_Recv( &ParFracC, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
   for( i=0; i<NumFactors; i++ ) {
      MPI_Recv( &Exps[i], 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
                MPI_COMM_WORLD, &status);
   }
   for( i=0; i<NumVariables-1; i++ )  {
      MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i, 
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
      RandA[i] = K( ZPElmStr );
   }
   for( i=0; i<NumVariables; i++ ) {
      MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i, 
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
      RandB[i] = K( ZPElmStr );
   }
   for( i=0; i<NumVariables-1; i++ )  {
      MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i, 
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
      RandC[i] = K( ZPElmStr );
   }
   MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
             BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
   ProbeElm = K( ZPElmStr );
   MPI_Recv( &ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
             BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
   GenElm = K( ZPElmStr );
   MPI_Recv( &ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
             BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
   Cont = K( ZPElmStr );
   for( i=0; i<=StaticDeg; i++ ) {
      MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i, 
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
      Ind[i] = K( ZPElmStr );
   }
   for( i=1; i<=StaticDeg; i++ ) {
      MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
      Reciprocal[i] = K( ZPElmStr );
   }
   for( j=0; j<=StaticDeg; j++ ) {
      MPI_Recv( &deg, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
      Basis[j] = K( (MP_INT*)&MPZero );
      for( i=0; i<=deg; i++ ) {
         MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
         Coeff = K( ZPElmStr );
         Basis[j] = Basis[j] + P( Coeff, i ); 
      }
   } 
   for( j=0; j<NumFactors; j++ ) {
      MPI_Recv( &deg, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
      FactorImages[j] = K( (MP_INT*)&MPZero );
      for( i=0; i<=deg; i++ ) {
         MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
         Coeff = K( ZPElmStr );
         FactorImages[j] = FactorImages[j] + P( Coeff, i ); 
      }
    } 
   for( j=0; j<SumExps; j++ ) {
      MPI_Recv( &deg, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
             MPI_COMM_WORLD, &status);
      FactorProducts[j] = K( (MP_INT*)&MPZero );
      for( i=0; i<=deg; i++ ) {
         MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
         Coeff = K( ZPElmStr );
         FactorProducts[j] = FactorProducts[j] + P( Coeff, i ); 
      }
   } 
   MPI_Recv( &deg, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
          MPI_COMM_WORLD, &status);
   pgsem1 = K( (MP_INT*)&MPZero );
   for( i=0; i<=deg; i++ ) {
      MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
             BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
      Coeff = K( ZPElmStr );
      pgsem1 = pgsem1 + P( Coeff, i ); 
   }
   for( i=0; i<=ParFracC; i++ ) {
      for( j=0; j<=ParFracR; j++ ) {
         MPI_Recv( &deg, 1, MPI_INT, DriverRank_i, BBMPI_FACTORS_STATIC,
                   MPI_COMM_WORLD, &status);
         ParFrac[i][j] = K( (MP_INT*)&MPZero );
         for( k=0; k<=deg; k++ ) {
            MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i,
                      BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
            Coeff = K( ZPElmStr );
            ParFrac[i][j] = ParFrac[i][j] + P( Coeff, k );
         }
      }
   }

   LogFile_m << "BlackBoxServerMPI : HandleFactors : Done!" << endl;
   LogFile_m.flush();
   mpz_clear( &MPZero ); 
}

template <class K, class P,class B> void 
BlackBoxServerMPI<K,P,B>::HandleFactorsEval(
   int       DriverRank_i,
   int       MyRank_i,
   ofstream& LogFile_m 
   )
{
   int          i,j;
   int          Request; // Storage for the request 
   MPI_Status   status;  // Return status for receive 
   char         ZPElmStr[BBMPI_KSTRZ];
   BiPoly< P >  FBar;
   BiPoly< P >  BiFactors[BBFACTOR_MAXFACTORS];
   K            EvalPnt;
   K            FacPnts[BBFACTOR_MAXVAR];

   for( i=0; i<NumVariables; i++ ) {
      MPI_Recv( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i, 
                BBMPI_FACTORS_STATIC, MPI_COMM_WORLD, &status);
      FacPnts[i] = K( ZPElmStr );
   }

   //LogFile_m << "BlackBoxServerMPI : HandleFactorsEval : Starting!" << endl;

   MapFtoFBar<K,P,B> BBFBar( BBFAC, RandA, RandB, FacPnts, NumVariables );

/*
   // general purpose bi-variate interpolation (w/degree guessing)
   BiDenseInterp( &BBFBar, ProbeElm, GenElm, StaticDeg, Ind,
                  Reciprocal, Basis, FBar );
*/

   // special pruning bi-variate interpolation
   SpecialBiDenseInterp( &BBFBar, DegF, GenElm, StaticDeg, Ind, Reciprocal,
                         Basis, FBar );

   Lift( FBar, FactorImages, NumFactors, FactorProducts, SumExps, Exps,
         ParFrac, pgsem1, BiFactors );

   EvalPnt = FacPnts[0] - RandB[0];

   for( i=0; i<NumFactors; i++) {
      ValueOfFactors[i] = (BiFactors[i])[0].eval( &EvalPnt );
      for( j=1; j<(BiFactors[i]).Deg(); j++ )
          ValueOfFactors[i] = ValueOfFactors[i] +
              (BiFactors[i])[j].eval( &EvalPnt );
      //ValueOfFactors[i] = ValueOfFactors[i] * (long)Sign;
   }

   for( i=0; i<NumFactors; i++ ) {
      ValueOfFactors[i].toString( ZPElmStr );
      MPI_Send( ZPElmStr, BBMPI_KSTRZ, MPI_CHAR, DriverRank_i, 
                BBMPI_FACTORS_ANS, MPI_COMM_WORLD);
   }

   //LogFile_m << "BlackBoxServerMPI : HandleFactorsEval : Done!" << endl;

}

#endif

