/*-------------------------------------------------------------
  module: mod.h

  author: Angel L. Diaz
  date: 10/3/96

  mapping function objects for generating modular black boxes
-------------------------------------------------------------*/

#ifndef _MOD
#define _MOD
#define MOD_KSTRZ 5000
#define MOD_PSTRZ 10000

#include "PlugIns/saclib.h"
#include "PlugIns/shoup.h"
#include <string.h>

class SaclibQShoupZP {
   private:
      int ConvertStatus;
   public:
      SaclibQShoupZP() {}
      ~SaclibQShoupZP() {} 

      inline ShoupZP operator () ( SaclibQ& );
      inline SaclibQ operator () ( ShoupZP& );
      inline int IsConverted() { return ConvertStatus; }
};

inline ShoupZP SaclibQShoupZP::operator () ( 
   SaclibQ& QElm
   )
{
   char    StrNumElm[MOD_KSTRZ];
   char    StrDenomElm[MOD_KSTRZ];
   MP_INT  MPNumElm;
   MP_INT  MPDenomElm;
   MP_INT  MPOne,MPZero;
   ShoupZP ZPNumElm;
   ShoupZP ZPDenomElm;
   ShoupZP ZPElm;

   QElm.NumtoString( StrNumElm );
   QElm.DenomtoString( StrDenomElm );

   mpz_init_set_ui( &MPOne, 1 );
   mpz_init_set_ui( &MPZero, 0 );
   mpz_init_set_str( &MPNumElm, StrNumElm , 10 );
   mpz_init_set_str( &MPDenomElm, StrDenomElm , 10 );
   
   ZPNumElm   = ShoupZP( (MP_INT*) &MPNumElm );
   ZPDenomElm = ShoupZP( (MP_INT*) &MPDenomElm );

   // check for zero denominator
   if ( ZPDenomElm != ShoupZP( (MP_INT*) &MPZero ) ) {  
      ZPElm = ( ShoupZP( (MP_INT*) &MPOne) / ZPDenomElm ) * ZPNumElm;
      ConvertStatus = BBDEFS_TRUE;
   }
   else {
      ZPElm = ShoupZP( (MP_INT*) &MPZero );
      ConvertStatus = BBDEFS_FALSE;
   }

   mpz_clear( &MPOne );
   mpz_clear( &MPZero );
   mpz_clear( &MPNumElm );
   mpz_clear( &MPDenomElm );

   return ZPElm;
}

inline SaclibQ SaclibQShoupZP::operator () ( 
   ShoupZP& ZPElm 
   )
{
   char    StrElm[MOD_KSTRZ];
   int     StrLen;

   ZPElm.toString( StrElm );
   StrLen = strlen( StrElm);   
   StrElm[StrLen] = '/'; StrLen++;
   StrElm[StrLen] = '1'; StrLen++;
   StrElm[StrLen] = '\0';

   return( SaclibQ( StrElm ) );
}

class SaclibQSaclibZP {
   private:
      int ConvertStatus;
   public:
      SaclibQSaclibZP() {}
      ~SaclibQSaclibZP() {}

      inline SaclibZP operator () ( SaclibQ& );
      inline SaclibQ operator () ( SaclibZP& );
      inline int IsConverted() { return ConvertStatus; }
};

inline SaclibZP SaclibQSaclibZP::operator () (
   SaclibQ& QElm
   )
{
   Word RN,MN,MD;
   MP_INT  MPOne,MPZero;
   SaclibZP ZPElm;

   mpz_init_set_ui( &MPOne, 1 );
   mpz_init_set_ui( &MPZero, 0 );
   RN = QElm.Native();
   MN = MIHOM( SACLIB_SLCard, RNNUM( RN ) ); 
   MD = MIHOM( SACLIB_SLCard, RNDEN( RN ) );

   // check for zero denominator
   if ( MD != 0 ) {
      ZPElm = ( SaclibZP( (MP_INT*) &MPOne) / SaclibZP( (Word)MD ) ) 
                * SaclibZP( (Word)MN );
      ConvertStatus = BBDEFS_TRUE;
   }
   else {
      ZPElm = SaclibZP( (MP_INT*) &MPZero );
      ConvertStatus = BBDEFS_FALSE;
   }

   mpz_clear( &MPOne );
   mpz_clear( &MPZero );

   return ZPElm;
}

inline SaclibQ SaclibQSaclibZP::operator () (
   SaclibZP& ZPElm
   )
{
   Word ZP;
   ZP = ZPElm.Native(); 
   return SaclibQ( (Word)RNINT(ZP) );
}

#endif
