/*-------------------------------------------------------------
  module: mod.h

  author: Angel L. Diaz
  date: 10/3/96

  mapping function objects for generating modular black boxes
-------------------------------------------------------------*/

#ifndef _MOD
#define _MOD
#define MOD_KSTRZ 500
#define MOD_PSTRZ 10000

#include "PlugIns/saclib.h"
#include "PlugIns/ntl2.0.h"
#include <string.h>

class SaclibQNTLzz_p {
   private:
      int ConvertStatus;
   public:
      SaclibQNTLzz_p() {}
      ~SaclibQNTLzz_p() {}

      inline NTLzz_p operator () ( SaclibQ& );
      inline SaclibQ operator () ( NTLzz_p& );

      inline int IsConverted() { return ConvertStatus; }
};

inline NTLzz_p SaclibQNTLzz_p::operator () (
   SaclibQ& QElm
   )
{
   char    StrNumElm[MOD_KSTRZ];
   char    StrDenomElm[MOD_KSTRZ];
   MP_INT  MPNumElm;
   MP_INT  MPDenomElm;
   MP_INT  MPOne,MPZero;
   NTLzz_p ZPNumElm;
   NTLzz_p ZPDenomElm;
   NTLzz_p ZPElm;

   QElm.NumtoString( StrNumElm );
   QElm.DenomtoString( StrDenomElm );

   mpz_init_set_ui( &MPOne, 1 );
   mpz_init_set_ui( &MPZero, 0 );
   mpz_init_set_str( &MPNumElm, StrNumElm , 10 );
   mpz_init_set_str( &MPDenomElm, StrDenomElm , 10 );

   ZPNumElm   = NTLzz_p( (MP_INT*) &MPNumElm );
   ZPDenomElm = NTLzz_p( (MP_INT*) &MPDenomElm );

   // check for zero denominator
   if ( ZPDenomElm != NTLzz_p( (MP_INT*) &MPZero ) ) {
      ZPElm = ( NTLzz_p( (MP_INT*) &MPOne) / ZPDenomElm ) * ZPNumElm;
      ConvertStatus = BBDEFS_TRUE;
   }
   else {
      cout << "zero denominator" << endl;
      exit(0);
      ZPElm = NTLzz_p( (MP_INT*) &MPZero );
      ConvertStatus = BBDEFS_FALSE;
   }

   mpz_clear( &MPOne );
   mpz_clear( &MPZero );
   mpz_clear( &MPNumElm );
   mpz_clear( &MPDenomElm );

   return ZPElm;
} 

inline SaclibQ
SaclibQNTLzz_p::operator () ( NTLzz_p& )
{
   cout << "SaclibQNTLzz_p::operator (): not implemented!" << endl;
   exit(0);
}


#endif
