find_parfrac_coeffs := proc(ufactors, uexpos)
   local r, i, m, d,
         h0s, gi0, hi0,
         si0,ti0,
         hat_gi0m, hat_hi0m,
         fprod,numeri,
         parfrac;

   if ( printlevel > 2 ) then
    printf(`find_parfrac_coeffs : Starting\n`);
   fi;

   r := nops(ufactors);
 
   if ( printlevel > 2 ) then
    printf(`find_parfrac_coeffs : r %d\n`,r);
    for i from 1 to r do
     printf(`find_parfrac_coeffs : ufactors[%d] = %a`,i,ufactors[i]); 
     printf(`\n`);
    od;
   fi;

   # array of univariate co-factors
   h0s := array[2..r];
   fprod := ufactors[r];
   h0s[r] := ufactors[r];
   for i from r-1 to 2 by -1 do
       fprod := expand(fprod * ufactors[i]);
       h0s[i] := fprod;
   od;

   if ( printlevel > 2 ) then
    for i from 2 to r do
     printf(`find_parfrac_coeffs : h0s[%d] = %a\n`,i,h0s[i]); 
    od;
   fi;    
   
   
   d := degree(ufactors[1], X) + degree(h0s[2], X);

   if ( printlevel > 2 ) then
    printf(`find_parfrac_coeffs : d = %d\n`,d);
   fi;

   parfrac = array(0..d-1, 1..r);

   if ( printlevel > 2 ) then
    printf(`find_parfrac_coeffs : calculating parfrac\n`);
   fi;
 
   si0 := array[1..r-1]; ti0 := array[1..r-1];
   for i from 1 to r-1 do
     gi0 := ufactors[i]; 
     hi0 := h0s[i+1];
     gcdex(gi0, hi0, 1, X, 'si0[i]','ti0[i]');
   od;

   for m from 0 to d-1 do
       numeri := X^m;
        for i from 1 to r-1 do
            gi0 := ufactors[i];
            hi0 := h0s[i+1];
            hat_gi0m := rem(ti0[i]*numeri,gi0,X);
            hat_hi0m := rem(si0[i]*numeri,hi0,X);
            if printlevel > 2 then
               printf(`\n*** m = %d ; i = %d ***\n`,m,i);
               printf(`find_parfrac_coeffs : gi0 %a\n`,gi0);
               printf(`find_parfrac_coeffs : hi0 %a\n`,hi0);
               printf(`find_parfrac_coeffs : hat_gi0m %a\n`,hat_gi0m);
               printf(`find_parfrac_coeffs : hat_hi0m %a\n`,hat_hi0m);
            fi;
            parfrac[m, i] := hat_gi0m/uexpos[i];
            if i = r-1 then
               parfrac[m, r] := hat_hi0m/uexpos[r];
            else 
               numeri := hat_hi0m;
            fi;
        od;
   od;

   if ( printlevel > 2 ) then
    for m from 0 to d-1 do
     for i from 1 to r do
      printf(`parfrac[%d,%d]=%a ; `,m,i,parfrac[m,i]);  
     od;
    printf(`\n`);
    od;
   fi;

   RETURN(eval(parfrac));
end;



lift := proc(fbar, g0, e, parfrac_coeffs, prod_g0_sup_e_minus_1)
   # fbar is a poly in X and Y, lc[Y](fbar) in K
   # g0 is a list of true factor images of fbar(X,0)
   # e is a list of corresponding multiplicities
   # parfrac_coeffs prestores the diophantine solutions
   # prod_g0_sup_e_minus_1 = prod(g0[i], i=1..r)

   local i, ii, j, k, m,   # loop variables
         r,                # number of factors
         fhat,
         g, ghat,
         tk, uk,
         d, dmax;

   if ( printlevel > 2 ) then
    printf(`lift                : Starting\n`);
   fi;
 
   r := nops(g0);
   d := array(1..r); dmax := 0; dsum := 0;
   for i from 1 to r do
       d[i] := degree(g0[i], X); dsum := dsum + d[i];
       if dmax < d[i] then dmax := d[i]; fi;
   od;

   dmax := degree(fbar);

   if ( printlevel > 2 ) then
    printf(`lift                : r %d\n`,r);
    printf(`lift                : dmax %d\n`,dmax);
    printf(`lift                : dsum %d\n`,dsum);
    for i from 1 to r do
     printf(`d[%d] =%a \n`,i,d[i]); 
    od;
   fi;

   # Y-adic coefficients of right side
   fhat := array(0..dmax);
   for i from 0 to dmax do
     fhat[i] := coeff(fbar, Y, i);
   od;

   if ( printlevel > 2 ) then
    printf(`lift                : Y-adic coefficients\n`); 
    for i from 0 to dmax do
     printf(`fhat[%d] =%a \n`,i,fhat[i]);
    od;
   fi;


   # just for purpose of debugging
   g := array[1..r, 0..dmax];
   ghat := array[1..r, 1..dmax];
   for i from 1 to r do g[i,0] := g0[i]; od;

   if ( printlevel > 2 ) then
    for m from 0 to dmax do
     for i from 1 to r do
      printf(`g[%d,%d]=%a ; `,i,m,g[i,m]);
     od;
    printf(`\n`);
    od;
   fi;

   # lift g0[i] from Y^(k-1) to Y^k
   for k from 1 to dmax do 
       tmp := coeff( expand(product(g[j,k-1]^e[j],j=1..r)), Y , k ); 
       tk := fhat[k] - tmp;
       if ( printlevel > 2 ) then
         printf(`lift                : *** k = %d ***\n`,k);
         printf(`lift                : tmp = %a\n`,tmp);
         printf(`lift                : fhat[%d] = %a\n`,k,fhat[k]);
         printf(`lift                : tk = %a\n`,tk);
       fi;

       if not(divide(tk, prod_g0_sup_e_minus_1, 'uk'))
          then ERROR(`untrue univariate factorization`);
       fi;

       for i from 1 to r do
           m := 'm';
           ghat[i,k] := sum(coeff(uk, X, m) * parfrac_coeffs[m,i], m=0..dsum-1);
           if ( printlevel > 2 ) then
             printf(`lift                : ghat[%d,%d] = %a\n`,i,k,ghat[i,k]);
           fi;

           g[i,k] := g[i,k-1] + Y^k * ghat[i,k];

           if ( printlevel > 2 ) then
             printf(`lift                : g[%d,%d] = %a\n`,i,k,g[i,k]);
           fi;
       od;
   od;

   if ( printlevel > 2 ) then
    for m from 0 to dmax do
     for i from 1 to r do
      printf(`g[%d,%d]=%a ; `,i,m,g[i,m]);
     od;
    printf(`\n`);
    od;
   fi;

RETURN([seq(g[ii,dmax], ii=1..r)]);
end;
