> ?dinterp
   
FUNCTION: dinterp - probabilistic degree interpolation
   
CALLING SEQUENCE:
   dinterp(f, n, k, d, p)
   
PARAMETERS:
   f       - a Maple procedure
   n, k, d - integers
   p       - prime modulus
   
SYNOPSIS:   
- Given a function f that evaluates a polynomial in n variables modulo p, and a
  degree bound d on the kth variable, determine probabilistically the degree of
  the kth variable.
   
- The dinterp function may return FAIL if it encounters a division by zero when
  evaluating f.  It may also return a result for the degree of the kth variable
  which is too low.  The probability that this happens can be decreased by
  using a larger modulus.  A 12 to 20 digit modulus is considered ideal.
   
- This function should be defined by the command readlib(dinterp) before it is
  used.   
   
EXAMPLES:   
> readlib(dinterp):
> f := proc(x,y,z,p) x^2+y^3+z^4 mod p end:
> dinterp(f,3,1,6,997);  
                                       2
   
> dinterp(f,3,2,6,997);  
                                       3
   
> dinterp(f,3,3,6,997);  
                                       4
   

>  interface(verboseproc=2);
> readlib(`dinterp`);

proc(g,n,k,d,p)
local a,b,e,f,i,r,t,v,x,y,minterp;
options `Copyright 1993 by Waterloo Maple Software`;
    if not type(g,procedure) and type(n,integer) and type(k,integer) and
        type(d,integer) and type(p,integer) and 1 < p then
        ERROR(`invalid arguments`)
    fi;
    minterp :=
      proc(x,y,k,r,c,z,p)
      local i,u,t;
      options `Copyright 1993 by Waterloo Maple Software`;
          if 0 < k then
              t := irem(x[k]-x[0],p);
              for i to k-1 do  t := irem(t*(x[k]-x[i]),p) od;
              r[k] := modp(1/t,p)
          fi;
          if k = 0 then c[0] := y[0]
          else
              t := c[k-1];
              for i from k-2 by -1 to 0 do  t := irem(t*(x[k]-x[i])+c[i],p) od;
              c[k] := irem((y[k]-t)*r[k],p)
          fi;
          u := c[k];
          for i from k-1 by -1 to 0 do  u := irem(u*(z-x[i])+c[i],p) od;
          u mod p
      end
      ;
    t := rand(p);
    a := ['t()' $ n];
    x := t();
    y := traperror(g(op(subsop(k = x,a)),p));
    if y = lasterror then RETURN(FAIL) fi;
    for i from 0 to d do
        userinfo(2,dinterp,'degree',i);
        userinfo(2,dinterp,'degree',i);
        e[i] := x;
        f[i] := y;
        x := t();
        y := traperror(g(op(subsop(k = x,a)),p));
        b := minterp(e,f,i,r,v,x,p);
        if y = lasterror then RETURN(FAIL) fi;
        if y = b then RETURN(i) fi
    od;
    FAIL
end

