read `bbpolygen.mpl`;
read `pade.mpl`;

NumDen := proc(bb,n,eb,r)
# bb a black box that evaluates f(x1,...,xn)/g(x1,....,xn) in K(x1,...,xn)
#    where K is a field of characterisic 0, f,g in K[x1,...,xn], 
#    GCD(f,g) = 1. The black box indicates when evaluated at a root of g.
# n  number of variables
# eb a bound on the deg(g)
# r  cardinality from which to select random elements
local i,    # loop counter
      a,b,  # random field elements
      rnd,  # random number generator
      pnts, #
      tpnts,#
      deg,  # choices of degree for f and g
      d,e,  # degree of f and g respectively
      relm, # random field elment
      ans,  # array that contains candidates for f and g
      fcfs, #
      lc,   #
      bbnd; #

      rnd  := rand(1..r);
      a    := array(2..n);
      b    := array(2..n);
      pnts := array(1..n);
      ans  := array(1..2);

      # select random field elements
      for i from 2 to n do 
        a[i] := rnd(); b[i] := rnd();
      od;

      pnts := [ X ];
      for i from 2 to n do
        pnts := [ op(pnts), a[i]*X + b[i] ];
      od;

      # probabilistically compute the degress of f and g
      deg := 0; lc := 1;
      while (true) do

        d := deg; e := deg;
        printf(`Working on deg %a`,deg);
        printf(`d = %a e = %a \n`,d,e);
        ans := Pade(bb,d,e,pnts,n,lc);
        # check candiates for f and g
        if ( ans[2] <> 0 ) then
          relm := rnd();
          tpnts := [ relm ];
          for i from 2 to n do
            tpnts := [ op(tpnts), a[i]*relm + b[i] ];
          od;

          for i from 1 to n do
            printf(`rndpnts[%d] = %a\n`,i,tpnts[i]);
          od;

          # must add a check for infinity 
          if ( bb(tpnts) = subs( X=relm, ans[1]/ans[2] ) ) then
             d := degree(ans[1]);
             e := degree(ans[2]);
             break;
          fi;
        fi;

        # try a new degree
        deg := deg + 1;
        if ( deg > eb ) then
          ERROR(`A new random projection must be tried, deg > eb`);
        fi;
      od;

      fcfs := [ denom(coeff(ans[1],X,0)) ];
      for i from 1 to nops(ans[1]) do
        fcfs := [ op(fcfs), denom(coeff(ans[1],X,i)) ];
      od;
      lc := lcm(op(fcfs));
      ans[1] := ans[1]*lc; ans[2] := ans[2]*lc;
    
      if ( printlevel > 2) then
        for i from 2 to n do
          printf(`a[%d] is %a     b[%d] is %a \n`,i,a[i],i,b[i]);
        od;
        printf(`pnts are %a\n`,pnts); 
        printf(`fcfs %a \n`,fcfs);
        printf(`lc %a\n`,lc);
        printf(`fh(X,0) is %a\n`,ans[1]);
        printf(`gh(X,0) is %a\n`,ans[2]);
        printf(`fh(X,0)/gh(X,0) is %a\n`,simplify(ans[1]/ans[2]));
      fi;

      # construct the black box program


      bbnd := `&proc`([], [_bb,_n,_a,_b,_d,_e,_fb0,_gb0,_lc], [] ,
                        `&statseq`(
                                   `&:=`(`&local`[1], eval(bb)),
                                   `&:=`(`&local`[2], n),
                                   `&:=`(`&local`[3], eval(a)),
                                   `&:=`(`&local`[4], eval(b)),
                                   `&:=`(`&local`[5], d),
                                   `&:=`(`&local`[6], e),
                                   `&:=`(`&local`[7], ans[1]),
                                   `&:=`(`&local`[8], ans[2]),
                                   `&:=`(`&local`[9], lc),
                                   `&RETURN`('NumDenAux(eval(`&local`[1]),
                                   `&local`[2],`&local`[3],
                                   `&local`[4],
                                   `&local`[5],`&local`[6],`&local`[7],
                                    `&local`[8],`&local`[9],`&args`[-1])')
                                  )
                      );

 RETURN(procmake(bbnd));

end:

NumDenAux := proc(bb,n,a,b,d,e,fb0,gb0,lc,p)
local i,    # loop variable
      j,    # correct polynomial counter
      I,    # values of Y 
      fblst,# list of fb(X,j1), ..., fb(X,jx) 
      gblst,# list od fb(X,j1), ..., gb(X,jx)
      J,    # list of Y's j1,....,jx
      pnts, #
      tpnts,#
      dep,  #
      fb,gb,#
      ans;  # array that contains candidates for f and g

      # construct fb(X,Y) = f(X,a2*X+b2 + Y(p2 - a2p1 - b1),.... )
      # construct gb(X,Y) = g(X,a2*X+b2 + Y(p2 - a2p1 - b1),.... )
      j := 0; I := 1;
      ans := array(1..2);
      fblst := [ fb0 ]; gblst := [ gb0 ]; J := [ 0 ];
  
      pnts := [ X ];
      for i from 2 to n do
        pnts := [ op(pnts), a[i]*X + b[i] + Y*(p[i] - a[i]*p[1] - b[i]) ];
      od;

      while (true) do;
       
        tpnts := [ X ];
        for i from 2 to n do
          tpnts := [ op(tpnts), subs(Y=I,pnts[i]) ];
        od;
        ans := Pade(bb,d,e,tpnts,n,lc);
        if ( degree(ans[1]) >= d and degree(ans[2]) >= e ) then
             J := [ op(J), I ];
             fblst := [ op(fblst), ans[1] ];
             gblst := [ op(gblst), ans[2] ];
             j := j + 1;
        fi;
        if j = (max(d,e)+1) then break; fi;
        if I = 2*d*e then 
          ERROR(`invalid black box, recompute`);
        fi;
        I := I + 1;
      od; 

      # interpolate fb(X,Y);
      fb := 0;
      for i from 0 to nops(J) do
        dep := [];
        for j from 1 to nops(fblst) do
          dep := [ op(dep), coeff(fblst[j],X,i) ];
        od;
        fb := fb + interp(J,dep,Y)*X^i;
      od;

      # interpolate gb(X,Y)
      gb := 0;
      for i from 0 to nops(J) do
        dep := [];
        for j from 1 to nops(gblst) do
          dep := [ op(dep), coeff(gblst[j],X,i) ];
          printf(`DEP = %a \n`,coeff(gblst[j],X,i));
        od;
        printf(`interp(J,dep,Y) = %a\n`,interp(J,dep,Y));
        gb := gb + interp(J,dep,Y)*X^i;
      od;
 
      if (printlevel >2) then
        printf(`fblst %a \n`,fblst);
        printf(`gblst %a \n`,gblst);
        printf(`J is %a\n`,J);
        printf(`gb is %a \n`,simplify(gb));
        printf(`fb is %a \n`,simplify(fb));
      fi;

      ans[1] := subs(X=p[1],Y=1,fb); 
      ans[2] := subs(X=p[1],Y=1,gb);
  RETURN(ans); 
end:

