##############################################################################
#                                                                            #
#                                 bbgcd.mpl                                  #
#                                                                            #
##############################################################################

##############################################################################
#                                                                            #
# functions:                                                                 #
#      inter           - compute the polynomial given a bivariate bb and pnt #
#      bbpolygen       - create a black box polynomial evaluation procedure  #
#      bbbargen        - create a black box polynomial of the bar operator   #
#      param           - auxilluary function used by the generated bar bb    #
#      bbgcdgen        - create a black box polynomial(s) gcd eval procedure #
#      bbgcdaux        - compute the gcd of the bb's in polylist             #
#                                                                            #
##############################################################################

##############################################################################
#                                                                            #
#                              load packages                                 #
#                                                                            #
##############################################################################
with (student):
with (linalg):
readlib(procmake):
readlib(procbody):
intdeg := 0;

#

##############################################################################
#                                                                            #
# subroutine: inter                                                          #
#                                                                            #
# inputs:  bb               - black box of a polynomial (point generator)    #
#          var              - variable to be used in polynomial              #
#          con              - value of second variable                       #
#          calc             - calcuate static information flag 1=yes         #
#          fgen             - random field element generator                 #
#          flg              - use deg flag 1=yes                             #
#          deg              - deg of poly to stop at (only if flg=1)         #
#                                                                            #
# globals: recip            - the static required inverses                   #
#          basis            - basis used for conversion to normal form       #
#          intdeg           - number of satic information kept               #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# debug:  none                                                               #
#                                                                            #
# return:   the univariate polynomial                                        #
#                                                                            #
# purpose:  compute the polynomial given a bivariate bb and pnt              #
#                                                                            #
##############################################################################

inter := proc(bb,var,con,calc,fegen,flg,deg)
local i,        # loop counter
      k,        # loop counter
      n,        # degree of interpolated polynomial 
      temp,     # temporary variable
      prod,     # product of difference of points used in inverse calculation
      v,        # newton coefficients     
      u,        # interpolated polynomial
      oldn,     # previouse degree
      A,        # random field element
      bbA,      # black box evaluated at random field element
      flag;     # stop loop condition

global recip,basis,intdeg;

 if (printlevel > 1) then 
  print(`inter      : enter proc() inter`);
  print(`inter      : variable to be used in polynomial `,var);
  print(`inter      : value of second variable `,con);
  print(`inter      : calculate static information flag 1=yes `,calc);  
  print(`inter      : random field element generator `,eval(fegen));
  print(`inter      : stop at deg flag 1=yes `,flg);
  print(`inter      : stopping degree `,deg);
 fi;

      flag := true;
      n    := 1;
      A    := fegen();
      bbA  := bb(A,con);

  if (printlevel > 2) then
   print(`inter      : random field element chosen `,A); 
   print(`inter      : black box evaluated at random field element `,bbA);
  fi;

      while ( flag ) do

       if ( flg = 0 ) then  
         flag := evalb(bbA <> subs(var=A,u));
       else
         flag := evalb(n<deg);
       fi;

  if (printlevel > 2) then 
   print(`inter      : degree `,n); 
  fi;

        oldn := n;
        #                               #
        # Compute the required inverses #
        #                               #
        if ( ( calc = 1 ) or (intdeg < n) )then
          for k from 1 to n do
            prod := k-0;
            for i from 1 to k-1 do  
              prod := prod*(k-i) 
            od;
            if prod = 0 then 
              ERROR(`the same point has been entered twice`) 
            fi;
            recip[k] := 1/prod;
          od;
        fi;

        #                                 #
        # Compute the Newton coefficients #
        #                                 #
        v[0] := bb(0,con);
        for k from oldn to n do
          temp := v[k-1];
          for i from k-2 by -1 to 0 do  
            temp := temp*(k-i)+v[i] 
          od;
          v[k] := (bb(k,con)-temp)*recip[k];
        od;

        #                                      #
        # Convert from newton to standard form #
        #                                      #
        if ( ( calc = 1 )  or (intdeg < n) ) then
          basis[0] := 1;
          for k from oldn to n do
            basis[k] := basis[k-1]*(var-(k-1));
          od;
        fi;
        u := 0;
        for k from 0 to n do
          u := u + v[k]*basis[k];
        od;
        if nops(indets(u)) = 1 then
          u := expand(u)
        else
          u := collect(normal(u),var)
        fi;
        n := n + 1;
      od;

      if ( intdeg < n ) then intdeg := n-1; fi;

 if (printlevel > 1) then
  print(`inter      : exit proc() inter`);
 fi;

 RETURN(u);
end;

#

##############################################################################
#                                                                            #
# subroutine: bbpolygen                                                      #
#                                                                            #
# inputs:  poly             - polynomial in n variables                      #
#                                                                            #
# globals: none                                                              #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# debug:  none                                                               #
#                                                                            #
# return:  a black box polynomial evaluation procedure                       #
#                                                                            #
# purpose: maple procedure to create a black box polynomial evaluation       #
#          procedure                                                         #
#                                                                            #
##############################################################################

bbpolygen := proc(poly)
local box,      # neutral form of black box procedure 
      varlist,  # list of indeterminants of the polynomial
      varlen;   # length of the list of indeterminants

      varlist := convert(indets(poly),list);
      varlen  := nops(varlist);

 if (printlevel > 1) then
  print(`bbpolygen  : enter proc() bbpolygen`);
  print(`bbpolygen  : polynomial to be converted to a black box ` ,poly);
  print(`bbpolyge   : list of indeterminants of the polynomial `,varlist);
  print(`bbpolyge   : length of the list of indeterminants `,varlen);
 fi;

       box := `&proc`([], [p,i,n,vl] , [] ,
                        `&statseq`(
                                    `&:=`(`&local`[1], poly),
                                    `&:=`(`&local`[3], varlen),
                                    `&:=`(`&local`[4], varlist),
                                    `&:=`(`&local`[1],
                                       'subs(seq(vl[`&local`[2]]=`&args`[-1]
                                             [`&local`[2]],
                                             `&local`[2]=1..`&local`[3]),p)'), 
                                    `&RETURN`(`&local`[1])
                                  )
                     );

 if (printlevel > 1) then
  print(`bbpolygen  :  exit proc() inter`);
 fi;

 RETURN(procmake(box));
end;

#

##############################################################################
#                                                                            #
# subroutine: bbdmgen                                                        #
#                                                                            #
# inputs:  poly             - matrix                                         #
#          vars             - indeterminants in matrix                       #
#                                                                            #
# globals: none                                                              #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# debug:  none                                                               #
#                                                                            #
# return:  a black box matrix determinant evaluation procedure               #
#                                                                            #
# purpose: maple procedure to create a black box matrix determinant          #
#          evaluation procedure                                              #
#                                                                            #
##############################################################################

bbdmgen := proc(A,vars)
local dmbb;     # neutral form of the matrix determinant black box

 if (printlevel > 1) then
  print(`bbdmgen    : enter proc() bbdmgen`);
  print(`bbdmgen    : matrix `,A);
  print(`bbdmgen    : indeterminants in the matrix `,vars);
 fi;

       dmbb := `&proc`([], [i,AA,p] , [] ,
                         `&statseq`(
                                    `&:=`(`&local`[2],A),
                                    `&:=`(`&local`[3],vars),
                                    `&:=`(`&local`[2],
                                    'subs(seq(p[`&local`[1]]=`&args`[-1]
                                             [`&local`[1]],
                                             `&local`[1]=1..`&args`[0]),op(AA)
                                         )'),  
                                    `&RETURN`('det(op(`&local`[2]))')
                                   )
                      );

 if (printlevel > 1) then
  print(`bbdmgen    : exit proc() bbdmgen `);
 fi;

  RETURN(procmake(dmbb));
end;

#

##############################################################################
#                                                                            #
# subroutine: bbbargen                                                       #
#                                                                            #
# inputs:  bb     - a black box representation of a polynomial               #
#          randa  - random field elements                                    #
#          randb  - random field elements                                    #
#          plist  - symbolic place holders for the evaluation points         #
#                                                                            #
# globals: none                                                              #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# debug:   none                                                              #
#                                                                            #
# return:  a black box polynomial evaluation procedure                       #
#                                                                            #
# purpose: maple procedure to create a black box polynomial evaluation       #
#          procedure for the bar operator                                    #
#                                                                            #
##############################################################################

bbbargen := proc(bb,randa,randb,plist)
local i,        # loop counter
      j,        # loop counter
      barbox,   # neutral form of the bar black box procedure
      arr,      # evaluation array
      lst,      # evaluation list
      no;       # number of elements in the evaluation list/array

 if (printlevel > 1) then
  print(`bbbargen   : enter proc() bbbargen`);
 fi;
 if (printlevel > 2) then
  print(`bbbargen   : an random elements `,op(randa));
  print(`bbbargen   : bn random elements `,op(randb));
  print(`bbbargen   : point list `,op(plist)); 
 fi;
      #                              #
      # compute the evaluation array #
      #                              #
      no :=  nops(plist);
      arr[1] := X;

      for i from 2 to no do
        arr[i] := Y*(plist[i] - randa[i]*plist[1] - randb[i])
                  + randa[i]*X + randb[i];
      od;

      lst := [ 'arr[j]' $ j=1..no ];

 if (printlevel > 1) then
  print(`bbbargen   : evaluation list`,lst);
 fi;

      barbox := `&proc`([], [para,ii,plen,a], [],
                   `&statseq`( `&:=`(`&local`[1],lst),
                               `&:=`(`&local`[3],'nops(`&local`[1])'),
                               `&:=`(`&local`[4],'table()'),
                                   `&for`(`&local`[2],
                                           1,
                                           1,
                                           `&local`[3],
                                           true,
                                           `&:=`(`&local`[4][`&local`[2]],
                                            'subs(X=`&args`[1],
                                                  Y=`&args`[2],
                                             `&local`[1][`&local`[2]])'
                                                 )
                                          ),
                               `&RETURN`
                                ('bb(op(param(`&local`[4],`&local`[2])))')
                             )
                       );

 if (printlevel > 1) then
  print(`bbbargen   : exit proc() bbbargen`);
 fi;

 RETURN(procmake(barbox));
end;

param := proc(a,no) 
 local j; 
  RETURN([ 'a[j]' $ j=1..no-1 ]); 
end;

#

##############################################################################
#                                                                            #
# subroutine: bbgcdgen                                                       #
#                                                                            #
# inputs:  args[1]          - list of polynomial black boxes                 #
#          args[2]          - list of n symbolic points to evaluate poly     #
#          args[3]          - procedure to generate random field elements    #
#          args[4]          - proc to gen rnd field elms for inter           #
#                                                                            #
# globals: none                                                              #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# debug:  printlevel > 1    - procedure milestone steps                      #
#         printlevel > 2    - random field elements                          #
#                             f1bar(X,0) and f0bar(X,0)                      #
#                                                                            #
# return:  a procedure that computes the gcd of the polynomials in args[1]   #
#                                                                            #
# purpose: compute the black box gcd procedure for r polynomials in n        #
#          variables                                                         #
#                                                                            #
##############################################################################

bbgcdgen  := proc()
 local polylist,      # list of black boxes
       i,             # loop iterator 
       plist,         # symbolic field elements to evaluate gcd of
       nplist,        # number of field elements
       dlist,         # degree of each black box
       fgen,          # random field element generator
       fegen,         # random field element generator for interpolation
       randa,         # random field elms a2 .. an
       randb,         # random field elms b2 .. bn
       randc,         # random field elms c2 .. cn
       r,             # number of polynomials
       f0bar,         # f0bar
       f1bar,         # f1bar 
       gbar,          # gcd of f0bar and f1bar
       tmpp,          # temporary polynomial
       bbgcd;         # polynomial gcd black box procedure
 
 if (printlevel > 1) then 
  print(`bbgcdgen   : enter proc() bbgcdgen`); 
 fi;

      #                  #
      # parse parameters #
      #                  #
      polylist := args[1];
      r        := nops(polylist);
      plist    := args[2];
      fgen     := args[3];
      fegen    := args[4];

 if (printlevel > 1) then
  print(`bbgcdgen   : black box list `,polylist);
  print(`bbgcdgen   : number of black boxes `,r);
  print(`bbgcdgen   : symbolic points `,plist);
  print(`bbgcdgen   : random field element function `,eval(fgen));
  print(`bbgcdgen   : random field element function for inter`,eval(fegen));
  print(`bbgcdgen   : filling up random field elements`); 
 fi;

      #                               #
      # fill up random field elements #
      #                               #
      nplist := nops(plist); 
      randa := array(1..nplist); randa[1]:=0;
      randb := array(1..nplist); randb[1]:=0;
      randc := array(1..r); randc[1]:=0;
 
      for i from 2 to nplist do
        randa[i] := fgen();
        randb[i] := fgen();

 if (printlevel > 2) then 
  print(`bbgcdgen   : randa[`.i.`] = `,randa[i],`randb[`.i.`] = `,randb[i]);
 fi;

      od;

      for i from 2 to r do
        randc[i] := fgen();

 if (printlevel > 2) then
  print(`bbgcdgen   : randc[`.i.`] = `,randc[i]);
 fi;

      od;

      dlist := array(1..nops(polylist)); 

 if (printlevel > 1) then 
  print(`bbgcdgen   : computing f1bar(X,0)`); 
 fi;

      #                    #
      # compute f1bar(X,0) #
      #                    #
      f1bar := inter(bbbargen(polylist[1],randa,randb,plist),X,0,1,eval(fegen),0,0);
      dlist[1] := degree(f1bar,X);
 if (printlevel > 2) then 
  print(`bbgcdgen   : f1bar(X,0) = `,simplify(f1bar));
 fi;

 if (printlevel > 1) then 
  print(`bbgcdgen   : computing f0bar(X,0)`); 
 fi;

      #                    #
      # compute f0bar(X,0) #
      #                    #
      f0bar :=0;
      for i from 2 to r do
        tmpp := inter(bbbargen(polylist[i],randa,randb,plist),X,0,0,eval(fegen),0,0);
        f0bar := f0bar + randc[i]*tmpp; 
        dlist[i] := degree(tmpp,X);
     od;

 if (printlevel > 2) then 
  print(`bbgcdgen   : f0bar(X,0) = `,simplify(f0bar));
  for i from 1 to r do
  print(`bbgcdgen   : Black Box `,i,` has degree `,dlist[i]); 
  od;
 fi;

 if (printlevel>1) then 
  print(`bbgcdgen   : computing gcd of f1bar(X,0) and  f0bar(X,0)`); 
 fi;

       #                                           #
       # compute gcd of f1bar(X,0) and  f0bar(X,0) #
       #                                           #
       gbar := gcd(f0bar,f1bar);

 if (printlevel > 2) then 
  print(`bbgcdgen   : gcd is `,gbar); 
 fi;

 if (printlevel > 1) then 
  print(`bbgcdgen   : generating gcd black box procedure`); 
 fi;

      bbgcd := `&proc`([], [pol,ra,rb,rc,dl,gb,dgb,ffegen], [] ,
                        `&statseq`(
                                   `&:=`(`&local`[1], polylist),
                                   `&:=`(`&local`[2], convert(randa, list)),
                                   `&:=`(`&local`[3], convert(randb, list)),
                                   `&:=`(`&local`[4], convert(randc, list)),
                                   `&:=`(`&local`[5], convert(dlist, list)),
                                   `&:=`(`&local`[6], gbar),
                                   `&:=`(`&local`[7], degree(gbar,X)),
                                   `&:=`(`&local`[8], eval(fegen)),
                                   `&RETURN`('bbgcdaux(`&local`[1],`&local`[2],
                                   `&local`[3],`&local`[4],[`&args`[-1]],
                                   `&local`[5],`&local`[6],`&local`[7],
                                    `&local`[8])')
                                  )
                      );

 if (printlevel > 1) then 
  print(`bbgcdgen   : exit proc() bbgcdgen`); 
 fi;

 RETURN(procmake(bbgcd));
end;

#

##############################################################################
#                                                                            #
# subroutine: bbgcdaux                                                       #
#                                                                            #
# inputs:  polylist         - list of black boxes                            #
#          randa            - random field elements                          #
#          randb            - random field elements                          #
#          randc            - random field elements                          #
#          plist            - point to evaluate gcd at                       #
#          dl               - degree of black boxes                          #
#          g0               - gcd of f0bar(X,0) and f1bar(X,0)               #
#          delta            - degree(g0)                                     #  
#          fegen            - proc to gen rnd field elms for inter           #
#                                                                            #
# globals: none                                                              #
#                                                                            #
# outputs: none                                                              #
#                                                                            #
# debug:  printlevel > 1    - procedure milestone steps                      #
#         printlevel > 2    - f1bar(X,ii) and f0bar(X,ii)                    #
#                             gcd of f1bar(X,11) and f0bar(X,ii)             #
#                                                                            #
# return:  a procedure that computes the gcd of the bb's in polylist         #
#                                                                            #
# purpose: compute the gcd of the bb's in polylist                           #
#                                                                            #
##############################################################################

bbgcdaux := proc(polylist,randa,randb,randc,plist,dl,g0,delta,fegen)
local i,        # loop counter
      ii,       # sufficient gcd's calculated counter
      iii,      # gcdarray index
      j,        #
      r,        # number of black boxes
      f1bar,    # f1bar
      f0bar,    # f0bar
      numcoeffs,#
      gcdarray, # array of gcd's of f0bar(X,i=0..?) and f1bar(X,i=0..?)
      indep,    # independent values for interp
      dep,      # dependent values for interp
      g,        # final interpolated gcd
      dg,       # degree of the ith gcd of f0bar(X,i) and f1bar(X,i)
      lg0,      # leading coefficient of gcd of f0bar(X,0) and f1bar(X,1)
      max;      # maximum degree bound given in dl list

 if (printlevel > 1) then 
  print(`bbgcdaux   : enter proc() bbgcdaux`); 
 fi;

 if (printlevel > 2) then 
  print(`bbgcdaux   : polylist`,polylist);
  print(`bbgcdaux   : plist `,plist);
  for i from 2 to nops(plist) do
    print(`bbgcdgen   : randa[`.i.`] = `,randa[i],`randb[`.i.`] = `,randb[i]);
  od;
  for i from 2 to nops(polylist) do
    print(`randc[`.i.`] = `,randc[i]);
  od;
  for i from 1 to nops(polylist) do
    print(`black box `,i,` has degree `,dl[i]);
  od;
 fi;

      #                             #
      # compute constant parameters #
      #                             #
      r           := nops(polylist);
      gcdarray[0] := g0;
      lg0         := lcoeff(g0);
      ii          := 1;
      iii         := 1;

 if (printlevel > 2) then
  print(`bbgcdaux   : number of black boxes `,r);
  print(`bbgcdaux   : gcdarray[0] `,gcdarray[0]);
  print(`bbgcdaux   : lcoeff of gcdarray[0] `,lg0);
 fi;  

 if (printlevel > 1) then 
  print(`bbgcdaux   : computing gcd's of f0bar(X,1..?) and f1bar(X,1..?)`); 
 fi;

      while ( ii <= delta ) do

 if (printlevel > 1) then 
   print(`bbgcdaux   : sufficient gcd's counter `,ii); 
   print(`bbgcdaux   : gcdarray index           `,iii);
 fi;

      #                                       #
      # compute f0bar(X,iii) and f1bar(X,iii) #
      #                                       #
      f1bar:=inter(bbbargen(polylist[1],randa,randb,plist),
                   X,iii,0,fegen,1,dl[1]);

 if (printlevel > 2) then 
  print(`bbgcdaux   : f1bar(X,`.iii.`) = `,f1bar); 
  print(`bbgcdaux   : degree bound given `,dl[1]);
 fi;

      f0bar :=0;
      max := 0;
      for i from 2 to r do
        f0bar := f0bar +
        randc[i]*
        inter(bbbargen(polylist[i],randa,randb,plist),X,iii,0,fegen,1,dl[i]);

 if (printlevel > 2) then
  if (dl[i] > max) then max := dl[i]; fi;
 fi;

      od;

 if (printlevel > 2) then 
  print(`bbgcdaux   :f0bar(X,`.iii.`) = `,f0bar); 
  print(`bbgcdaux   : max degree bound given `,max);
 fi;

      #                                                  #
      # compute the gcd of f0bar(X,iii) and f1bar(X,iii) #
      #                                                  # 
      gcdarray[iii] := gcd(f0bar,f1bar);

 if (printlevel > 2) then
  print(`bbgcdaux   : w/o normalization gcdarray[`.iii.`] `,gcdarray[iii]);
 fi;

      gcdarray[iii] := (gcdarray[iii]/lcoeff(gcdarray[iii]))*lg0;

 if (printlevel > 2) then 
  print(`bbgcdaux   : gcdarray[`.iii.`] `,gcdarray[iii]); 
 fi;

      #                                            #
      # break if dg(gcd(f0bar(X,1),f1bar(X,1)))    #
      #           = dg(gcd(f0bar(X,0),f1bar(X,0))) #
      #                                            #
      dg := degree(gcdarray[iii]);

      if ( (iii = 1) and ( dg = delta) ) then

 if (printlevel > 1) then
  print(`bbgcdaux   : done at gcd of f0bar(X,1) and f1bar(X,1)`); 
  print(`bbgcdaux   : delta was `,delta);
 fi;

        RETURN(subs(X=plist[1],gcdarray[iii]));
      fi;

      #                            #
      # break on invalid black box #
      #                            #
      if ( dg < delta ) then 
        ERROR(`invalid black box`);
        RETURN(`invalid black box`);
      fi;

      #                      #
      # check for bad points #
      #                      #
      if ( dg > delta ) then
        print(`discarding point `.iii);
        gcdarray[iii] := 'bad';
        iii := iii + 1;
      else
        ii  := ii  + 1;
        iii := iii + 1;
      fi;
 
     od;

     #                                               #
     # interpolate coeffecients to calculate the gcd #
     #                                               #

 if (printlevel > 1) then
  print(`bbgcdaux   : interpolating gcd's`);
 fi;
 
     g := 0;
     for j from 0 to iii-1 do
       for i from 0 to iii-1 do
         if ( gcdarray[i] <> 'bad' ) then
           indep[i] := i;
           dep[i]   := coeff(gcdarray[i],x,j);
         fi;
       od;

 if (printlevel > 2) then
  print(`bbgcdaux   : gcdarray[`.j.`] = `,gcdarray[j]);
 fi;

       g := g + X^j*interp(convert(indep,list),convert(dep,list),Y);
     od;

 if (printlevel > 1) then
  print(`bbgcdaux   : done interpolating gcd's`); 
  print(`bbgcdaux   : gcd `,g);
  print(`bbgcdaux   : exit proc() bbgcdaux`);
 fi;

 RETURN(subs(X=plist[1],Y=1,g));

end;
