Pade := proc(bb,d,e,p,n,lc)
# bb   black box for f/g
# d,e  degree bounds
# p    linearly transformed points in varibale X
local J,
      i,k,
      b,
      I,
      A,
      h, 
      y,
      pnts,
      value,
      ans;

   # find a set of d+e+1 integers not roots of 
   # GHat(X) = g(X,u2*X+v2,...,un*X+vn) as well as 
   # values of f/g((X,u2*X+v2,...,un*X+vn) at those points
   b    := d+2*e+1;
   k    := 0;
   ans  := array(1..2);
   A    := []; I := []; 
 
   for J from 1 to b do
     # must check for invalid points
     pnts := [];
     for i from 1 to n do pnts := [ op(pnts), subs(X=J,p[i]) ]; od;
     value := bb(pnts);
     k := k + 1;
     A := [ op(A), value ];
     I := [ op(I), J ];
     if k = d+e+1 then break; fi;
   od;
   if k < d+e+1 then ans[1] := 0; ans[2] := 0; RETURN(ans); fi;

   h := interp(I,A,X);
   y := (X - I[1]);
   for i from 2 to k do 
     y := y*(X - I[i]);
   od;

   ans := SpecialExtGcd(h,y,d,lc);

   # for debugging purposes
   if ( ans[1] <> rem(ans[2]*h,y,X) )
    then ERROR(`incorrect FHat and GHat`);
   fi;

 RETURN(ans);
end:
 
#-----------------------------------------------------------------------
# function: SpecialExtGcd 
# date: 6/15/96
#
# reference: E. Kaltofen and B. Trager, "On Computing with 
#    Polynomials Given By Black Boxes for Their Evaluations: 
#    Greatest Common Divisors, Factorization, Seperation of 
#    Numerator and Denominators", J. Sym. Comp. (1990), 9/3,
#    pg. 315. 
#
# description: A version of the specialized extended Euclidean
#    algorithm found in the above reference. Given polynomials
#    H(X) and Y(X) and a degree bound d, compute polynomials 
#    FHat(X) and GHat(X) such that 
#          1) FHat(X) == GHat(X)*H(X) (mod Y(X))
#          2) GCD( FHat(X), GHat(X) ) = 1
#    and deg(FHat(X)) <= d, deg(GHat(X)) <= e, ldcf(GHat(X)) = 1
#
# return: 
#-----------------------------------------------------------------------

SpecialExtGcd := proc(
   H,      # H(X), deg(H(X)) <= d + e
   Y,      # Y(X) = (X - i_1) * (X - i_2) * ... (X - i_d+e+1)
   d,      # d = deg( F ), e = deg( G )
   lc 
   )
local  h0,h1,h2,
       g0,g1,g2,
       q,
       ans;

   h0  := Y;
   h1  := H;
   g0  := 0;
   g1  := 1;
   ans := array(1..2);

   while ( degree( h1 ) > d ) do
    h2 := rem( h0, h1, X );
    if not( divide( h0 - h2, h1 ,'q') )
       then ERROR(`division not exact`);
    fi;
    g2 := g0 - q*g1; 
    h0 := h1; h1 := h2;
    g0 := g1; g1 := g2;
  od;

  ans[1] := lc*(expand(h1)/lcoeff(expand(g1))); 
  ans[2] := lc*(expand(g1)/lcoeff(expand(g1)));

 RETURN(ans);

end:
