read `bbpolygen.mpl`:
read `vsolve.mpl`:
read `kaltofencore.mpl`;

kaltofen3 := proc(bb,ivars,degs,degf,r,m)
# bb    black box polynomial
# ivars list of input variables  [x1,...,xn]
# degs  exact degrees [deg(x1),...,deg(xn)]
# r     cardinality for random number generator
# all operations are in Z mod m
local  n,      # number of variables x1,...,xn
       vars,   # expanded set of variables
       d0,     # bound on all of the degrees [deg(x1),...,deg(xn)]
       ii,     # loop variables
       rnd,    # random number generator
       i,      # loop over the variables 1...n
       Ji,     # set of exponents e1,...,ei-1
       nJi,    # expanded set of exponents
       k,      # loop over the expanded Ji set 
       ji,     # cardinality of the expanded Ji set 
       a,      # initial random elements
       l,      # d_e1,...,ei-1 for each element of Ji 
       t,      # number of terms
       x,      # x1...x((d0+1)^n)*d0 solution
       incpoly,# polynomial of completed monomials
       pans,   #
       bbcnt,  # number of black box calls
       state;  # modified interpolation state


       # initialize variables
       state := array(1..4);
       bbcnt := 0; incpoly := 0; d0 := degs[1]; 
       n := nops(ivars); 
       d0 := degs[1]; 
       for i from 1 to n do
        if ( d0 < degs[i] ) then d0 := degs[i]; fi;
       od;
       n := n + 1;
       vars := array(1..n);
       vars[1] := `z`;
       for i from 2 to n do vars[i] := ivars[i-1] od; 

       rnd := rand(1..r);
 
       x := array(1..((d0+1)^n))*d0; # ((d0+1)^n)*d0 is an upper bound on ji

       t  := 1;
       Ji := [[]];
       l  := [];

       # select initial values a2,...,an
       a := array(2..n); 
       for i from 2 to n do a[i] := rnd(); od;

       if (printlevel>2) then 
         for ii from 2 to n do 
          printf(`a[%d]=%a\n`,ii,a[ii]); 
         od;
       fi;

       # interpolate one more variable
       for i from 1 to n do

         l := []; ji := 0;
         for ii from 1 to t do;
           if i = 1 then 
             l  := [ degf ]; 
             ji := degf+1; 
           else 
             l  := [ op(l), min(degs[i-1],kgetdeg(Ji[ii],i-1))]; 
             ji := ji + min(degs[i-1],kgetdeg(Ji[ii],i-1))+1; 
           fi;
         od; 
         if (printlevel>2) then
           printf(`Interpolating %a (i=%d)\n`,vars[i],i);
           printf(`t=%a\n`,t); printf(`ji=%a\n`,ji);
           printf(`l=%a\n`,l); printf(`Ji=%a\n`,Ji);
           printf(`Pruneing\n`);
         fi;

         pans := kprune(l,Ji,vars,x);
         l := pans[1];
         Ji:= pans[2];
         incpoly := incpoly + pans[3];
         t  := nops(Ji);

         l := []; ji := 0;
         for ii from 1 to t do;
           if i = 1 then
             l  := [ degf ];
             ji := degf+1;
           else
             l  := [ op(l), min(degs[i-1],kgetdeg(Ji[ii],i-1))];
             ji := ji + min(degs[i-1],kgetdeg(Ji[ii],i-1))+1;
           fi;
         od;

         if (printlevel>2) then
           printf(`t=%a\n`,t); printf(`ji=%a\n`,ji); 
           printf(`l=%a\n`,l); printf(`Ji=%a\n`,Ji); 
           printf(`incpoly=%a\n`,incpoly);
         fi;

         printf(`Computed black box calls for kaltofen : %a\n`,ji); 
         if ( i = 1) then 
         printf(`Computed black box calls for zippel : %a\n`,degf+1);
         else
         printf(`Computed black box calls for zippel : %a\n`,t*(degs[i-1]+1));
         fi;

         state := kaltofencore(n,vars,i,a,d0,rnd,incpoly,ji,Ji,t,l,m);

         printf(`Actual number of black box calls %a\n`,state[4]);

         x       := state[1];
         Ji      := state[2];
         t       := state[3];
         bbcnt   := bbcnt + state[4];
         incpoly := state[5];
         nJi     := state[6];
         k       := state[7]; 

         if (printlevel>2) then
          printf(`x=%a\n`,x);
          printf(`Ji=%a\n`,Ji);
          printf(`t=%a\n`,t);
         fi;
       od;

   printf(`Algorithm       : Kaltofen's sparse conversion (mod p) 3\n`);
   printf(`Polynomial      : %a\n`,incpoly +
           subs('z'=1,kbuildpoly(x,nJi,k-1,vars,n,m)));
   printf(`Black Box calls : (kaltofen) %d\n`,bbcnt);
   printf(`Black Box calls : (zippel) \n`);

 RETURN(incpoly + subs('z'=1,kbuildpoly(x,nJi,k-1,vars,n,m))); 
end:

