kaltofencore := proc(n,vars,i,pnts,d0,rnd,incpoly,ji,Ji,t,l,m)
# n       number of variables
# vars    list of variables [x1,...,xn]
# i       current variable index
# pnts    initial values a2...an
# d0      bound on all of the degrees [deg(x1),...,deg(xn)]
# rnd     random number generator
# incpoly pruned monomials
# ji      cardinality of the expanded Ji set
# Ji      set of exponents e1,...,ei-1
# t       number of terms
# l       d_e1,...,ei-1 for each element of Ji
# all operations are in Z mod m
local  k,e,d,ii,# loop variables
       p,         # random points p1,...,pi
       tmpp,      # (p1)^k,...,(pi)^k,ai+1,...,an
       gki,       # blackbox evaluated at (p1)^k,...,(pi)^k,ai+1,...,an
       nJi,       # expanded set of exponents
       newJi, 
       newt,
       v,         # v1...v((d0+1)^n)*d0 describes a transposed vandermonde matrix
       x,         # x1...x((d0+1)^n)*d0 solution
       incval,    #
       bbcnt,     # number of black box calls
       est,
       state;     # modified interpolation state

       # initialize variables
       state := array(1..7);
       bbcnt := 0;
       p := array(1..n); tmpp := array(1..n);
       v := array(1..((d0+1)^n))*d0; # ((d0+1)^n)*d0 is an upper bound on ji
       x := array(1..((d0+1)^n))*d0;

       for ii from 2 to n do; tmpp[ii] := pnts[ii]; od;

       # select random points p1,...,pk
       for ii from 1 to i do p[ii] := rnd(); od;

       if (printlevel>2) then
        for ii from 1 to i do printf(`p[%d]=%a\n`,ii,p[ii]); od;
       fi;

       # compute gki
       for k from 1 to ji do
         for ii from 1 to i do;
          tmpp[ii] := modp( p[ii]^(k-1), m);
         od;
         incval := modp(
                   subs((vars[zz]=tmpp[zz]*tmpp[1]) $ (zz=1..n), incpoly),m);
         gki[k] := bbcall(bb,tmpp,n,m) - incval;
         bbcnt := bbcnt + 1;

         if (printlevel>2) then
           for ii from 1 to n do printf(`tmpp[%d]=%a\n`,ii,tmpp[ii]); od;
           printf(`black box call=%a\n`,bbcall(bb,tmpp,n,m));
           printf(`incremental poly=%a\n`,incval);
           printf(`gki[%d]=%a\n`,k,gki[k]);
         fi;
       od;

       if (printlevel>2) then
        for ii from 1 to i do printf(`p[%d]=%a\n`,ii,p[ii]); od;
       fi;

       # set up the ji by ji linear system
       k := 1;
       nJi :=[];
       # for every element in Ji (exponent vector)
       for e from 1 to t do

       if (printlevel>2) then
         printf(`working on exponent vector %a\n`,Ji[e]);
       fi;

       # for every degree
       for d from 0 to l[e] do

       if (printlevel>2) then printf(`d %d\n`,d); fi;

       v[k] := kraise(p,Ji[e],d,i,m);

       if (printlevel>2) then printf(`v[%d]=%a\n`,k,v[k]); fi;

       k := k + 1;
       if i = 1 then nJi := [ op(nJi), [d] ];
           else nJi := [ op(nJi), [ op(Ji[e]), d ] ]; fi;
         od;
       od;

       # solve vandermonde system
       x  := kvsolve(v,gki,ji,m);
       newJi := knextJi(x,nJi,k-1);
       newt  := nops(newJi);

       #if ( Ji <> 0 )then est := ComputeAccuracy(x,ji); else
       #est := 1; fi;

       state[1] := x;
       state[2] := newJi;
       state[3] := newt;
       state[4] := bbcnt;
       state[5] := incpoly;
       state[6] := nJi;
       state[7] := k;
 RETURN(state);
end:

ComputeAccuracy := proc(solutionVector,length)
local i,zeroCnt;

  if  nops(solutionVector) <> length  then
   ERROR(`size mismatch, vector <> length`);
  fi;
  zeroCnt := 0;
  for i from 1 to length do
    if solutionVector[i] = 0 then zeroCnt := zeroCnt + 1; fi;
  od;

 RETURN(evalf(zeroCnt/length*100));
end:

kraise := proc(p,e,d,i,m)
local j,prod;

   if i = 1 then
    prod := modp( p[1]^d, m);
   else
    prod := modp( p[1]^e[1], m);
    for j from 2 to i-1 do
     prod := modp( prod*(p[j]^e[j]), m);
    od;
    prod := modp( prod*p[i]^d, m);
   fi;

 RETURN(prod);
end:

knextJi := proc(x,nJi,k)
local i,Ji;

   Ji := [];
   for i from 1 to k do
     if x[i] <> 0 then
       Ji := [ op(Ji), nJi[i] ];
     fi;
   od;

 RETURN(Ji);
end:

kbuildpoly := proc(x,nJi,k,vars,n,modulus)
local i,j,poly,m;

   poly := 0;
   for i from 1 to k do
    if x[i] <> 0 then
      m := 1;
      for j from 1 to n do
       m := m*vars[j]^nJi[i][j];
      od;
      poly := modp( poly + x[i]*m, modulus);
    fi;
   od;

 RETURN(poly);
end:

kgetdeg := proc(exp,i)
local j,d;

   d := exp[1];
   for j from 2 to i do
     d := d - exp[j];
   od;

 RETURN(d);
end:

kprune :=proc(l,Ji,vars,x)
local i,n,xx,nn,
      p,nl,nJi,ans;

    nn := nops(x);
    n := nops(l);
    p := 0;
    ans := [];
    nl := []; nJi := [];


    xx := [];
    for i from 1 to nn do
      if x[i] <> 0 then
       xx := [ op(xx), x[i] ];
      fi;
    od;

    for i from 1 to n do
      if l[i] = 0 then
        p := p + xx[i]*kgetmon(Ji[i],vars);
      else
        nl  := [ op(nl), l[i] ];
        nJi := [ op(nJi), Ji[i] ];
      fi;
    od;
    ans := [ nl, nJi, p ];

 RETURN(ans);
end:

kgetmon := proc(exps,vars)
local i,n,monomial;

   n := nops(exps);
   monomial := 1;
   for i from 2 to n do
     monomial := monomial*vars[i]^exps[i];
   od;

 RETURN(monomial);
end:

