read `bbpolygen.mpl`:
read `vsolve.mpl`:
read `zippelcore.mpl`;

zippel2 := proc(bb,vars,degs,p,m)
# bb   black box polynomial
# vars list of variables  [x1,...,xn]
# degs corresponding list of degrees [deg(x1),...,deg(xn)]
# p    cardinality for random number generator
# all operations are in Z mod m
 local i,k,      # loop counters
       n,        # number of variables x1,...,xn
       rnd,      # random number generator
       d0,       # maximum degree found in degs
       t,        # current number of terms 
       exps,     # current list monomial exponents 
       bkj,      # k-1 tuple of randomly selected points a1,...,ak-1
       incpoly,  # incrementally built phase two polynomial  
       bbcnt,    # number of black box calls
       state;    # modified interpolation state

       # initialize variables
       state := array(1..4);
       bbcnt := 0;
       n := nops(vars);
       rnd := rand(1..p);
       d0 := degs[1];
       for i from 1 to n do
        if ( d0 < degs[i] ) then d0 := degs[i]; fi;
       od;
       t := 1;
       exps := [[]];

       # select initial values bkj[1],...,bkj[n]
       bkj := array(1..n); 
       for i from 1 to n do bkj[i] := rnd(); od;

       #interpolate one more variable
       for k from 1 to n do

         if (printlevel>2) then
          printf(`Interpolating variable %a\n`,vars[k]);
         fi;
      
        printf(`Computed number of black box calls %a\n`,t*(degs[k]+1));

        state := zippelcore(n,bkj,rnd,k,degs,d0,bb,t,exps,m);
   
        t       := state[1];
        exps    := state[2];
        incpoly := state[3]; 
        bbcnt   := bbcnt + state[4];

        printf(`Actual number of black box calls %a\n`,state[4]);
        if (printlevel>2) then
          printf(`t=%a\n`,t); printf(`incpoly=%a\n`,incpoly);
          printf(`exps=%a\n`,exps);
        fi;
  od;

  printf(`Algorithm       : zipple's sparse interpolation 2\n`);
  printf(`Polynomial      : %a\n`,convert(incpoly,`+`));
  printf(`Black Box calls : %d\n`,bbcnt);

  RETURN(convert(incpoly,`+`));
end:
