zippelcore := proc(n,pnts,rnd,k,degs,dmax,bb,t,exps,m)
# n     number of variables
# pnts  initial values 1...n
# rnd   random number generator
# k     variable
# degs  exact degrees [deg(x1),...,deg(xn)]
# dmax  maximum degree found in degs
# bb    bb   black box polynomial
# t     current number of terms
# exps  current list monomial exponents
# all operations are in Z mod m
local j,ii,i,   # loop variables
      bkj,      # k-1 tuple of randomly selected points a1,...,ak-1
      tmpbkj,   # temporary k-1 tuple (bkj[1])^i,...,(bkj[k-
      newt,     # adjusted number of terms
      newexps,  # adjusted list of monomial exponents
      v,        # v1...v(dmax+1)^n describes a transposed vandermonde matrix
      a,        # a1...a(dmax+1)^n evaluation points (Ax=a)
      x,        # x1...x((dmax+1)^n solution
      indep,    # independant values for univariate interpolation
      dep,      # dependant values for univariate interpolation
      cpoly,    # current list of monomial coefficients
      tmp,      # temporary univariate polynomial
      incpoly,  # incrementally built phase two polynomial
      bbcnt,    # actual number of black box calls
      state;    # modified interpolation state 

      state := array(1..4);
      cpoly := array(0..dmax);
      v := array(1..(dmax+1)^n); # (dmax+1)^n is an upper bound on t
      a := array(1..(dmax+1)^n);
      x := array(1..(dmax+1)^n);

      # put initial values in bkj
      for ii from 1 to n do
        bkj[ii] := pnts[ii];
        tmpbkj[ii] := pnts[ii];
      od;

      indep := [];
      bbcnt := 0;
      for j from 0 to degs[k] do # we have performed the first k-1 stages
        # select a random k-1 tuple bkj[1],...,bkj[k-1]
        for i from 1 to k-1 do bkj[i] := rnd(); od;

        # compute the v1...vt
        for i from 1 to t do v[i] := zraise(bkj,exps[i],k-1,m); od;

        # setect a random bkj[k]
        bkj[k] := rnd(); indep := [ op(indep), bkj[k] ];

        if (printlevel>2) then
          for i from 1 to n do printf(`bkj[%d]=%a\n`,i,bkj[i]); od;
           for i from 1 to t do printf(`v[%d]=%a\n`,i,v[i]); od;
        fi;

        # compute a1...at
        tmpbkj[k] := bkj[k];
        for i from 1 to t do;
          for ii from 1 to k-1 do tmpbkj[ii] := modp( (bkj[ii])^(i-1), m) od;
          a[i] := bb(tmpbkj,m); bbcnt := bbcnt + 1;

          if (printlevel>2) then
            for ii from 1 to n do printf(`tmpbkj[%d]=%a\n`,ii,tmpbkj[ii]);od;
            printf(`a[%d]= %a\n`,i,a[i]);
          fi;
        od;

         # solve transpose(vandermond(v))=x*a
         x := kvsolve(v,a,t,m);
         cpoly[j] := x;

         if (printlevel>2) then printf(`cpoly[%d]=%a\n`,j,eval(cpoly[j])); fi;
       od;

       # interpolate coefficients
       incpoly := 0;
       for j from 1 to t do
         dep := [];
         for i from 0 to degs[k] do dep := [ op(dep), cpoly[i][j] ]; od;
         tmp := modp( interp(indep,dep,vars[k],vars), m);
         if type(tmp,`+`) then
           tmp := [op(sort(tmp,vars))];
         else
           tmp := [tmp];
         fi;

         if (printlevel>2) then printf(`tmp=%a\n`,tmp); fi;

         incpoly := modp( sort(expand(
                    zbuildpoly(incpoly,tmp,vars,exps[j],k-1,m)),vars), m);
        od;

       if type(incpoly,`+`) then
         incpoly:= [op(sort(incpoly,vars))];
       else
         incpoly := [incpoly];
       fi;
       newt := nops(incpoly);
       newexps := zgetexps(incpoly,vars,k,newt);

       if (printlevel>2) then
         printf(`t=%a\n`,newt); printf(`incpoly=%a\n`,incpoly);
         printf(`exps=%a\n`,newexps);
       fi;

       state[1] := newt;
       state[2] := newexps;
       state[3] := incpoly;
       state[4] := bbcnt;
  RETURN(state);
end:

zraise := proc(y,e,k,m)
local i,prod;

   if k = 0 then
     prod := modp( y[1], m);
   else
     prod := modp( y[1]^e[1], m);
   fi;
   for i from 2 to k do
    prod := modp( prod*(y[i]^e[i]), m);
   od;

 RETURN(prod);
end:

zgetexps := proc(monlst,vars,k,T)
local i,
      exps;

   exps := [];
   for i from 1 to T do
    exps := [ op(exps), zgetexpsaux(monlst[i],vars,k) ];
   od;

 RETURN(exps);
end:

zgetexpsaux := proc(mon,vars,k)
local i,
      exp;

   exp := [];
   for i from 1 to k do
     exp := [ op(exp), degree(mon,vars[i]) ];
   od;

 RETURN(exp);
end:

zbuildpoly :=proc(incpoly,poly,vars,exps,k,m)
local bpoly,cpoly,i,ii;

   if (k = 0) then RETURN(convert(poly,`+`)); fi;
   bpoly := 1;
   for i from 1 to k do bpoly := modp( bpoly*vars[i]^exps[i], m); od;

   cpoly := 0;
   for i from 1 to nops(poly) do; cpoly := modp( cpoly + poly[i], m); od;
   bpoly := modp( incpoly + bpoly*cpoly, m);

 RETURN(bpoly);
end:

