read `bbpolygen.mpl`:
read `vsolve.mpl`:
read `kaltofenaux3.mpl`;

kaltofen3 := proc(bb,ivars,degs,degf,r,m)
# bb    black box polynomial
# ivars list of input variables  [x1,...,xn]
# degs  exact degrees [deg(x1),...,deg(xn)]
# r     cardinality for random number generator
# all operations are in Z mod m
local  n,      # number of variables x1,...,xn
       vars,   # expanded set of variables
       d0,     # bound on all of the degrees [deg(x1),...,deg(xn)]
       ii,e,d, # loop variables
       rnd,    # random number generator
       i,      # loop over the variables 1...n
       Ji,     # set of exponents e1,...,ei-1
       nJi,    # expanded set of exponents
       k,      # loop over the expanded Ji set 
       ji,     # cardinality of the expanded Ji set 
       a,      # initial random elements
       l,      # d_e1,...,ei-1 for each element of Ji 
       t,      # number of terms
       p,      # random points p1,...,pi
       tmpp,   # (p1)^k,...,(pi)^k,ai+1,...,an
       gki,    # blackbox evaluated at (p1)^k,...,(pi)^k,ai+1,...,an
       v,      # v1...v((d0+1)^n)*d0 describes a transposed vandermonde matrix
       x,      # x1...x((d0+1)^n)*d0 solution
       incpoly,# polynomial of completed monomials
       pans,   #
       incval, #
       bbcnt;  # number of black box calls

       # initialize variables
       bbcnt := 0; incpoly := 0; d0 := degs[1]; 
       n := nops(ivars); 
       d0 := degs[1]; 
       for i from 1 to n do
        if ( d0 < degs[i] ) then d0 := degs[i]; fi;
       od;
       n := n + 1;
       vars := array(1..n);
       vars[1] := `z`;
       for i from 2 to n do vars[i] := ivars[i-1] od; 

       rnd := rand(1..r);
 
       p := array(1..n); tmpp := array(1..n);
       v := array(1..((d0+1)^n))*d0; # ((d0+1)^n)*d0 is an upper bound on ji
       x := array(1..((d0+1)^n))*d0;

       t  := 1;
       Ji := [[]];
       l  := [];

       # select initial values a2,...,an
       a := array(2..n); 
       for i from 2 to n do a[i] := rnd(); tmpp[i] := a[i]; od;

       if (printlevel>2) then 
         for ii from 2 to n do 
          printf(`a[%d]=%a\n`,ii,a[ii]); printf(`tmpp[%d]=%a\n`,ii,tmpp[ii]); 
         od;
       fi;

       # interpolate one more variable
       for i from 1 to n do

         l := []; ji := 0;
         for ii from 1 to t do;
           if i = 1 then 
             l  := [ degf ]; 
             ji := degf+1; 
           else 
             l  := [ op(l), min(degs[i-1],getdeg(Ji[ii],i-1))]; 
             ji := ji + min(degs[i-1],getdeg(Ji[ii],i-1))+1; 
           fi;
         od; 
         if (printlevel>2) then
           printf(`Interpolating %a (i=%d)\n`,vars[i],i);
           printf(`t=%a\n`,t); printf(`ji=%a\n`,ji);
           printf(`l=%a\n`,l); printf(`Ji=%a\n`,Ji);
           printf(`Pruneing\n`);
         fi;

         pans := prune(l,Ji,vars,x);
         l := pans[1];
         Ji:= pans[2];
         incpoly := incpoly + pans[3];
         t  := nops(Ji);

         l := []; ji := 0;
         for ii from 1 to t do;
           if i = 1 then
             l  := [ degf ];
             ji := degf+1;
           else
             l  := [ op(l), min(degs[i-1],getdeg(Ji[ii],i-1))];
             ji := ji + min(degs[i-1],getdeg(Ji[ii],i-1))+1;
           fi;
         od;

         if (printlevel>2) then
           printf(`t=%a\n`,t); printf(`ji=%a\n`,ji); 
           printf(`l=%a\n`,l); printf(`Ji=%a\n`,Ji); 
           printf(`incpoly=%a\n`,incpoly);
         fi;
         printf(`At Variable %a Pruned %a\n`,vars[i],pans[3]);
 
         # select random points p1,...,pk
         for ii from 1 to i do
          p[ii] := rnd();                    
         od;

         if (printlevel>2) then
          for ii from 1 to i do printf(`p[%d]=%a\n`,ii,p[ii]); od;
         fi;
 
         # compute gki
         for k from 1 to ji do
           for ii from 1 to i do;
            tmpp[ii] := modp( p[ii]^(k-1), m);
           od;
           incval := modp( 
                     subs((vars[zz]=tmpp[zz]*tmpp[1]) $ (zz=1..n), incpoly),m);
           gki[k] := bbcall(bb,tmpp,n,m) - incval;
           bbcnt := bbcnt + 1;

           if (printlevel>2) then
             for ii from 1 to n do printf(`tmpp[%d]=%a\n`,ii,tmpp[ii]); od;
             printf(`black box call=%a\n`,bbcall(bb,tmpp,n,m));
             printf(`incremental poly=%a\n`,incval);
             printf(`gki[%d]=%a\n`,k,gki[k]);
           fi;
         od; 
       
         if (printlevel>2) then
          for ii from 1 to i do printf(`p[%d]=%a\n`,ii,p[ii]); od;
         fi;

         # set up the ji by ji linear system
         k := 1;
         nJi :=[];
         # for every element in Ji (exponent vector)
         for e from 1 to t do 

           if (printlevel>2) then
               printf(`working on exponent vector %a\n`,Ji[e]);
           fi;

           # for every degree
           for d from 0 to l[e] do
 
             if (printlevel>2) then printf(`d %d\n`,d); fi;

             v[k] := raise(p,Ji[e],d,i,m);

             if (printlevel>2) then printf(`v[%d]=%a\n`,k,v[k]); fi;

             k := k + 1;
             if i = 1 then nJi := [ op(nJi), [d] ];
             else nJi := [ op(nJi), [ op(Ji[e]), d ] ]; fi;
           od;
         od;
        
         # solve vandermonde system 
         x  := kvsolve(v,gki,ji,m);
         Ji := nextJi(x,nJi,k-1);
         t  := nops(Ji);

         if (printlevel>2) then
          printf(`x=%a\n`,x);
          printf(`Ji=%a\n`,Ji);
          printf(`t=%a\n`,t);
         fi;
       od;

   printf(`Algorithm       : Kaltofen's sparse conversion (mod p) 3\n`);
   printf(`Polynomial      : %a\n`,incpoly +
           subs('z'=1,buildpoly(x,nJi,k-1,vars,n,m)));
   printf(`Black Box calls : %d\n`,bbcnt);

 RETURN(incpoly + subs('z'=1,buildpoly(x,nJi,k-1,vars,n,m))); 
end:

