zvsolve := proc(v,a,n,p)
# Zipple's solution to solving a transposed vandermonde system 
# Ax=a , B = transpose(a), x = transpose(inverse(B))*a
# v v1...vn describes a transposed vandermonde matrix A
# a a1...a solution vector a	
# n matrix and vector dimension
# all operations are in Z mod p
local j,k,i, # loop variable
      B,
      tx,x;  # solution

    B := array(1..n);
    x := [];
    # compute Bj(z) 
    for j from 1 to n do
      B[j] := 1;
      for k from 1 to n do
        if  not( j = k ) then 
          B[j] := modp( B[j]*(('z' - v[k])/(v[j] - v[k])) , p ); 
        fi;
      od; 
    od;

    # compute x
    for j from 1 to n do
      tx := modp( coeff(collect(B[j],'z'),'z',0)*a[1] , p );
      for i from 2 to n do
        tx := modp( tx + coeff(collect(B[j],'z'),'z',i-1)*a[i] , p );
      od;
      x := [op(x),tx];
    od;
 
  RETURN(eval(x));
end:

kvsolve := proc(v,a,n,p)
# Kaltofen's solution to solving a transposed vandermonde system
# Ax=a , x = inverse(A)*a
# v v1...vn describes a transposed vandermonde matrix A
# a a1...a solution vector a
# n matrix and vector dimension
# all operations are in Z mod p
local j,k,i, # loop variable
      B,D,
      BP,
      BD,
      QN, 
      x;     # solution

      B := modp( (`z` - v[1]), p );
      for i from 2 to n do
        B := modp( B*(`z` - v[i]), p );
      od;
      B := modp( sort(expand(B),`z`) , p);
      BP := modp( sort(expand(diff(B,`z`)),`z`), p);

      D := `z`^n*a[1]; 
      for i from 2 to n do
        D :=D+`z`^(n-i+1)*a[i];
      od;
      BD := modp( sort(expand(B*D),`z`), p );

      QN := coeff(BD,`z`,n+1);
      for i from n+2 to 2*n do
        QN := modp( QN + `w`^(i-n-1)*coeff(BD,`z`,i), p);
      od; 

      x := [];
      for i from 1 to n do;
        x := [ op(x) , modp( subs(w=v[i],QN)/subs(z=v[i],BP), p) ];
      od;
 
  RETURN(eval(x));
end:
