read `bbpolygen.mpl`:
read `vsolve.mpl`:
read `zippelaux.mpl`;

zippel1 := proc(bb,vars,degs,p,m)
# bb   black box polynomial
# vars list of variables  [x1,...,xn]
# degs corresponding list of degrees [deg(x1),...,deg(xn)]
# p    cardinality for random number generator
# all operations are in Z mod m
 local i,j,k,ii, # loop counters
       n,        # number of variables x1,...,xn
       rnd,      # random number generator
       dmax,     # maximum degree found in degs
       indep,    # independant values for univariate interpolation
       dep,      # dependant values for univariate interpolation
       cpoly,    # current list of monomial coefficients
       t,        # current number of terms 
       exps,     # current list monomial exponents 
       bkj,      # k-1 tuple of randomly selected points a1,...,ak-1
       tmpbkj,   # temporary k-1 tuple (bkj[1])^i,...,(bkj[k-1])^i i=0...t-1
       v,        # v1...v(dmax+1)^n describes a transposed vandermonde matrix 
       a,        # a1...a(dmax+1)^n evaluation points (Ax=a)
       x,        # x1...x((dmax+1)^n solution
       tmp,      # temporary univariate polynomial
       incpoly,  # incrementally built phase two polynomial  
       bbcnt;    # number of black box calls

       if ( p>m ) then
         ERROR(`cardinality of random number set > modulus`);
       fi;

       # initialize variables
       bbcnt := 0;
       n := nops(vars);
       rnd := rand(1..p);
       dmax := degs[1];
       for i from 1 to n do
        if ( dmax < degs[i] ) then dmax := degs[i]; fi;
       od;
       t := 1;
       exps := [[]];

       cpoly := array(0..dmax);
       v := array(1..(dmax+1)^n); # (dmax+1)^n is an upper bound on t
       a := array(1..(dmax+1)^n);
       x := array(1..(dmax+1)^n); 

       # select initial values bkj[1],...,bkj[n]
       bkj := array(1..n); tmpbkj := array(1..n);
       for i from 1 to n do bkj[i] := rnd(); tmpbkj[i] := bkj[i]; od;  


       #interpolate one more variable
       for k from 1 to n do

         if (printlevel>2) then
          printf(`Interpolating variable %a\n`,vars[k]);
         fi;
 
         indep := [];
         for j from 0 to dmax do # we have performed the first k-1 stages
           # select a random k-1 tuple bkj[1],...,bkj[k-1]
           for i from 1 to k-1 do bkj[i] := rnd(); od;

           # compute the v1...vt 
           for i from 1 to t do v[i] := raise(bkj,exps[i],k-1,m); od;

           # setect a random bkj[k]
           bkj[k] := rnd(); indep := [ op(indep), bkj[k] ];

           if (printlevel>2) then
             for i from 1 to n do printf(`bkj[%d]=%a\n`,i,bkj[i]); od;
             for i from 1 to t do printf(`v[%d]=%a\n`,i,v[i]); od;
           fi;

         # compute a1...at
         tmpbkj[k] := bkj[k];
         for i from 1 to t do;
           for ii from 1 to k-1 do tmpbkj[ii] := modp( (bkj[ii])^(i-1), m) od;
           a[i] := bb(tmpbkj,m); bbcnt := bbcnt + 1;

           if (printlevel>2) then 
             for ii from 1 to n do printf(`tmpbkj[%d]=%a\n`,ii,tmpbkj[ii]);od;
             printf(`a[%d]= %a\n`,i,a[i]);
           fi;
         od;

         # solve transpose(vandermond(v))=x*a
         x := kvsolve(v,a,t,m);
         cpoly[j] := x; 

         if (printlevel>2) then printf(`cpoly[%d]=%a\n`,j,eval(cpoly[j])); fi;
       od;

       # interpolate coefficients
       incpoly := 0;
       for j from 1 to t do
         dep := []; 
         for i from 0 to dmax do dep := [ op(dep), cpoly[i][j] ]; od;    
         tmp := modp( interp(indep,dep,vars[k],vars), m);
         if type(tmp,`+`) then 
           tmp := [op(sort(tmp,vars))]; 
         else 
           tmp := [tmp]; 
         fi;

         if (printlevel>2) then printf(`tmp=%a\n`,tmp); fi;

         incpoly := modp( sort(expand(
                    buildpoly(incpoly,tmp,vars,exps[j],k-1,m)),vars), m);
        od;

        if type(incpoly,`+`) then 
          incpoly:= [op(sort(incpoly,vars))];
        else 
          incpoly := [incpoly]; 
        fi;
        t := nops(incpoly);
        exps := getexps(incpoly,vars,k,t);
     
        if (printlevel>2) then
          printf(`t=%a\n`,t); printf(`incpoly=%a\n`,incpoly);
          printf(`exps=%a\n`,exps);
        fi;
  od;

  printf(`Algorithm       : zipple's sparse interpolation 1\n`);
  printf(`Polynomial      : %a\n`,convert(incpoly,`+`));
  printf(`Black Box calls : %d\n`,bbcnt);

  RETURN(convert(incpoly,`+`));
end:
