zvsolve := proc(v,a,n)
# Zipple's solution to solving a transposed vandermonde system 
# Ax=a , B = transpose(a), x = transpose(inverse(B))*a
# v v1...vn describes a transposed vandermonde matrix A
# a a1...a solution vector a	
# n matrix and vector dimension
local j,k,i, # loop variable
      B,
      tx,x;  # solution

    B := array(1..n);
    x := [];
    # compute Bj(z) 
    for j from 1 to n do
      B[j] := 1;
      for k from 1 to n do
        if  not( j = k ) then 
          B[j] := B[j]*(('z' - v[k])/(v[j] - v[k])); 
        fi;
      od; 
    od;

    # compute x
    for j from 1 to n do
      tx := coeff(collect(B[j],'z'),'z',0)*a[1];
      for i from 2 to n do
        tx := tx + coeff(collect(B[j],'z'),'z',i-1)*a[i];
      od;
      x := [op(x),tx];
    od;
 
  RETURN(eval(x));
end:

kvsolve := proc(v,a,n)
# Kaltofen's solution to solving a transposed vandermonde system
# Ax=a , x = inverse(A)*a
# v v1...vn describes a transposed vandermonde matrix A
# a a1...a solution vector a
# n matrix and vector dimension
local j,k,i, # loop variable
      B,D,
      BP,
      BD,
      QN, 
      x;     # solution

      B := (`z` - v[1]);
      for i from 2 to n do
        B := B*(`z` - v[i]);
      od;
      B := sort(expand(B),`z`);
      BP := sort(expand(diff(B,`z`)),`z`);

      D := `z`^n*a[1]; 
      for i from 2 to n do
        D :=D+`z`^(n-i+1)*a[i];
      od;
      BD := sort(expand(B*D),`z`);

      QN := coeff(BD,`z`,n+1);
      for i from n+2 to 2*n do
        QN := QN + `w`^(i-n-1)*coeff(BD,`z`,i);
      od; 

      x := [];
      for i from 1 to n do;
        x := [ op(x) , subs(w=v[i],QN)/subs(z=v[i],BP) ];
      od;
 
  RETURN(eval(x));
end:
