read `bbpolygen.mpl`:
read `vsolve.mpl`:
read `kaltofenaux1.mpl`;

kaltofen1 := proc(bb,vars,degb,degf,r)
# bb   black box polynomial
# vars list of variables  [x1,...,xn]
# degb bound on all of the degrees [deg(x1),...,deg(xn)]
# degf upper bound on the total degree of f
# r    cardinality for random number generator
local  n,     # number of variables x1,...,xn
       ii,e,d,# loop variables
       rnd,   # random number generator
       d0,    # bound on all of the degrees [deg(x1),...,deg(xn)]
       i,     # loop over the variables 1...n
       Ji,    # set of exponents e1,...,ei-1
       nJi,   # expanded set of exponents
       k,     # loop over the expanded Ji set 
       ji,    # cardinality of the expanded Ji set 
       a,     # initial random elements
       l,     # d_e1,...,ei-1 for each element of Ji 
       t,     # number of terms
       p,     # random points p1,...,pi
       tmpp,  # (p1)^k,...,(pi)^k,ai+1,...,an
       gki,   # blackbox evaluated at (p1)^k,...,(pi)^k,ai+1,...,an
       v,     # v1...v((d0+1)^n)*d0 describes a transposed vandermonde matrix
       x,     # x1...x((d0+1)^n)*d0 solution
       bbcnt; # number of black box call

       # initialize variables
       bbcnt := 0;
       n := nops(vars);
       rnd := rand(1..r);
       d0 := min(degb,degf);

       p := array(1..n); tmpp := array(1..n);
       v := array(1..((d0+1)^n))*d0; # ((d0+1)^n)*d0 is an upper bound on ji
       x := array(1..((d0+1)^n))*d0;

       t  := 1;
       Ji := [[]];
       l  := [];

       # select initial values a2,...,an
       a := array(2..n); 
       for i from 2 to n do a[i] := rnd(); tmpp[i] := a[i]; od;

       if (printlevel>2) then 
         for ii from 2 to n do 
          printf(`a[%d]=%a\n`,ii,a[ii]); printf(`tmpp[%d]=%a\n`,ii,tmpp[ii]); 
         od;
       fi;

       # interpolate one more variable
       for i from 1 to n do

         l := []; ji := 0;
         for ii from 1 to t do;
           if i = 1 then 
             l  := [ d0 ]; 
             ji := d0+1; 
           else 
             l  := [ op(l), min(d0,degf-expdeg(Ji[ii],i-1)) ]; 
             ji := ji + min(d0,degf-expdeg(Ji[ii],i-1)) + 1;
           fi;
         od; 

         if (printlevel>2) then
           printf(`Interpolating %a (i=%d)\n`,vars[i],i);
           printf(`t=%a\n`,t); printf(`ji=%a\n`,ji);
           printf(`l=%a\n`,l); printf(`Ji=%a\n`,Ji);
         fi;

         # select random points p1,...,pk
         for ii from 1 to i do
          p[ii] := rnd();                    
         od;

         if (printlevel>2) then
          for ii from 1 to i do printf(`p[%d]=%a\n`,ii,p[ii]); od;
         fi;
 
         # compute gki
         for k from 1 to ji do
           for ii from 1 to i do;
            tmpp[ii] := p[ii]^(k-1);
           od;
           gki[k] := bb(tmpp); bbcnt := bbcnt + 1;      

           if (printlevel>2) then
             for ii from 1 to n do printf(`tmpp[%d]=%a\n`,ii,tmpp[ii]); od;
             printf(`gki[%d]=%a\n`,k,gki[k]);
           fi;
         od; 
       
         if (printlevel>2) then
          for ii from 1 to i do printf(`p[%d]=%a\n`,ii,p[ii]); od;
         fi;

         # set up the ji by ji linear system
         k := 1;
         nJi :=[];
         # for every element in Ji (exponent vector)
         for e from 1 to t do 

           if (printlevel>2) then
               printf(`working on exponent vector %a\n`,Ji[e]);
           fi;

           # for every degree
           for d from 0 to l[e] do
 
             if (printlevel>2) then printf(`d %d\n`,d); fi;

             v[k] := raise(p,Ji[e],d,i);

             if (printlevel>2) then printf(`v[%d]=%a\n`,k,v[k]); fi;

             k := k + 1;
             if i = 1 then nJi := [ op(nJi), [d] ];
             else nJi := [ op(nJi), [ op(Ji[e]), d ] ]; fi;
           od;
         od;
        
         # solve vandermonde system 
         x  := kvsolve(v,gki,ji);
         Ji := nextJi(x,nJi,k-1);
         t  := nops(Ji);

         if (printlevel>2) then
          printf(`x=%a\n`,x);
          printf(`Ji=%a\n`,Ji);
          printf(`t=%a\n`,t);
         fi;
       od;

   printf(`Algorithm       : Kaltofen's sparse conversion 1\n`);
   printf(`Polynomial      : %a\n`,buildpoly(x,nJi,k-1,vars,n));
   printf(`Black Box calls : %d\n`,bbcnt);

 RETURN(buildpoly(x,nJi,k-1,vars,n)); 
end:

