#-----------------------------------------------------------------------
# module: bbpoly.mpl
#
# author: Angel Diaz
# date: 6/15/96
#
# (c) Copyright 1996 Angel Diaz and Eirch Kaltofen, All Rights Reserved 
# This code is provided WITHOUT WARRANTY either expressed or implied.
#-----------------------------------------------------------------------

readlib(procmake):    # routines for creating Maple procedures

#-----------------------------------------------------------------------
# function: bbpoly_mod
# date: 6/15/96
#
# reference : This procedure is similar to a procedure described in
#    A. Diaz and E. Kaltofen, "On computing greatest common divisors 
#    with polynomials given by black boxes for their evaluation", Proc. 
#    1995 Internat. Symp. Symbolic Algebraic Comput. ISSAC '95, ACM Press, 
#    A. H. M. Levelt ed. New York, N. Y., pg. 232--239
#
# return: A Maple procedure which evaluates the input polynomial modulo m
#
# example: bb := bbpoly_mod(x+y,[x,y]); bb([1,2],13);
#
# assumptions: the arguments and variables pntsnf_i,modulusnf_i,
#    polynf,inf,numvarsnf,varlistnf can not be defined globally.
#-----------------------------------------------------------------------

bbpoly_mod := proc(
   poly_i,    # a maple polynomial
   varlist_i  # a list of the variables in poly_i
)
   local blackbox,      # neutral form of procedure (black box of poly_i)
         numvars;       # length of the list of indeterminants

   numvars  := nops(varlist_i);
   blackbox := `&proc`([pntsnf_i, modulusnf_i], 
                       [polynf,inf,numvarsnf,varlistnf],[],
                       `&statseq`(
                          `&:=`(`&local`[1], poly_i),
                          `&:=`(`&local`[3], numvars),
                          `&:=`(`&local`[4], varlist_i),
                          `&:=`(`&local`[1],
                                  'Eval(`&local`[1],{seq(`&local`[4]
                                  [`&local`[2]]=`&args`[1][`&local`[2]], 
                                  `&local`[2]=1..`&local`[3])}) 
                                  mod `&args`[2]'),
                          `&RETURN`(`&local`[1])
                       )
                     );
   RETURN(procmake(blackbox));
end:

